#!/bin/sh
# UpdateScript for Counter Gen2 with SD card only!

export PATH=/sbin:/usr/sbin:/bin:/usr/bin


# Definitions:
UBOOT_VAR_RFS_ID=rfsID_SD
UBOOT_VAR_KERNEL_IMAGE_NO=kernel_image_no
MD5SUM_FILE=md5sums.txt
KERNEL_IMAGE=uImage.zynq
DTB_IMAGE=zynq-counter2.dtb
RFS_TARBALL=rfs.tgz
VERSION_CURRENT=`cat /version.txt`
VERSION_NEW=`cat version.txt 2>/dev/null`
UPDATE_LOGFILE=/var/log/update.log
UPDATE_HOUR_METER_LOG=/var/log/update-hour-meter.log
POSTUPDATESCRIPT_ERROROFFSET=80

# RFS information:
RFS1_ID=/dev/mmcblk0p2
RFS2_ID=/dev/mmcblk0p3

RunPrePostScripts()
{
    script=$1
    if [ -f ${script} ] ; then
        echo "$(date): Executing script ${script} ..." >>${UPDATE_LOGFILE}
        /bin/sh ${script}
        RET=$?
        if [ "0" -ne "${RET}" ] ; then
            echo "$(date): script ${script} returned FAILURE #${RET}" >>${UPDATE_LOGFILE}
            exit `expr ${RET} + ${POSTUPDATESCRIPT_ERROROFFSET}`
        fi
        echo "$(date): script ${script} executed successfully" >>${UPDATE_LOGFILE}
    fi
}

SCRIPT_NAME=$(basename $0)

if [ ! -f ${MD5SUM_FILE} ] ; then
    exit 1
fi

md5sum -c ${MD5SUM_FILE} >/dev/null 2>&1
if [ "0" -ne "$?" ] ; then
    # Checksum mismatch
    exit 1
fi


# Firmware update compatibility check. Abort if not or an error occurred.
COMP_CHECK_SCRIPT=FirmwareUpdateCompatibilityCheck.sh
if [ -f ${COMP_CHECK_SCRIPT} ] ; then
    /bin/sh ${COMP_CHECK_SCRIPT}
    RET=$?
    if [ "0" -ne "${RET}" ] ; then
        LOG_STR="The firmware update compatibility check failed (error code: ${RET})."
        logger -p error -t ${SCRIPT_NAME} "${LOG_STR}"
        echo "$(date): ${SCRIPT_NAME}: Error: ${LOG_STR}" >>${UPDATE_LOGFILE}
        exit 14
    fi
else
    # script not found, log and exit
    LOG_STR="The firmware update compatibility check script not found (${COMP_CHECK_SCRIPT})."
    logger -p error -t ${SCRIPT_NAME} "${LOG_STR}"
    echo "$(date): ${SCRIPT_NAME}: Error: ${LOG_STR}" >>${UPDATE_LOGFILE}
    exit 14
fi

if [ ! -f ${DTB_IMAGE} ] ; then
	CURRENTMODEL=`grep "Model=" /hwconfig/hwconfig.txt | sed -e 's/.*=\(..\).*/\1/'`
	if [ ${CURRENTMODEL} == "EH" ] || [ ${CURRENTMODEL} == "GH" ] ; then
		FILENAME=zynq-counter2-EH300_SD.dtb
	elif [ ${CURRENTMODEL} == "HH" ] || [ ${CURRENTMODEL} == "IH" ] || [ ${CURRENTMODEL} == "JH" ] ; then
		FILENAME=zynq-counter2-HH100.dtb
	fi
	test ! -z ${FILENAME} || exit 2 # device not compatible
	test -f ${FILENAME} || exit 2 # missing file
	mv ${FILENAME} ${DTB_IMAGE}
fi

# Commitment: We always flash all three files. Why? Because it is likely that the kernel or dtb
# will change some time. From this time on we need to flash all files anyway.
test -f ${KERNEL_IMAGE} || exit 2
test -f ${DTB_IMAGE}    || exit 2
test -f ${RFS_TARBALL}  || exit 2

LOGSTR="$(date): Performing a FullUpdate from version ${VERSION_CURRENT} to version ${VERSION_NEW} ..."
echo ${LOGSTR} >>${UPDATE_LOGFILE}
logger ${LOGSTR}

# Determine IDs to be written:
RFS_ID=`fw_printenv ${UBOOT_VAR_RFS_ID} |sed -e's/.*=//' 2>/dev/null`
if [ "x${RFS_ID}" = "x${RFS1_ID}" ] ; then
    NEW_RFS_ID=${RFS2_ID}
else
    # RFS_ID is RFS2_ID or unset or set to unknown value, set it to No. 1:
    NEW_RFS_ID=${RFS1_ID}
fi
KERNEL_IMAGE_NO=`fw_printenv ${UBOOT_VAR_KERNEL_IMAGE_NO} | sed -e's/.*=//' 2>/dev/null`
if [ "x${KERNEL_IMAGE_NO}" = "xI1" ] ; then
    NEW_KERNEL_IMAGE_NO=I2
else
    NEW_KERNEL_IMAGE_NO=I1
fi

mount -oremount,rw /boot

# Just in case /mnt is already used, we umount it:
umount /mnt 2>/dev/null

# Write RFS tarball:
mkfs.ext4 -F -q -E stripe-width=2048 -b 4096 ${NEW_RFS_ID} >/dev/null 2>&1 || exit 6
mount -t ext4 ${NEW_RFS_ID} /mnt || exit 7
tar xzf ${RFS_TARBALL} -C /mnt || exit 7
umount /mnt || exit 7

# Write kernel image:
cp -f ${KERNEL_IMAGE} /boot/uImage.${NEW_KERNEL_IMAGE_NO}.zynq || exit 5
cp -f ${DTB_IMAGE}    /boot/zynq-counter2.${NEW_KERNEL_IMAGE_NO}.dtb || exit 5

### Possible pre update scripts (scripts starting with non-capital p):
PRE_SCRIPTS=`ls p*sh 2>/dev/null`
for script in ${PRE_SCRIPTS} ; do
    RunPrePostScripts $script
done


TMPFILE=`mktemp`

# Set U-Boot environment variables for booting new images:
echo ${UBOOT_VAR_RFS_ID} ${NEW_RFS_ID}        >>${TMPFILE}
echo ${UBOOT_VAR_KERNEL_IMAGE_NO} ${NEW_KERNEL_IMAGE_NO}   >>${TMPFILE}
# To ensure we start from SD card next time:
echo "bootcmd run bc_sd"                      >>${TMPFILE}

# To force an as rapid as possible modification of the U-Boot environment (i.e. file
# /boot/uboot.env) we previously sync. So the new file operation will be "as atomar
# as possible".:
sync
fw_setenv -s ${TMPFILE} >/dev/null || exit 8

if [ -f BOOT.BIN ] ; then
    cp -f BOOT.BIN /boot/BOOT.BIN
fi

rm ${TMPFILE}


echo "$(date): Successfully updated kernel and rfs to version ${VERSION_NEW}" >>${UPDATE_LOGFILE}


# Patch access rights
# On some devices the access rights of the directory /var/Counter/DeviceSpecific are not set 
# correctly by the production software. 
# This will cause an error, if another user than root will connect to the device via ssh and 
# needs to read the passwd file. E.g. for creating keys with ssh-keygen or calling whoami would 
# not work in this case.
chmod 755 /var/Counter/DeviceSpecific
sync
mount -oremount,ro /var/Counter
mount -oremount,rw /var/Counter


### Possible post update scripts (scripts starting with capital P):
POST_SCRIPTS=`ls P*sh 2>/dev/null`
for script in ${POST_SCRIPTS} ; do
    RunPrePostScripts $script
done

sync
mount -oremount,ro /boot

# reset update hour meter
# TODO: check whether this is still wanted:
/bin/rm -f ${UPDATE_HOUR_METER_LOG} >/dev/null 2>&1 
/bin/echo "0" > ${UPDATE_HOUR_METER_LOG}

exit 0
