#!/bin/sh

source /etc/init.d/rc1.d/_HelperFunctions/XmlParameter
. snapshot_creation_fn.sh

############################################################
#
# Script collects some data
#  - files (syslog, counter, )
#  - system information (about time, network, processes, disk space, ...)
#  - parameter set data
# and creates a compressed tar archive file:
#    "/tmp/DiagnosisData.tar.gz"
# It could be downloaded by HMI client.
#
############################################################


##############################
# Helper functions
##############################
fnEchoErr() {
    echo "$@" 1>&2;
}

fnCleanup() {
    rm -rf ${TAR_TARGET_DIR} >/dev/null 2>&1
}


##############################
# Defs
##############################
# tar file
TAR_TARGET_DIR="/tmp/tmp_diagnosis_data/"
TAR_FILE="/tmp/DiagnosisData.tar.gz"

# sub directory for snapshots
SNAPSHOT_SUB_DIR=snapshots

# special files to include in tar
ORG_FILE_VERSION_PATH=/version.txt
ADD_FILE_VERSION_PATH=${TAR_TARGET_DIR}version.txt
ORG_FILE_COMPATIBILITY_LIST=/etc/Counter/update_compatibility_whitelist.txt
ADD_FILE_COMPATIBILITY_LIST=${TAR_TARGET_DIR}update_file_compatibility_data.txt

# privacy mode flag
FILE_PRIVACY_MODE=/tmp/privacy-mode

##############################
# Initial Clean-up
##############################
# Check if tar file exist, if exist delete it.
rm -f ${TAR_FILE} >/dev/null 2>&1
fnCleanup


##############################
# Create data for export
##############################
# Create tmp directory
mkdir -p ${TAR_TARGET_DIR} >/dev/null 2>&1
RES_TMP=$?
if [ ${RES_TMP} -ne 0 ] ; then
    logger -p user.error "$0: could not create tmp directory: '${TAR_TARGET_DIR}'"
    fnEchoErr "could not create tmp directory: '${TAR_TARGET_DIR}'"
    fnCleanup
    exit 1
fi
# Create and copy system data
date > ${TAR_TARGET_DIR}date.log
hwclock > ${TAR_TARGET_DIR}hwclock.log
cp -f -p ${ORG_FILE_VERSION_PATH} ${ADD_FILE_VERSION_PATH} >/dev/null 2>&1
cp -f -p ${ORG_FILE_COMPATIBILITY_LIST} ${ADD_FILE_COMPATIBILITY_LIST} >/dev/null 2>&1
cp -f -p /var/log/*.cnt ${TAR_TARGET_DIR}
cp -f -p /var/log/*.log ${TAR_TARGET_DIR}
cp -f -p /var/log/messages* ${TAR_TARGET_DIR}
cp -f -p /tmp/boot_type ${TAR_TARGET_DIR}
cp -f -p /tmp/ws.log* ${TAR_TARGET_DIR}
/bin/sh /sbin/ExportParamSet.sh
mv /tmp/ExportParamSet.tar.gz ${TAR_TARGET_DIR}
free > ${TAR_TARGET_DIR}free.log
mpstat -P ALL > ${TAR_TARGET_DIR}mpstat.log
top -n 1 > ${TAR_TARGET_DIR}top.log
ps -T > ${TAR_TARGET_DIR}ps.log
dmesg > ${TAR_TARGET_DIR}dmesg.log
ifconfig > ${TAR_TARGET_DIR}ifconfig.log
route -e > ${TAR_TARGET_DIR}route.log
hostname > ${TAR_TARGET_DIR}hostname.log
df -a -T > ${TAR_TARGET_DIR}df.log
fw_printenv > ${TAR_TARGET_DIR}fw_printenv.log
uname -a > ${TAR_TARGET_DIR}uname.log
cat /proc/cmdline > ${TAR_TARGET_DIR}cmdline.log
/sbin/get_temp_val.sh > ${TAR_TARGET_DIR}cur_temp.log
ls -l -R /var/Counter/ > ${TAR_TARGET_DIR}ls_varCounter.log
cp -f -p -R /hwconfig ${TAR_TARGET_DIR}

# get SDCard Info
ROOTDIR=/sys/devices/soc0/amba/e0100000.sdhci/mmc_host/mmc0
if [ -d $ROOTDIR ]; then
    SUBDIR=`ls ${ROOTDIR} | grep mmc0:`
    SDNAMEPATH="${ROOTDIR}/${SUBDIR}/name"
    SDDATEPATH="${ROOTDIR}/${SUBDIR}/date"
    touch ${TAR_TARGET_DIR}mmc-info.txt
    if [ -e $SDNAMEPATH ] ; then
        cat ${SDNAMEPATH} >> ${TAR_TARGET_DIR}mmc-info.txt
    fi
    if [ -e $SDDATEPATH ] ; then
        cat ${SDDATEPATH} >> ${TAR_TARGET_DIR}mmc-info.txt
    fi
fi

# get video snapshots
if [ -f ${FILE_PRIVACY_MODE} ]; then
    logger -p user.notice "$0: Video snapshots supressed by privacy flag"
else
    # video snapshots
    fnRecordVideoSnapshots ${TAR_TARGET_DIR}/${SNAPSHOT_SUB_DIR}
    RES_TMP=$?
    if [ ${RES_TMP} -ne 0 ] ; then
        logger -p user.error "$0: failed to obtain video snapshots"
        fnEchoErr "failed to obtain video snapshots"
        fnCleanup
        exit 3
    fi

    # get stitched image snapshot when device is master in multisensor fusion mode
    MSF_MODE=`GetParamValue aps_master_slave_params MasterSlaveMode`
    if [ ${MSF_MODE} -eq 1 ]
    then
        CreateStitchedImgSnapshot.sh ${TAR_TARGET_DIR}/${SNAPSHOT_SUB_DIR}/snapshot_stitched_img.png
    fi
fi


# TODO more files ?
#      - production data ?
#      - stereo camera calibration ?
#      - webserver user db ?
#      - licenses ?
#      - default configs data ?
#      - OSD ?
#      - ls from /CustomData ? other ?
# TODO other data ?


##############################
# tar (compressed with gzip) copied data for export
##############################
tar -c -z -f ${TAR_FILE} -C ${TAR_TARGET_DIR} . >/dev/null 2>&1
RES_TAR=$?
if [ ${RES_TAR} -ne 0 ] ; then
    logger -p user.error "$0: tar returned an error: ${RES_TAR}"
    fnEchoErr "tar command returned an error: ${RES_TAR}"
    rm -f ${TAR_FILE} >/dev/null 2>&1
    fnCleanup
    exit 2
fi


##############################
# exit with success, tar file for download shall exist in ${TAR_FILE}
##############################
logger -p user.notice "$0: Diagnosis data creation successfully performed, file ready for download: '${TAR_FILE}'"
fnCleanup
exit 0
