#!/usr/local/bin/python3
# -*- coding: utf-8 -*-
##########################################################################
#
#    name:           CreateModelName.py
#    author:         Tilo Gericke (tig)
#    description:    Creates the model name (brand name) of this device
#                    based on hardware configuration file passed by -i option
#                    and writes this name to file passed by -o option
#
##########################################################################

import os
import sys
import argparse
import syslog
import re


def GetSD(attributes):
    if attributes['SDCard'] is not None:
        return "-{}".format(attributes['SDCard'])
    else:
        return ""

def GetIO(attributes):
    ioStr = ""
    try:
        if (int(attributes['InputCount']) > 0) or (int(attributes['OutputCount']) > 0):
            ioStr = "-IO"
    except:
        pass
    return ioStr

def IsFlashlessType(attributes):
    hwType = attributes['BoardID'][0:2]
    idNum  = int(attributes['BoardID'][2:5])
    if hwType == 'EH' and idNum >= 511:
        return True  # flashless type
    elif hwType == 'GH' and idNum >= 405:
        return True  # flashless type
    else:
        return False   # with flash

def GetName(attributes):
    modelName = ""

    if attributes['BoardID'] is None:
        raise Exception("'BoardID' not found.")
    
    hwType = attributes['BoardID'][0:2]
    if hwType == 'EH':
        flashLess = IsFlashlessType(attributes)
        modelName = "APS-180" + ('E' if flashLess else '') + GetIO(attributes) + (GetSD(attributes) if not flashLess else '')
    elif hwType == 'GH':
        flashLess = IsFlashlessType(attributes)
        modelName = "APS-90" + ('E' if flashLess else '') + GetIO(attributes) + (GetSD(attributes) if not flashLess else '')
    elif hwType == 'HH':
        modelName = "APS-90-Outdoor-PoE"
    elif hwType == 'IH':
        modelName = "APS-90-Outdoor"
    else:
        raise Exception("Unknown 'BoardID' found: '{}'.".format(attributes['BoardID']))
    
    return modelName

def CreateModelName(hwconfigfile, outputFile):

    hwAttributes = { 'BoardID'    : None,
                     'InputCount' : None,
                     'OutputCount': None,
                     'SDCard'     : None
                    }
    
    # read hwconfig and get required attributes
    with open(hwconfigfile) as fi:
        lines = fi.readlines()
    
    for line in lines:
        for attr in hwAttributes:
            if hwAttributes[attr] is not None:
                continue
            match = re.match("^{attr}=(.*)\n$".format(attr=attr), line)
            if match is not None:
                hwAttributes[attr] = match.group(1).rstrip()
                break
            
    # create the model name based on hwconfig attributes
    modelName = GetName(hwAttributes)
    
    # save model name to file
    with open(outputFile, mode='w') as fo:
        fo.write(modelName)
        
    syslog.syslog(syslog.LOG_NOTICE, "Wrote model name '{}' to '{}'.".format(modelName, outputFile))
    


#############################################################################
# main
#############################################################################
def main (argv):
    
    parser = argparse.ArgumentParser(description='Creates model name of this device and writes to output file')
    parser.add_argument('-i', dest='hwconfigfile', type=str, required=True, help="Input file 'hwconfig.txt' (required)")
    parser.add_argument('-o', dest='outputFile', type=str, required=True, help='Output file for created model name (required)')
    args = parser.parse_args()
    
    try:
        CreateModelName(args.hwconfigfile, args.outputFile)
    except Exception as e:
        syslog.syslog(syslog.LOG_ERR, "Failed to create model name. {}".format(e))
        sys.exit(1)

if __name__ == "__main__":
   main(sys.argv)