#!/bin/sh

fnEchoErr() { 
    echo "$@" 1>&2;
}

fnCleanup() {
    rm -rf ${TAR_TARGET_DIR} >/dev/null 2>&1
}

# Definitions:
#exit codes, shall be euqal with exit codes in C and in HMI
EXIT_CODE_SUCCESS=0
EXIT_CODE_EXPORT_UNKNOWN_RES=50
EXIT_CODE_EXPORT_INVALID_PARAM_STRUCT=51
EXIT_CODE_EXPORT_TAR_ERROR=52
EXIT_CODE_EXPORT_COULD_NOT_WRITE_FILE=53
EXIT_CODE_EXPORT_CREATE_TMP_DATA=54

# tar file
TAR_TARGET_DIR=/tmp/tmp_param_export/
TAR_FILE=/tmp/ExportParamSet.tar.gz

# root dir for tar parameter sets
PARAMS_ROOT_DIR=/var/Counter/configs/

# Parameter set directory IDs
# NOTE: for current firmware only one directory '0' currently used
CONFIGS_DIR_MIN_NUM=0
CONFIGS_DIR_MAX_NUM=0

# special files to include in tar
ORG_FILE_VERSION_PATH=/version.txt
ADD_FILE_VERSION_PATH=${TAR_TARGET_DIR}version.txt
ORG_FILE_COMPATIBILITY_LIST=/etc/Counter/update_compatibility_whitelist.txt
ADD_FILE_COMPATIBILITY_LIST=${TAR_TARGET_DIR}update_file_compatibility_data.txt

#logging
LOG_SCRIPT_NAME="ExportParamSet.sh"


# check if file exist, if exist delete it
if [ -f ${TAR_FILE}  ] ; then
    logger -p user.notice "${LOG_SCRIPT_NAME}: ${TAR_FILE} exist, will be deleted"
    rm -f ${TAR_FILE} >/dev/null 2>&1
fi


# check for configs0 ... configs31 existence
for i in `seq ${CONFIGS_DIR_MIN_NUM} ${CONFIGS_DIR_MAX_NUM}` ; do    
    if [ ! -d ${PARAMS_ROOT_DIR}${i} ] ; then
        logger -p user.error "${LOG_SCRIPT_NAME}: Parameter set missing: ${PARAMS_ROOT_DIR}${i}"
        fnEchoErr "invalid parameter directory structure on device"
        exit ${EXIT_CODE_EXPORT_INVALID_PARAM_STRUCT}
    fi
done


# Create data for export
# Create data directory in /tmp/
mkdir -p ${TAR_TARGET_DIR} >/dev/null 2>&1
RES_TMP=$?
if [ ${RES_TMP} -ne 0 ] ; then
    logger -p user.error "${LOG_SCRIPT_NAME}: could not create tmp directory: ${TAR_TARGET_DIR}"
    fnEchoErr "could not create tmp directory: ${TAR_TARGET_DIR}"
    fnCleanup
    exit ${EXIT_CODE_EXPORT_CREATE_TMP_DATA}
fi
# Copy compatibility list file
cp -f -p ${ORG_FILE_COMPATIBILITY_LIST} ${ADD_FILE_COMPATIBILITY_LIST} >/dev/null 2>&1
RES=$?
if [ ${RES} -ne 0 ] ; then
    logger -p user.error "${LOG_SCRIPT_NAME}: could not copy compatibility list file"
    fnEchoErr "could not copy compatibility list file"
    fnCleanup
    exit ${EXIT_CODE_EXPORT_CREATE_TMP_DATA}
fi
# Copy version.txt
cp -f -p ${ORG_FILE_VERSION_PATH} ${ADD_FILE_VERSION_PATH} >/dev/null 2>&1
RES=$?
if [ ${RES} -ne 0 ] ; then
    logger -p user.error "${LOG_SCRIPT_NAME}: could not copy version information file"
    fnEchoErr "could not copy version information file"
    fnCleanup
    exit ${EXIT_CODE_EXPORT_CREATE_TMP_DATA}
fi

# Copy required parameter files for export
for i in `seq ${CONFIGS_DIR_MIN_NUM} ${CONFIGS_DIR_MAX_NUM}` ; do
    # Create directory
    mkdir ${TAR_TARGET_DIR}${i}/ >/dev/null 2>&1
    RES=$?
    if [ ${RES} -ne 0 ] ; then
        logger -p user.error "${LOG_SCRIPT_NAME}: could not create tmp directory: ${TAR_TARGET_DIR}${i}"
        fnEchoErr "could not create tmp directory: ${TAR_TARGET_DIR}${i}"
        fnCleanup
        exit ${EXIT_CODE_EXPORT_CREATE_TMP_DATA}
    fi
    
    #
    # Copy XML parameter files
    #
    cp -f -p ${PARAMS_ROOT_DIR}${i}/*.xml ${TAR_TARGET_DIR}${i}/ >/dev/null 2>&1
    RES=$?
    if [ ${RES} -ne 0 ] ; then
        logger -p user.error "${LOG_SCRIPT_NAME}: could not copy 'XML' parameter files to tmp directory"
        fnEchoErr "could not copy 'XML' parameter files to tmp directory"
        fnCleanup
        exit ${EXIT_CODE_EXPORT_CREATE_TMP_DATA}
    fi
    
    #
    # Copy mandatory binary parameter files  (NOTE: add new binary files to string MANDATORY_BINARY_PARAMETER_FILES)
    #
    MANDATORY_BINARY_PARAMETER_FILES="dispmask.bit"
    for BIN_FILE in ${MANDATORY_BINARY_PARAMETER_FILES}; do
        cp -f -p ${PARAMS_ROOT_DIR}${i}/${BIN_FILE} ${TAR_TARGET_DIR}${i}/ >/dev/null 2>&1
        RES=$?
        if [ ${RES} -ne 0 ] ; then
            logger -p user.error "${LOG_SCRIPT_NAME}: could not copy '${BIN_FILE}' parameter file to tmp directory"
            fnEchoErr "could not copy '${BIN_FILE}' parameter file to tmp directory"
            fnCleanup
            exit ${EXIT_CODE_EXPORT_CREATE_TMP_DATA}
        fi
    done

    #
    # Copy optional binary parameter files  (NOTE: add new binary files to string OPTIONAL_BINARY_PARAMETER_FILES)
    #
    OPTIONAL_BINARY_PARAMETER_FILES="iotApiToken wifiAccess.conf"
    for BIN_FILE in ${OPTIONAL_BINARY_PARAMETER_FILES}; do
        if [ -f ${PARAMS_ROOT_DIR}${i}/${BIN_FILE} ] ; then
            cp -f -p ${PARAMS_ROOT_DIR}${i}/${BIN_FILE} ${TAR_TARGET_DIR}${i}/ >/dev/null 2>&1
            RES=$?
            if [ ${RES} -ne 0 ] ; then
                logger -p user.error "${LOG_SCRIPT_NAME}: could not copy '${BIN_FILE}' parameter file to tmp directory"
                fnEchoErr "could not copy '${BIN_FILE}' parameter file to tmp directory"
                fnCleanup
                exit ${EXIT_CODE_EXPORT_CREATE_TMP_DATA}
            fi
        else
            logger -p user.notice "${LOG_SCRIPT_NAME}: optional parameter file '${BIN_FILE}' not found."
        fi
    done

    #
    # Copy symlinks (NOTE: add new symlink below)
    #
    # Copy symlink: localtime (NOTE: only if exists)
    LOCALTIME_LINK=localtime
    if [ -h ${PARAMS_ROOT_DIR}${i}/${LOCALTIME_LINK} ] ; then
        cp -f -d -p ${PARAMS_ROOT_DIR}${i}/${LOCALTIME_LINK} ${TAR_TARGET_DIR}${i}/ >/dev/null 2>&1
        RES=$?
        if [ ${RES} -ne 0 ] ; then
            logger -p user.error "${LOG_SCRIPT_NAME}: could not copy '${LOCALTIME_LINK}' parameter file (symlink) to tmp directory"
            fnEchoErr "could not copy '${LOCALTIME_LINK}' parameter file (symlink) to tmp directory"
            fnCleanup
            exit ${EXIT_CODE_EXPORT_CREATE_TMP_DATA}
        fi
    fi
done


# tar (compressed with gzip) copied data for export
tar -c -z -f ${TAR_FILE} -C ${TAR_TARGET_DIR} . >/dev/null 2>&1
RES_TAR=$?
if [ ${RES_TAR} -ne 0 ] ; then
    logger -p user.error "${LOG_SCRIPT_NAME}: tar returned an error: ${RES_TAR}"
    fnEchoErr "tar command returned an error: ${RES_TAR}"
    rm -f ${TAR_FILE} >/dev/null 2>&1
    fnCleanup
    exit ${EXIT_CODE_EXPORT_TAR_ERROR}
fi


# exit with success, tar file for download shall exist in ${TAR_FILE}
logger -p user.notice "${LOG_SCRIPT_NAME}: Export operation successfully performed."
fnCleanup
exit ${EXIT_CODE_SUCCESS}
