#!/bin/sh

fnEchoErr() { 
    echo "$@" 1>&2;
}

fnCleanup() {
    rm -f ${TAR_FILE} >/dev/null 2>&1
    rm -R -f ${EXTRACT_TMP_DIR} >/dev/null 2>&1
}

# Definitions:
#arguments (mode)
MODE_ALL=1
MODE_PRESERVE_NETWORK_CONFIG=11
#exit codes
EXIT_CODE_SUCCESS=0
EXIT_CODE_IMPORT_UNKNOWN_RES=20
EXIT_CODE_IMPORT_TAR_FILE_NOT_FOUND=21
EXIT_CODE_IMPORT_COULD_NOT_EXTRACT_TAR=22
EXIT_CODE_IMPORT_TAR_VER_NOT_FOUND=23
EXIT_CODE_IMPORT_VERSION_ERR=24
EXIT_CODE_IMPORT_TAR_FILE_NOT_COMPLETE=26
EXIT_CODE_IMPORT_INVALID_ARG=27
EXIT_CODE_IMPORT_CONFIGS_DIR_NOT_EXIST=28
EXIT_CODE_IMPORT_COMPATIBILITY_CHECK_ERR=29

EXTRACT_TMP_DIR=/tmp/tmp_import/

# root directory for parameter sets
CONFIGS_DIR=/var/Counter/configs/

# Parameter set directory IDs
# NOTE: for current firmware only one directory '0' currently used
CONFIGS_DIR_MIN_NUM=0
CONFIGS_DIR_MAX_NUM=0

# version files
CUR_VER_FILE="/version.txt"
TAR_VER_FILE=${EXTRACT_TMP_DIR}version.txt

# invalid flag
INVALID_FLAG="_invalid"

# modified flag
MODIFIED_FLAG_FILE="_parameter_set_modified_after_import"

#logging
LOG_SCRIPT_NAME="ImportParamSet.sh"


# check arguments, exit if invalid
if [ "${#}" -ne 2 ] ; then
    logger -p user.error "${LOG_SCRIPT_NAME}: invalid number of arguments!"
    fnEchoErr "import script called with an invalid number of arguments (= ${#})"
    exit ${EXIT_CODE_IMPORT_INVALID_ARG}
fi


# get argument 1 (tar file FQFN), check if tar file exist, exit if not
TAR_FILE=${1}
if [ ! -f ${TAR_FILE}  ] ; then
    logger -p user.error "${LOG_SCRIPT_NAME}: ${TAR_FILE} not found"
    fnEchoErr "imported file not exist"
    exit ${EXIT_CODE_IMPORT_TAR_FILE_NOT_FOUND}
fi


# get argument 2 (mode value), check argument value
MODE=${2}
case ${MODE} in
    ${MODE_ALL})
        break
        ;;
    ${MODE_PRESERVE_NETWORK_CONFIG})
        break
        ;;
    *)
        logger -p user.error "${LOG_SCRIPT_NAME}: invalid mode argument (= ${MODE})!"
        fnEchoErr "invalid import operation mode (= ${MODE})"
        rm -f ${TAR_FILE}
        exit ${EXIT_CODE_IMPORT_INVALID_ARG}
esac


# check target parameter directory
# TODO adapt it if specification changed
if [ ! -d ${CONFIGS_DIR} ] ; then
    logger -p user.error "${LOG_SCRIPT_NAME}: '${CONFIGS_DIR}' is not an existing directory"
    fnEchoErr "parameter directory not exist on device"
    fnCleanup
    exit ${EXIT_CODE_IMPORT_CONFIGS_DIR_NOT_EXIST}
fi
# TODO check if link "/var/Counter/config" points to a directory "/var/Counter/configs/0...31/" ?


# extract (compressed with gzip) tar files
if [ ! -d ${EXTRACT_TMP_DIR} ] ; then
    mkdir ${EXTRACT_TMP_DIR}
fi
tar -x -z -f ${TAR_FILE} -C ${EXTRACT_TMP_DIR} >/dev/null 2>&1
RES_TAR=$?
if [ ${RES_TAR} -ne 0 ] ; then
    logger -p user.error "${LOG_SCRIPT_NAME}: tar returned an error: ${RES_TAR}"
    fnEchoErr "tar command returned an error: ${RES_TAR}"
    fnCleanup
    exit ${EXIT_CODE_IMPORT_COULD_NOT_EXTRACT_TAR}
fi


# hardware / firmware compatibility check
cd ${EXTRACT_TMP_DIR}
/bin/sh /sbin/FirmwareUpdateCompatibilityCheck.sh
RES=$?
if [ ${RES} -ne 0 ] ; then
    logger -p user.error "${LOG_SCRIPT_NAME}: hardware / firmware compatibility check failed (error code: ${RES})"
    fnEchoErr "hardware / firmware compatibility check failed (error code: ${RES})"
    fnCleanup
    exit ${EXIT_CODE_IMPORT_COMPATIBILITY_CHECK_ERR}
fi


# check version
if [ ! -f ${TAR_VER_FILE} ] ; then
    logger -p user.error "${LOG_SCRIPT_NAME}: Could not find version.txt in imported tar file."
    fnEchoErr "no version.txt file in imported tar file"
    fnCleanup
    exit ${EXIT_CODE_IMPORT_TAR_VER_NOT_FOUND}
fi
# get current version
CUR_VER_MAJOR=`cat ${CUR_VER_FILE} | awk 'BEGIN {FS="."}{print $1}'`
CUR_VER_MINOR=`cat ${CUR_VER_FILE} | awk 'BEGIN {FS="."}{print $2}'`
# get version from import
TAR_VER_MAJOR=`cat ${TAR_VER_FILE} | awk 'BEGIN {FS="."}{print $1}'`
TAR_VER_MINOR=`cat ${TAR_VER_FILE} | awk 'BEGIN {FS="."}{print $2}'`
# Version check (error if current version (major.minor) is lower than version from import)
if [ ${TAR_VER_MAJOR} -gt ${CUR_VER_MAJOR} ] ; then
    logger -p user.error "${LOG_SCRIPT_NAME}: version in imported tar (${TAR_VER_MAJOR}.${TAR_VER_MINOR}) is greater than current version (${CUR_VER_MAJOR}.${CUR_VER_MINOR})."
    fnEchoErr "version in imported tar (${TAR_VER_MAJOR}.${TAR_VER_MINOR}) is greater than current version (${CUR_VER_MAJOR}.${CUR_VER_MINOR})."
    fnCleanup
    exit ${EXIT_CODE_IMPORT_VERSION_ERR}
elif [ ${TAR_VER_MAJOR} -eq ${CUR_VER_MAJOR} ] ; then
    if [ ${TAR_VER_MINOR} -gt ${CUR_VER_MINOR} ] ; then
        logger -p user.error "${LOG_SCRIPT_NAME}: version in imported tar (${TAR_VER_MAJOR}.${TAR_VER_MINOR}) is greater than current version (${CUR_VER_MAJOR}.${CUR_VER_MINOR})."
        fnEchoErr "version in imported tar (${TAR_VER_MAJOR}.${TAR_VER_MINOR}) is greater than current version (${CUR_VER_MAJOR}.${CUR_VER_MINOR})."
        fnCleanup
        exit ${EXIT_CODE_IMPORT_VERSION_ERR}
    fi
fi


# Check imported files
CHECK_TOOL_BIN="/sbin/CheckXmlParamFile"
META_FILES_ROOT_PATH=/etc/Counter/ParamMetaFiles/
for i in `seq ${CONFIGS_DIR_MIN_NUM} ${CONFIGS_DIR_MAX_NUM}` ; do
    if [ -d ${EXTRACT_TMP_DIR}${i} ] ; then
        # get XML parameter file names and meta file names
        ls ${EXTRACT_TMP_DIR}${i}/ | grep .xml | while read LINE
        do
            FILE_PART_FOUND=`echo ${LINE} | grep -c '_values.xml'`
            if [ ${FILE_PART_FOUND} -eq 1 ] ; then
                # May be we have found a XML parameter file name
                PART=`echo ${LINE%_values.xml}`
                META_FILE=${META_FILES_ROOT_PATH}${PART}_meta.xml
                #echo "META_FILE : ${META_FILE}"
                VALUE_FILE=${EXTRACT_TMP_DIR}${i}/${LINE}
                #echo "VALUE_FILE: ${VALUE_FILE}"

                # XML parameter file check tool
                LOG_STR_1="XML parameter file check: ${VALUE_FILE} ... "
                ${CHECK_TOOL_BIN} -m ${META_FILE} -v ${VALUE_FILE} >/dev/null 2>&1
                # Evaluate result, error handling (and logging)
                RES_CHECK=$?
                case ${RES_CHECK} in
                    0)
                      true
                      ;;
                    1)
                      LOG_STR_2="failed (${RES_CHECK}: invalid arguments for ${CHECK_TOOL_BIN}), "
                      logger -p user.error "${LOG_STR_1} ${LOG_STR_2}"
                      rm -f ${VALUE_FILE}
                      ;;
                    2)
                      LOG_STR_2="failed (${RES_CHECK}: unknown error occured)."
                      logger -p user.error "${LOG_STR_1} ${LOG_STR_2}"
                      rm -f ${VALUE_FILE}
                      ;;
                    3)
                      LOG_STR_2="failed (${RES_CHECK}: creation error occured."
                      logger -p user.error "${LOG_STR_1} ${LOG_STR_2}"
                      rm -f ${VALUE_FILE}
                      ;;
                    *)
                      LOG_STR_2="failed (${RES_CHECK}: unknown return value from ${CHECK_TOOL_BIN})!"
                      logger -p user.error "${LOG_STR_1} ${LOG_STR_2}"
                      rm -f ${VALUE_FILE}
                      ;;
                esac
            fi
        done
    fi
done


# TODO may be further checks
# Currently only the XML files will be checked but not other files like
# - binary files (e.g. dispmask.bit)
# - symlinks (e.g. localtime)


# replace target directory content with imported directory content (depends on MODE parameter)
for i in `seq ${CONFIGS_DIR_MIN_NUM} ${CONFIGS_DIR_MAX_NUM}` ; do
    if [ -d ${EXTRACT_TMP_DIR}${i} ] ; then
        # check if target directory exist
        if [ ! -d ${CONFIGS_DIR}${i} ] ; then
            logger -p user.notice "${LOG_SCRIPT_NAME}: Directory ${CONFIGS_DIR}${i} not found, will be created."
            mkdir ${CONFIGS_DIR}${i}
        fi

        # MODE dependend handling for target directory (CR 522747)
        if [ "${MODE}" -eq "${MODE_PRESERVE_NETWORK_CONFIG}" ] ; then
            # MODE == MODE_PRESERVE_NETWORK_CONFIG
            # do not delete current network configuration parameter file(s) in target directory
            # - keep current network configuration files on device
            # - delete all other files
            find ${CONFIGS_DIR}${i}/* ! \(  -name nw_eth0_dhcp_params_values.xml        \
                                         -o -name nw_eth0_hostname_params_values.xml    \
                                         -o -name nw_eth0_static_dns_params_values.xml  \
                                         -o -name nw_eth0_static_ipv4_params_values.xml \
                                        \)                                              \
                                        -delete >/dev/null 2>&1
        else
            # MODE == MODE_ALL
            # clean the complete target directory (CR 386045)
            # NOTE: current maskdoor learning data file will be deleted too so 
            #       learning phase needs to pass through again.
            rm -Rf ${CONFIGS_DIR}${i}/* >/dev/null 2>&1
        fi
        
        
        # modified flag files shall not imported, delete them if exist
        # Commented out cause modified flag file currently not exist !
        #if [ -f ${EXTRACT_TMP_DIR}${i}/${MODIFIED_FLAG_FILE} ] ; then
        #    rm -f ${EXTRACT_TMP_DIR}${i}/${MODIFIED_FLAG_FILE} >/dev/null 2>&1
        #fi
        
        
        # MODE dependend handling of imported directory (CR 522747)
        if [ "${MODE}" -eq "${MODE_PRESERVE_NETWORK_CONFIG}" ] ; then
            # MODE == MODE_PRESERVE_NETWORK_CONFIG
            # avoid import of network configuration settings
            # - delete param files that configure network settings from import directory
            rm -f ${EXTRACT_TMP_DIR}${i}/nw_eth0_dhcp_params_values.xml >/dev/null 2>&1
            rm -f ${EXTRACT_TMP_DIR}${i}/nw_eth0_hostname_params_values.xml >/dev/null 2>&1
            rm -f ${EXTRACT_TMP_DIR}${i}/nw_eth0_static_dns_params_values.xml >/dev/null 2>&1
            rm -f ${EXTRACT_TMP_DIR}${i}/nw_eth0_static_ipv4_params_values.xml >/dev/null 2>&1            
        fi
        
        
        # move all files from import to target directory
        # TODO: May be a whitelist could be useful to avoid copy unknown files from import to target directory.
        #       Currently it is possible to copy unknown files via import operation to target parameter set.
        #       Then, every file name has to be checked. Furthermore, a whitelist is software version depended and
        #       must be maintained for each change.
        mv -f ${EXTRACT_TMP_DIR}${i}/* ${CONFIGS_DIR}${i}/
        # set ownership
        chown -R -h 0:0 ${CONFIGS_DIR}${i}
    fi   
done


# clean up and exit
fnCleanup
logger -p user.notice "${LOG_SCRIPT_NAME}: Import operation successfully performed."
exit ${EXIT_CODE_SUCCESS}
