#!/bin/sh
# This script will set the configuration files to the default stored in the directory FactorySettings.
# The first parameter is the base path of the configuration (default: /var/Counter)
# The following steps are executed:
# 1.) Check, if the subdirectory DeviceSpecfic exists an that the given path is not /.
# 2.) Remove all current configuration files.
# 3.) Remove current and temporary calibration files
# 4.) Remove DynCalib debug folders
# 5.) Restore login passwords
# 6.) Copy the FactorySettings directory

# Return immediately on an error
# set -e

# Set the debug flag
# set -x

# TODO: 
# Some directories and files will not exist, if we call this script from the fallback system. E.g.
# - /var/tmp/cal
# - /media
# - /var/home
# - /CustomData
# - /var/log/dynCalib.log

# Extract the parameter
VARCOUNTER=$1

# Set default value, if undefined
if [ -z ${VARCOUNTER} ] ; then
	VARCOUNTER=/var/Counter
fi

# We do not want to use the root directory as base path
if [ ${VARCOUNTER} = "/" ] ; then
	echo "Error: Path / is forbidden as base path for the configuration!"
	exit 1
fi

echo "Using directory ${VARCOUNTER} as configuration base path."

# Check for use of uninitialized variables
set -u

# Set some parameters
DEVICESPECIFIC=DeviceSpecific
FACTORYSETTINGS=FactorySettings
CONFIGS=configs
VARTMPCAL=/var/tmp/cal

# Change to the base director
cd ${VARCOUNTER}

# Check the existence of the DeviceSpecific subdirectory
if ! [ -d ${DEVICESPECIFIC} ] ; then
	echo "Error: ${VARCOUNTER}/${DEVICESPECIFIC} does not exist!"
	exit 2
fi

# Delete configuration files and localtime symlink
find configs -type d -mindepth 2 -exec rm -rf {} \; 2> /dev/null
find configs -type f -exec rm -f {} \; 2> /dev/null
find configs -name localtime -exec rm -f {} \; 2> /dev/null

# Create at least the subdirectory 0
mkdir -p ${CONFIGS}/0

# delete the password files. Will be restored later from default settings location or at next boot
rm -f ${DEVICESPECIFIC}/shadow
rm -f ${DEVICESPECIFIC}/gshadow

# delete configuration files of wireless services
rm -f ${DEVICESPECIFIC}/hostapd.conf
rm -f ${DEVICESPECIFIC}/hostapd.accept
rm -f ${DEVICESPECIFIC}/udhcpd.conf

# sshd
rm -f ${DEVICESPECIFIC}/sshd_config
rm -f ${DEVICESPECIFIC}/known_hosts

# remove calibration files
rm -f ${DEVICESPECIFIC}/intern_640x480.ini.cur
rm -f ${DEVICESPECIFIC}/intern_640x480.ini.cur.md5
rm -rf ${VARTMPCAL}

# watson iot
# iotApiToken is in /var/Counter/config and wil be removed implicitly
rm -f ${DEVICESPECIFIC}/iotDevToken

# vpn
rm -f /etc/openvpn/client.conf

# mbproxy / multi-sensor-fusion
rm -f /var/home/mbproxy/.ssh/*

# Copy configuration files from default settings location
cp -r ${FACTORYSETTINGS}/* .

# Clean SOAP storage
# Check if SD card is mounted:
mount |grep -q '/media/sdcard'
if [ $? -eq 0 ] ; then
	# SD card is mounted
	SOAP_TASK_PATH=/media/sdcard/HAGL/SOAP/tasks/
	if [ -d ${SOAP_TASK_PATH} ] ; then
		# delete all stored SOAP tasks
		rm -f ${SOAP_TASK_PATH}*
	fi
	SOAP_NOTIFICATIONS_PATH=/media/sdcard/HAGL/SOAP/notifications/
	if [ -d ${SOAP_NOTIFICATIONS_PATH} ] ; then
		# delete all stored Notifications
		rm -f ${SOAP_NOTIFICATIONS_PATH}*
	fi
fi

# Delete all SOAP tasks stored on internal flash
SOAP_TASK_PATH=/CustomData/HAGL/SOAP/tasks/
if [ -d ${SOAP_TASK_PATH} ] ; then
	# delete all stored SOAP tasks
	rm -f ${SOAP_TASK_PATH}*
fi
# Delete all stored Notifications on internal flash
SOAP_NOTIFICATIONS_PATH=/CustomData/HAGL/SOAP/notifications/
if [ -d ${SOAP_NOTIFICATIONS_PATH} ] ; then
	# delete all stored Notifications
	rm -f ${SOAP_NOTIFICATIONS_PATH}*
fi

# Clean dynamic calibration debug data
# Test for GetSorePath.sh which does not exist in the fallback system
GET_STORE_PATH_SCRIPT=/sbin/GetStorePath.sh
if [ -x ${GET_STORE_PATH_SCRIPT} ] ; then
    VALS=`${GET_STORE_PATH_SCRIPT}`
    set ${VALS}
    STORE_PATH=$2
    DYN_CALIB_PATH=${STORE_PATH}/DynCalib
    if [ -d ${DYN_CALIB_PATH} ] ; then
        rm -rf ${DYN_CALIB_PATH}/*
    fi
fi

# remove dyncalib logfile
rm -f /var/log/dynCalib.log

logger "Factory Reset performed"

sync

sleep 0.2
if [ -d ${VARCOUNTER} ] ; then
    mount -oremount,ro ${VARCOUNTER}
    mount -f -oremount,ro ${VARCOUNTER}
    sleep 0.2
    mount -oremount,rw ${VARCOUNTER}
fi

exit 0

