#!/usr/bin/env /usr/local/bin/python3
# -*- coding: utf-8 -*-
##########################################################################
#
#    name:           StartScheduledVideoRecording.py
#    author:         Andrej Wunrau@hella.com (awu)
#    description:    Starts video recording and terminates recording
#                    after a desired amount of time.
#
##########################################################################

import os
import sys
import argparse
import struct
import syslog
import time
from enum import Enum
import threading
from threading import Timer,Thread,Event
from crontab import CronTab


# set correct import paths and import needed modules
sys.path.append(os.path.join(os.path.dirname(__file__), '/opt/WebServices/server/lib'))
from msgBroker.PyMsgBroker import PyMsgBroker


#############################################################################
# helper classes
#############################################################################

# enumeration class for state of video recording daemon
class VideoRecordingState(Enum):
    uninitialized   = 0
    active          = 1
    inactive        = 2

# holds all information concerning the parameter
class ParamData:
    recordDuration  = 0
    beVerbose       = False
    repeatMode      = False

# holds data that is shared between threads 
class SharedData:
    continueRecording   = False
    waitForMessage      = False 
    videoRecordingState = VideoRecordingState(0)

    def __init__(self):
        self.lock = threading.Lock()

    def getContinueRecording(self):
        self.lock.acquire()
        retVal = self.continueRecording
        self.lock.release()
        return retVal

    def setContinueRecording(self, value):
        self.lock.acquire()
        self.continueRecording = value
        self.lock.release()

    def getWaitForMessage(self):
        self.lock.acquire()
        retVal = self.waitForMessage
        self.lock.release()
        return retVal

    def setWaitForMessage(self, value):
        self.lock.acquire()
        self.waitForMessage = value
        self.lock.release()

    def getVideoRecordingState(self):
        self.lock.acquire()
        retVal = self.videoRecordingState
        self.lock.release()
        return retVal

    def setVideoRecordingState(self, value):
        self.lock.acquire()
        self.videoRecordingState = value
        self.lock.release()


#############################################################################
# class ScheduledVideoRecorder
#############################################################################
class ScheduledVideoRecorder:

    # the Ids here are hardcoded,
    # TODO: we should not define those Ids ourselves,
    # but should reuse the Ids that are defined only once globally
    
    # AppIds
    APP_ID_VIDEORECORDING_D                     = 23
    
    # MsgIds
        # video recording
    APC_VIDEORECORDING_D_MSG_ID_STARTSTOP       = 1
    APC_VIDEORECORDING_D_MSG_ID_REQ_STATUS      = 2
    APC_VIDEORECORDING_D_MSG_ID_RES_STATUS      = 3


    def __init__(self):
        self.paramData  = ParamData()
        self.sharedData = SharedData()
        self.msgBroker  = PyMsgBroker()

        # message to listen to
        msgIdRegList = [
                (self.APP_ID_VIDEORECORDING_D,               self.APC_VIDEORECORDING_D_MSG_ID_RES_STATUS),
                (self.APP_ID_VIDEORECORDING_D,               self.APC_VIDEORECORDING_D_MSG_ID_STARTSTOP)
        ]
        # connect to MsgBroker and register MsgIds
        self.msgBroker.connectToMsgBroker()
        self.msgBroker.subscribeToMsgIDs(msgIdRegList, self.__msgCallback)


    def __durationTimerCallback(self):
        self.sharedData.setContinueRecording(False)
        
    def __messageTimerCallback(self):
        self.sharedData.setWaitForMessage(False)

    def __startVideoRecording(self):
        msgPayload = struct.pack( "I", 1)
        self.msgBroker.sendMessage(self.APP_ID_VIDEORECORDING_D, self.APC_VIDEORECORDING_D_MSG_ID_STARTSTOP, msgPayload)

    def __stopVideoRecording(self):
        msgPayload = struct.pack( "I", 0)
        self.msgBroker.sendMessage(self.APP_ID_VIDEORECORDING_D, self.APC_VIDEORECORDING_D_MSG_ID_STARTSTOP, msgPayload)
    
    def __getVideoRecordingState(self):
        retVal = True
        # request video recording state
        self.msgBroker.sendMessage(self.APP_ID_VIDEORECORDING_D, self.APC_VIDEORECORDING_D_MSG_ID_REQ_STATUS)
        
        # wait for answer, but timeout after 1 second
        self.sharedData.setWaitForMessage(True)
        msgTimer = Timer(1, self.__messageTimerCallback)
        msgTimer.start()
        while (self.sharedData.getWaitForMessage()):
            pass

        # check if recording state was received
        if (self.sharedData.getVideoRecordingState() == VideoRecordingState.uninitialized):
            syslog.syslog(syslog.LOG_ERR, 'Error: could not retrieve status of video recording, waiting for message timed out.')
            retVal = False
        return retVal


    def __msgCallback(self, appID, msgID, i_Data):
        if (self.paramData.beVerbose):
            syslog.syslog(syslog.LOG_INFO, 'message callback triggered')

        # APP_ID_VIDEORECORDING_D
        if (appID == self.APP_ID_VIDEORECORDING_D):
            # APC_VIDEORECORDING_D_MSG_ID_RES_STATUS
            if (msgID == self.APC_VIDEORECORDING_D_MSG_ID_RES_STATUS):
                if (self.paramData.beVerbose):
                    syslog.syslog(syslog.LOG_INFO, 'message received: APC_VIDEORECORDING_D_MSG_ID_RES_STATUS')
                # extract video recording state
                data = struct.unpack("I", i_Data)
                recState = data[0]

                if (recState != 0 ):
                    self.sharedData.setVideoRecordingState(VideoRecordingState.active)
                else:
                    self.sharedData.setVideoRecordingState(VideoRecordingState.inactive)

                # end waiting for message
                self.sharedData.setWaitForMessage(False)

            # APC_VIDEORECORDING_D_MSG_ID_STARTSTOP
            elif (msgID == self.APC_VIDEORECORDING_D_MSG_ID_STARTSTOP):
                # extract command
                data = struct.unpack("I", i_Data)
                cmd = data[0]
                
                # stop-command received?
                if (cmd == 0):
                    if (self.paramData.beVerbose):
                        syslog.syslog(syslog.LOG_INFO, 'message received: APC_VIDEORECORDING_D_MSG_ID_STARTSTOP - stop')
                    # trigger recording stop
                    self.sharedData.setContinueRecording(False)


    # public functions
    def parseArgs(self):
        parser = argparse.ArgumentParser(description='Starts video recording and terminates recording after a desired amount of time.')

        # configure parser
        parser.add_argument('record_duration', metavar='record_duration', type=int, 
                    help='desired video duration in seconds')
        parser.add_argument('-v' , '--verbose', dest='verbose', action='store_true', default=False,
                    help='verbose output')
        parser.add_argument('-repeatMode' , dest='repeatMode', action='store_true', default=False, help='repetition interval is set')

        # parse
        args = parser.parse_args()

        # extract arguments
        self.paramData.recordDuration = args.record_duration
        self.paramData.beVerbose = args.verbose
        self.paramData.repeatMode = args.repeatMode

        # debug
        if (self.paramData.beVerbose):
            syslog.syslog(syslog.LOG_INFO, 'script called with arguments: ')
            syslog.syslog(syslog.LOG_INFO, '\tduration: ' + str(self.paramData.recordDuration))
            syslog.syslog(syslog.LOG_INFO, '\tbeVerbose: ' + str(self.paramData.beVerbose))
            syslog.syslog(syslog.LOG_INFO, '\trepeatMode: ' + str(self.paramData.repeatMode))
            
    def removeCronTabItems(self):
        #TODO put this to a central point
        featureComment = 'CUSTOMER_VIDEO_TIMER'
        try: 
            myCronTab = CronTab(user='root')
        except:
            #we dont have a crontab nothing to remove
            return
        jobs = myCronTab.find_comment(featureComment)
        for job in jobs:
            syslog.syslog(syslog.LOG_INFO,"StartScheduledVideoRecording -> remove customer video timer: "+str(job))
            myCronTab.remove(job)
        myCronTab.write()

    def executeOperation(self):
        retVal = True;

        try: 
            # start message listener
            if (not self.msgBroker.startMsgListener()):
                syslog.syslog(syslog.LOG_ERR, 'Error, startMsgListener() returned false.')
                retVal = False
            else:
                # message broker listener is active
                # ask for recording state                
                if ( not self.__getVideoRecordingState()):
                    retVal = False
                else:
                    # recording state received, check
                    if (self.sharedData.getVideoRecordingState() == VideoRecordingState.active):
                        syslog.syslog(syslog.LOG_INFO, 'Video recording is already active, aborting ...')
                        retVal = False
                    else:
                        # setup timer that cancels recording after timeout
                        durationTimer = Timer(self.paramData.recordDuration, self.__durationTimerCallback)
                        durationTimer.start()

                        # init variable continueRecording
                        self.sharedData.setContinueRecording(True)

                        # start video recording
                        self.__startVideoRecording()

                        # wait until variable continueRecording is set to False (by either expired timer or pipeline monitor stop message)
                        while(self.sharedData.getContinueRecording()):
                            time.sleep(0.1)   # TODO: workaround to reduce cpu load, better use async event handling

                        # variable continueRecording was set to False -> recording duration exceeded or user stopped recording manually
                        # stop recording
                        self.__stopVideoRecording()


        except Exception as e:
            syslog.syslog(syslog.LOG_ERR, "Exception: {0}".format(e))
            retVal = False

        return retVal


#############################################################################
# main
#############################################################################
def main (argv):
    # create recorder object
    recorder = ScheduledVideoRecorder()
    # get script call arguments
    recorder.parseArgs()
    if (recorder.paramData.repeatMode == False):
        #remove all crontab items related to this feature. Script is executed only once
        recorder.removeCronTabItems()
        
    # and execute the corresponding operations
    retVal = recorder.executeOperation()
    if (retVal == False):
        sys.exit(1)


if __name__ == "__main__":
   main(sys.argv)
