#!/bin/sh

#################################################################################################
#   (c) 2016 by Hella Aglaia Mobile Vision GmbH
#   Author: Andrej Wunrau
#   No usage without written permission.
#   All rights reserved.
#
#   Module: WirelessServices
#   File: btSetMode.sh
#
#   Description:    
#
#       The script can be used to switch between the bluetooth services modes.
#       The following modes are defined: 
#           - MODE_OFF
#           - MODE_WIRELESS_TRACKING
#
#################################################################################################


# import global variables and functions 
. btGlobalVariables.sh
. btGlobalFunctions.sh


usage()
{
    log "Usage: $0 <interface> <new_mode> [<old_mode>]"
    log "mode can be : MODE_OFF/MODE_WIRELESS_TRACKING"
    log "old_mode is optional, if not provided script will take old_mode from param files"
}


get_wireless_mode_from_param_file ()
{
    MODE=`GetValueFromXmlFile -k Mode -v ${GLOBAL_BLUETOOTH_SERVICES_PARAM_VALUE_FILE} -m ${GLOBAL_BLUETOOTH_SERVICES_PARAM_META_FILE}`
    if [ $? -ne 0 ]
    then
        log "Error, call to GetValueFromXmlFile failed."
        echo ""
        return 1
    fi

    echo ${MODE}
    return 0
}


validate_mode ()
{
    MODE=$1

    # check arguments
    if [ $# -ne 1 ]
    then
        log "Error, wrong number of parameters"
        return 1
    fi

    case ${MODE} in

        MODE_OFF|MODE_WIRELESS_TRACKING)
            return 0
        ;;

        *)
            log "Error, undefined mode: \"${MODE}\""
            return 1
        ;;
    esac

    return 0
}


#################################################################################################
# script starts here
#################################################################################################

# arguments
INTERFACE=$1
NEW_MODE=$2
CURR_MODE=$3

# check arguments
if [ $# -lt 2 -o $# -gt 3 ]
then
    log "Error, $0 wrong number of parameters!"
    usage
    exit 1
fi

# check new mode argument
validate_mode ${NEW_MODE}
if [ $? -ne 0 ]
then
    log "Error, invalid argument ${NEW_MODE}"
    usage
    exit 1
fi

# check bt device existence
dev_state=`is_bluetooth_device_available ${INTERFACE}`
if [ $dev_state != "up" ]
then
    log "Error, no bt device present in system!"
    exit 3
fi

 
# if current mode is not provided -> derive from param files
if [ -z ${CURR_MODE} ]
then
    CURR_MODE=`get_wireless_mode_from_param_file`
fi

# check current mode
validate_mode ${CURR_MODE}
if [ $? -ne 0 ]
then
    log "Warning, invalid mode \"${CURR_MODE}\" found in parameter file. "
    # the current mode is invalid, something is fishy
    # set current mode empty string, this will trigger error handling
    CURR_MODE=""
fi


# stop all modes if current mode is invalid
if [ -z ${CURR_MODE} ]
then
    # stop "all" modes
    btModeTracking.sh ${INTERFACE} stop
else 
    # first disable old mode (if not off or unavailable)
    if [ $CURR_MODE == MODE_WIRELESS_TRACKING ]
    then
        btModeTracking.sh ${INTERFACE} stop
    fi
fi

# enable new mode
if [ $NEW_MODE == MODE_WIRELESS_TRACKING ]
then
    btModeTracking.sh ${INTERFACE} start
    ERR_CODE=$?
    if [ ${ERR_CODE} -ne 0 ]
    then
        log "Error, failed to enable new $NEW_MODE, btModeTracking.sh returned with error code ${ERR_CODE}"
        exit 8
    fi
else
    if [ $NEW_MODE == MODE_OFF ]
    then
        btModeTracking.sh ${INTERFACE} stop
    fi
fi

# all ok
logger -p user.notice "Info, new wireless mode $NEW_MODE"
exit 0
