#!/bin/sh

#################################################################################################
#   (c) 2016 by Hella Aglaia Mobile Vision GmbH
#   Author: Andrej Wunrau
#   No usage without written permission.
#   All rights reserved.
#
#   Module: WirelessServices
#   File: wifiGetConfiguration.sh
#
#   Description:    
#
#       Script allows to read certain configuration values from config files used for
#       the different wireless service modes. The values are read from the configuration
#       files and are printed to stdout.
#
#################################################################################################


# import global variables and functions 
. wifiGlobalVariables.sh
. wifiGlobalFunctions.sh


usage ()
{
    log "Usage: $0 <command>"
    log "possible commands:"
    log "       hostapd_ssid"
    log "       hostapd_channel" 
    log "       hostapd_hide_ssid"
    log "       hostapd_use_mac_filter"
    log "       wpa_ssid"
}


get_value_from_hostapd_conf ()
{
    OPTION=$1

    # check arguments
    if [ $# -ne 1 ]
    then
        log "Error: wrong number of parameters"
        return 1
    fi

    # get value from config file
    if [ -f ${GLOBAL_HOSTAPD_CONF} ]
    then
        value=`get_option_from_config_file ${GLOBAL_HOSTAPD_CONF} ${OPTION} =`
        if [ -z ${value} ]
        then
            log "Error: could not obtain value ${OPTION} from config file ${GLOBAL_HOSTAPD_CONF}"
            return 3
        else 
            echo $value
            return 0
        fi
    else
        log "Error: could not find configuration file ${GLOBAL_HOSTAPD_CONF}"
        return 2
    fi
    return 0
}


get_ssid_from_wpa_config ()
{
    if [ -f ${GLOBAL_WIFI_ACCESS_CONF} ]
    then
        ssid=`cat ${GLOBAL_WIFI_ACCESS_CONF} | sed -n -e 's/\(.*\)\"\(.*\)\"/\2/p'`
        echo "${ssid}"
    else
        echo ""
    fi
    return 0
}


#################################################################################################
# script starts here
#################################################################################################

if [ $# -ne 1 ]
then
    log "Error, wrong number of parameters in call to script"
    usage
    exit 1
else 
    # arguments
    COMMAND=$1

    # license check, only continue if license exists on device
    lic_available=`is_wifi_license_available`
    if [ ${lic_available} = "no" ]
    then
        log "Error, license for wireless services missing on device"
        exit 2
    fi
   
    case ${COMMAND} in

        hostapd_ssid)
            ssid=`get_value_from_hostapd_conf ssid`
            if [ -z ${ssid} ]
            then
                exit 4
            else
                echo ${ssid}
                exit 0
            fi
        ;;

        hostapd_channel)
            channel=`get_value_from_hostapd_conf channel`
            if [ -z ${channel} ]
            then
                exit 5
            else
                echo ${channel}
                exit 0
            fi
        ;;

        hostapd_hide_ssid)
            hide=`get_value_from_hostapd_conf ignore_broadcast_ssid`
            if [ -z ${hide} ]
            then
                exit 6
            else
                echo ${hide}
                exit 0
            fi
        ;;

        hostapd_use_mac_filter)
            use_mac_filter=`get_value_from_hostapd_conf macaddr_acl`
            if [ -z ${use_mac_filter} ]
            then
                exit 7
            else
                echo ${use_mac_filter}
                exit 0
            fi
        ;;

        wpa_ssid)
            ssid=`get_ssid_from_wpa_config`
            if [ -z ${ssid} ]
            then
                exit 8
            else
                echo ${ssid}
                exit 0
            fi
        ;;

        *)
            log "Error, undefined command: \"$1\"."
            usage
            exit 3
        ;;
    esac

    exit 0
fi
