#!/bin/sh

#################################################################################################
#   (c) 2016 by Hella Aglaia Mobile Vision GmbH
#   Author: Andrej Wunrau
#   No usage without written permission.
#   All rights reserved.
#
#   Module: WirelessServices
#   File: wifiGlobalFunctions.sh
#
#   Description:    
#
#       Script holds a set of functions that are shared between
#       the different script of the wireless services module.
#
#################################################################################################

# set DEBUG=1 to write logs to stderr (in addition to logs in /var/log/messages)
DEBUG=0


###################################################################################################################################
# logging functions

# logs to std err
logToConsole () { 
    echo -e "$0, $@" 1>&2;
}

# logs to /var/log/messages
log () {
    logger -p user.error "$0: $1"
    # if DEBUG is 1 -> also log to console
    if [ ${DEBUG} -eq 1 ]
    then
        logToConsole $1
    fi
}


notice () {
    logger -p user.notice "$0: $1"
    # if DEBUG is 1 -> also log to console
    if [ ${DEBUG} -eq 1 ]
    then
        logToConsole $1
    fi
}


###################################################################################################################################
# license functions
is_wifi_license_available () { 
    result=`liccheck -f ${GLOBAL_WIRELESS_LICENSE_NAME}`
    if [ $? -eq 0 ]
    then
        echo "yes"
    else
        echo "no"
    fi
   return 0
}


###################################################################################################################################
# dongle status functions
is_wifi_device_available ()
{
    if [ $# -eq 1 ]
    then
        INTERFACE=$1
        result=`iw dev | awk '/Interface wlan/{ found = 1} END { if (found == 1) { print "up" } else { print "down"} }'`
        echo ${result}
        return 0
    else
        log "Error: {$0}, wrong number of parameters on call to function is_wifi_device_available."
        return 1 
    fi
}


###################################################################################################################################
# regulatory domain functions
get_interface_regulatory_domain ()
{
    if [ $# -eq 0 ]
    then
        result=`iw reg get | awk '/country ([0-9A-Z]{2}):/{ print substr($2, 1, length($2 - 1))}'`
        echo ${result}
        return 0
    else
        log "Warning: {$0}, wrong number of parameters on call to function get_interface_regulatory_domain."
        return 1
    fi
}


###################################################################################################################################
# interface info functions    

get_interface_state ()
{
    if [ $# -eq 1 ]
    then
        INTERFACE=$1
        result=`ifconfig ${INTERFACE} | awk '/ UP /{ found = 1} END { if (found == 1) { print "up" } else { print "down"} }'`
        echo ${result}
        return 0
    else
        log "Error: {$0}, wrong number of parameters on call to function check_interface_state."
        return 1
    fi
}

get_interface_ip ()
{
    if [ $# -eq 1 ]
    then
        INTERFACE=$1
        result=`ifconfig ${INTERFACE} | awk '/inet addr/ {print substr($2,6)}'`
        echo ${result}
        return 0
    else
        log "Error: {$0}, wrong number of parameters on call to function get_interface_ip."
        return 1
    fi
}

get_interface_mac ()
{
    if [ $# -eq 1 ]
    then
        INTERFACE=$1
        result=`ifconfig ${INTERFACE} | awk '/ HWaddr / {print $5}'`
        echo ${result}
        return 0
    else
        log "Error: {$0}, wrong number of parameters on call to function get_interface_mac."
        return 1
    fi
}

get_connection_state ()
{
    if [ $# -eq 1 ]
    then
        INTERFACE=${1}
        result=`iw ${INTERFACE} link | awk '/^Not connected/ {print "down"} /^Connected/ {print "up"}'`
        echo ${result}
        return 0
    else
        log "Error: {$0}, wrong number of parameters on call to function get_connection_state."
        return 1
    fi
}


###################################################################################################################################
# config file handling functions

add_option_to_config_file ()
{
    CONF_FILE=$1
    OPTION_NAME=$2
    SEP=$3
    OPTION_VALUE=$4

    if [ $# -ne 4 ]
    then
        log "Error: wrong number of parameters"
        return 1
    else
        # find entry "${OPTION_NAME}  ${OPTION_VALUE}" in ${CONF_FILE}, if found, don't change file
        FOUND=`cat ${CONF_FILE} | grep "${OPTION_NAME}${SEP}${OPTION_VALUE}" | wc -l`
        if [ ${FOUND} -eq 0 ]
        then
            # find entry ${OPTION_NAME} in ${CONF_FILE}
            FOUND=`cat ${CONF_FILE} | grep ${OPTION_NAME} | wc -l`
            if [ ${FOUND} -eq 0 ]
            then
                # option not found in file, just add, also add line feed if neccessary
                LAST_CHAR=$(tail -c 1 ${CONF_FILE})
                if [ "${LAST_CHAR}" != "" ]
                then
                    echo "" >> ${CONF_FILE}
                fi
                echo "${OPTION_NAME}${SEP}${OPTION_VALUE}" >> ${CONF_FILE}
            else
                # config option found in file: replace
                TEMPFILE=`mktemp`
                sed -e "s/^${OPTION_NAME}.*/${OPTION_NAME}${SEP}${OPTION_VALUE}/" ${CONF_FILE} >> ${TEMPFILE}
                cp ${TEMPFILE} ${CONF_FILE}
                rm ${TEMPFILE}
            fi
        fi
    fi

    return 0
}


get_option_from_config_file ()
{
    CONF_FILE=$1
    OPTION_NAME=$2
    SEP=$3

    if [ $# -ne 3 ]
    then
        log "Error: wrong number of parameters"
        return 1
    else
        FOUND=`cat ${CONF_FILE} | grep "^${OPTION_NAME}${SEP}" | wc -l`
        if [ ${FOUND} -eq 1 ]
        then
            VALUE=`cat ${CONF_FILE} | grep "^${OPTION_NAME}${SEP}" | awk -F '=' '{print $2}'`
            echo "${VALUE}"
            return 0
         else
            echo ""
            return 2
        fi
    fi

    return 0
}


###################################################################################################################################
# mode status functions

is_process_active ()
{
    PROC_NAME=$1

    if [ $# -ne 1 ]
    then
        log "Error: wrong number of parameters"
        return 1
    fi

    RES=`ps | grep ${PROC_NAME} | wc -l `
    if [ ${RES} -gt 1 ]
    then
        echo "yes"
    else
        echo "no"
    fi

    return 0
}


is_interface_in_monitor_mode ()
{
    INTERFACE=$1

    if [ $# -ne 1 ]
    then
        log "Error: wrong number of parameters"
        return 1
    fi

    RES=`iw wlan0 info | grep "type monitor" | wc -l`
    if [ ${RES} -eq 1 ]
    then
        echo "yes"
    else
        echo "no"
    fi

    return 0
}


is_interface_in_ap_mode ()
{
    INTERFACE=$1

    if [ $# -ne 1 ]
    then
        log "Error: wrong number of parameters"
        return 1
    fi

    # RES=`iw wlan0 info | grep "type AP\|channel" | wc -l`
    # if [ ${RES} -eq 2 ]
    RES=`iw wlan0 info | grep "type AP" | wc -l`
    if [ ${RES} -eq 1 ]
    then
        echo "yes"
    else
        echo "no"
    fi

    return 0
}


is_interface_in_wifi_access_mode ()
{
    INTERFACE=$1

    if [ $# -ne 1 ]
    then
        log "Error: wrong number of parameters"
        return 1
    fi

    RES=`iw wlan0 info | grep "type managed\|channel" | wc -l`
    if [ ${RES} -eq 2 ]
    then
        echo "yes"
    else
        echo "no"
    fi

    return 0
}


get_current_wirless_mode ()
{
    INTERFACE=$1

    if [ $# -ne 1 ]
    then
        log "Error: wrong number of parameters"
        return 1
    fi

    interface_state=`get_interface_state ${INTERFACE}`

    if [ ${interface_state} = "down" ]
    then 
        # if interface is down -> off mode
        echo "MODE_OFF"
        return 0
    else
        if [ ${interface_state} = "up" ]
        then 

             # log "INTERFACE_MONITOR_MODE: ${INTERFACE_MONITOR_MODE}"
             # log "INTERFACE_AP_MODE: ${INTERFACE_AP_MODE}"
             # log "INTERFACE_ACCESS_MODE: ${INTERFACE_ACCESS_MODE}"
             # log "INTERFACE_CONNECTION_STATE: ${INTERFACE_CONNECTION_STATE}"

             # log "HOSTAPD_ACTIVE: ${HOSTAPD_ACTIVE}"
             # log "UDHCPD_ACTIVE: ${UDHCPD_ACTIVE}"
             # log "TRACKING_DAEMON_ACTIVE: ${TRACKING_DAEMON_ACTIVE}"
             # log "WPA_SUPPLICANT_ACTIVE: ${WPA_SUPPLICANT_ACTIVE}"

            TRACKING_DAEMON_ACTIVE=`is_process_active WirelessTracking_d`
            INTERFACE_MONITOR_MODE=`is_interface_in_monitor_mode ${INTERFACE}`

            if [ ${TRACKING_DAEMON_ACTIVE} = "yes" -a ${INTERFACE_MONITOR_MODE} = "yes" ]
            then
                echo "MODE_WIRELESS_TRACKING"
                return 0
            fi

            INTERFACE_AP_MODE=`is_interface_in_ap_mode ${INTERFACE}`
            HOSTAPD_ACTIVE=`is_process_active hostapd`
            UDHCPD_ACTIVE=`is_process_active udhcpd`

            if [ ${HOSTAPD_ACTIVE} = "yes" -a ${UDHCPD_ACTIVE} = "yes" -a ${INTERFACE_AP_MODE} = "yes" ]
            then
                echo "MODE_ACCESS_POINT"
                return 0
            fi

            INTERFACE_ACCESS_MODE=`is_interface_in_wifi_access_mode ${INTERFACE}`
            INTERFACE_CONNECTION_STATE=`get_connection_state ${INTERFACE}`
            WPA_SUPPLICANT_ACTIVE=`is_process_active wpa_supplicant`

            if [ ${WPA_SUPPLICANT_ACTIVE} = "yes" -a ${INTERFACE_ACCESS_MODE} = "yes" -a ${INTERFACE_CONNECTION_STATE} = "up" ]
            then
                echo "MODE_WIFI_ACCESS"
                return 0
            fi
        fi
    fi

    # no mode matched??
    echo ""
    log "Error: could not determine current wireless services mode!"
    return 1
}
