#!/bin/sh

#################################################################################################
#   (c) 2016 by Hella Aglaia Mobile Vision GmbH
#   Author: Andrej Wunrau
#   No usage without written permission.
#   All rights reserved.
#
#   Module: WirelessServices
#   File: wifiModeAP.sh
#
#   Description:    
#
#       Script holds all functionality concerning the access point mode of wireless
#       services. Use this script to either start, restart or stop MODE_ACCESS_POINT.
#       When started device uses wifi interface to work as an access point. This mode allows
#       access to HMI via wifi connection.       
#
#################################################################################################


# import global variables and functions 
. wifiGlobalVariables.sh
. wifiGlobalFunctions.sh

. /etc/init.d/rc1.d/_HelperFunctions/XmlParameter

HMI_ACCESS_PARAMFILE_NAME="aps_wireless_hmi_access_params"

usage()
{
    log "Usage: $0 <wifi-interface> <start|stop|restart>"
}


modify_udhcp_config ()
{
    INTERFACE=$1
    IP=$2

    if [ $# -ne 2 ]
    then
        log "Error: wrong number of parameters"
        return 1
    else
        # start ip is interface ip + 1
        START_IP=`echo "${IP}" | awk -F '.' '{ print $1"."$2"."$3"."$4 + 1 }'`
     
        # end ip is start ip + 10
        END_IP=`echo "${START_IP}" | awk -F '.' '{ print $1"."$2"."$3"."$4 + '${GLOBAL_WIFI_DHCP_NO_OF_OFFERED_IPS}' }'`

        echo "start: ${START_IP}"
        echo "end: ${END_IP}"

        add_option_to_config_file ${GLOBAL_UDHCPD_CONF} "interface" " " ${INTERFACE}
        add_option_to_config_file ${GLOBAL_UDHCPD_CONF} "start" " " ${START_IP}
        add_option_to_config_file ${GLOBAL_UDHCPD_CONF} "end" " " ${END_IP}
    fi
    return 0
}

start_ap_mode ()
{
    INTERFACE=$1
    INTERFACE_IP=$2
    INTERFACE_NETMASK=$3

    if [ $# -ne 3 ]
    then
        log "Error: wrong number of parameters"
        return 1
    else
        # setup uhdcpd configuration
        modify_udhcp_config ${INTERFACE} ${INTERFACE_IP}

        ifconfig ${INTERFACE} up
        if [ $? -ne 0 ]
        then
            log "Error: setting interface up using ifconfig failed"
            return 2
        else
            ifconfig ${INTERFACE} ${INTERFACE_IP} netmask ${INTERFACE_NETMASK}
            if [ $? -ne 0 ]
            then
                log "Error: call to ifconfig failed, check interface, ip addr and netmask parameters"
                return 3
            else
                ${GLOBAL_HOSTAPD_PATH} -B ${GLOBAL_HOSTAPD_CONF}
                if [ $? -ne 0 ]
                then
                    log "Error: call to hostapd returned with error!"
                    return 4
                else
                    ${GLOBAL_UDHCPD_PATH} ${GLOBAL_UDHCPD_CONF}
                    if [ $? -ne 0 ]
                    then
                        log "Error: call to udhcpd returned with error!"
                        return 5
                    fi

                    # TODO: security mechanisms!!!
                fi
            fi
        fi
    fi
    return 0
}

stop_ap_mode ()
{
    INTERFACE=$1

    if [ $# -ne 1 ]
    then
        log "Error: wrong number of parameters"
        return 1
    else
        killall -q udhcpd
        killall -q hostapd

        ifconfig ${INTERFACE} down
    fi
    return 0
}

restart_ap_mode ()
{
    INTERFACE=$1
    INTERFACE_IP=$2
    INTERFACE_NETMASK=$3

    if [ $# -ne 3 ]
    then
        log "Error: wrong number of parameters"
        return 1
    else
        stop_ap_mode ${INTERFACE}
        if [ $? -ne 0 ]
        then
            log "Error: call to stop_ap_mode failed"
            return 2
        else
            sleep 1
            start_ap_mode ${INTERFACE} ${INTERFACE_IP} ${INTERFACE_NETMASK}
            if [ $? -ne 0 ]
            then
                log "Error: call to start_ap_mode failed"
                return 3
            fi
        fi
    fi
    return 0
}



#################################################################################################
# script starts here
#################################################################################################
if [ $# -ne 2 ]
then
    log "Error, wrong number of parameters"
    usage
    exit 1
else 
    # arguments
    WIFI_INTERFACE=$1
    ARGUMENT=$2

    # license check, only continue if license exists on device
    lic_available=`is_wifi_license_available`
    if [ ${lic_available} = "no" ]
    then
        log "Error, license for wireless services missing on device"
        exit 3
    fi

    # derive suitbale ip for wlan0 from eth0
    AP_IP_ADDR=`GetParamValue ${HMI_ACCESS_PARAMFILE_NAME} hmiAccessIp`
    AP_NETMASK=`GetParamValue ${HMI_ACCESS_PARAMFILE_NAME} hmiAccessNetmask`

    # start operation depending on argument
    if   [ ${ARGUMENT} = "start" ]
    then
        start_ap_mode ${WIFI_INTERFACE} ${AP_IP_ADDR} ${AP_NETMASK}
        START_RESULT=$?
        if [ ${START_RESULT} -ne 0 ]
        then
            log "Error: start_ap_mode failed with error code: ${START_RESULT}"
            #stop_ap_mode ${WIFI_INTERFACE}
            exit 4
        fi
    elif [ ${ARGUMENT} = "stop" ]
    then
        stop_ap_mode ${WIFI_INTERFACE}
        STOP_RESULT=$?
        if [ ${STOP_RESULT} -ne 0 ]
        then
            log "Error: start_ap_mode failed with error code: ${STOP_RESULT}"
            exit 5
        fi
    elif [ ${ARGUMENT} = "restart" ]
    then
        restart_ap_mode ${WIFI_INTERFACE} ${AP_IP_ADDR} ${AP_NETMASK}
        RESTART_RESULT=$?
        if [ ${RESTART_RESULT} -ne 0 ]
        then
            log "Error: restart_ap_mode failed with error code: ${RESTART_RESULT}"
            stop_ap_mode ${WIFI_INTERFACE}
            exit 6
        fi
    else
        # undefined argument used
        log "Error, undefined argument used."
        usage
        exit 2
    fi
fi

