#!/bin/sh

#################################################################################################
#   (c) 2016 by Hella Aglaia Mobile Vision GmbH
#   Author: Andrej Wunrau
#   No usage without written permission.
#   All rights reserved.
#
#   Module: WirelessServices
#   File: wifiModeAccess.sh
#
#   Description:    
#
#       Script holds all functionality concerning the wifi access mode of wireless
#       services. Use this script to either start, restart or stop MODE_WIFI_ACCESS.
#       When started wifi interface connects to parameterized ssid with
#       corresponding password. 
#
#################################################################################################


# import global variables and functions 
. wifiGlobalVariables.sh
. wifiGlobalFunctions.sh


usage()
{
    log "Usage: $0 <wifi-interface> <start|stop|restart>"
}


connect_to_access_point ()
{
    INTERFACE=$1

    if [ $# -ne 1 ]
    then
        log "Error: wrong number of parameters"
        return 1
    fi

    # connect to access point
    ${GLOBAL_WPA_SUPPLICANT_PATH} -B -Dnl80211 -i ${INTERFACE} -c ${GLOBAL_WIFI_ACCESS_CONF}
    if [ $? -ne 0 ]
    then
        log "Error: call to ${GLOBAL_WPA_SUPPLICANT_PATH} failed"
        return 2
    fi

    # loop until connection state is "up" or until timeout occures
    COUNT=0
    TIMEOUT=10
    while [ ${COUNT} -lt ${TIMEOUT} ]
    do
        echo "${COUNT}"
        sleep 1
        # get connection state
        conn_state=`get_connection_state ${INTERFACE}`
        if [ ${conn_state} == "up" ]
        then
            break
        fi
        COUNT=$((COUNT + 1))
    done

    # are we connected?
    conn_state=`get_connection_state ${INTERFACE}`
    if [ ${conn_state} != "up" ]    
    then
        log "Error: timeout (${TIMEOUT}s) while trying to establish a connection"
        return 3
    fi

    return 0
}


request_ip_via_dhcp ()
{
    INTERFACE=$1

    if [ $# -ne 1 ]
    then
        log "Error: wrong number of parameters"
        return 1
    fi

    udhcpc -i ${INTERFACE} bound 

    # loop until connection state is "up" or until timeout occures
    COUNT=0
    TIMEOUT=10
    SUCCESS=0
    while [ ${COUNT} -lt ${TIMEOUT} ]
    do
        echo "${COUNT}"
        sleep 1
        # check if interface has a valid ip
        ip=`get_interface_ip ${INTERFACE}`
        if [ ${ip} ]
        then
            SUCCESS=1
            break

        fi
        COUNT=$((COUNT + 1))
    done

    if [ ${SUCCESS} -eq 0 ]
    then
        log "Error: timeout (${TIMEOUT}s) while waiting for ip address via DHCP."
        return 2
    fi

    return 0
}


start_wifi_access_mode ()
{
    INTERFACE=$1

    if [ $# -ne 1 ]
    then
        log "Error: wrong number of parameters"
        return 1
    fi


    if [ -f ${GLOBAL_WIFI_ACCESS_CONF} ]
    then
        ifconfig ${INTERFACE} up
                # starting will fail if interface was in monitor mode. For this switch to auto
        iwconfig  ${INTERFACE} mode auto

        connect_to_access_point ${INTERFACE}
        CONN_RES=$?
        if [ ${CONN_RES} -ne 0 ]
        then
            log "Error: connect_to_access_point failed (error code: ${CONN_RES})."
            return 3
        fi

        request_ip_via_dhcp ${INTERFACE}
        DHCP_RES=$?
        if [ ${DHCP_RES} -ne 0 ]
        then
            log "Error: request_ip_via_dhcp failed (error code: ${DHCP_RES})."
            return 4
        fi

       # TODO:
        # set routing table
        # security mechnisms??

    else 
        log "Error: need configuration file for wpa_supplicant but file ${GLOBAL_WIFI_ACCESS_CONF} missing"
        return 5
    fi

    return 0
}

stop_wifi_access_mode ()
{
    INTERFACE=$1

    if [ $# -ne 1 ]
    then
        log "Error: wrong number of parameters"
        return 1
    else
        # stop wireless tracking daemon
        killall wpa_supplicant
        ifconfig ${INTERFACE} down
        #ip address flush dev ${INTERFACE}
        # reconfigure eth0
        /usr/local/sbin/StartStopNetworkServices
        # removed due to reboots on change from data access to hmi access mode
        # /usr/local/sbin/StartStopDhcp

    fi
    return 0
}

restart_wifi_access_mode ()
{
    INTERFACE=$1

    if [ $# -ne 1 ]
    then
        log "Error: wrong number of parameters"
        return 1
    else
        stop_wifi_access_mode ${INTERFACE}
        if [ $? -ne 0 ]
        then
            log "Error: call to stop_wifi_access_mode failed"
            return 2
        else
            sleep 1
            start_wifi_access_mode ${INTERFACE}
            if [ $? -ne 0 ]
            then
                log "Error: call to start_wifi_access_mode failed"
                return 3
            fi
        fi
    fi
}



#################################################################################################
# script starts here
#################################################################################################
if [ $# -ne 2 ]
then
    log "Error, wrong number of parameters"
    usage
    exit 1
else 
    # arguments
    WIFI_INTERFACE=$1
    ARGUMENT=$2

    # license check, only continue if license exists on device
    lic_available=`is_wifi_license_available`
    if [ ${lic_available} = "no" ]
    then
        log "Error, license for wireless services missing on device"
        exit 3
    fi
   
    # start operation depending on argument
    if   [ ${ARGUMENT} = "start" ]
    then
        start_wifi_access_mode ${WIFI_INTERFACE}
        START_RESULT=$?
        if [ ${START_RESULT} -ne 0 ]
        then
            log "Error: start_wifi_access_mode failed with error code: ${START_RESULT}"
            stop_wifi_access_mode ${WIFI_INTERFACE}
            exit 4
        fi
    elif [ ${ARGUMENT} = "stop" ]
    then
        stop_wifi_access_mode ${WIFI_INTERFACE}
        STOP_RESULT=$?
        if [ ${STOP_RESULT} -ne 0 ]
        then
            log "Error: stop_wifi_access_mode failed with error code: ${STOP_RESULT}"
            exit 5
        fi
    elif [ ${ARGUMENT} = "restart" ]
    then
        restart_wifi_access_mode
        RESTART_RESULT=$?
        if [ ${RESTART_RESULT} -ne 0 ]
        then
            log "Error: restart_wifi_access_mode failed with error code: ${RESTART_RESULT}"
            stop_wifi_access_mode ${WIFI_INTERFACE}
            exit 6
        fi
    else
        # undefined argument used
        log "Error, undefined argument used."
        usage
        exit 2
    fi
fi

