#!/bin/sh

#################################################################################################
#   (c) 2016 by Hella Aglaia Mobile Vision GmbH
#   Author: Andrej Wunrau
#   No usage without written permission.
#   All rights reserved.
#
#   Module: WirelessServices
#   File: wifiSetConfiguration.sh
#
#   Description:    
#
#       Script allows to write certain configuration values to config files used for
#       the different wireless service modes.
#
#################################################################################################


# import global variables and functions 
. wifiGlobalVariables.sh
. wifiGlobalFunctions.sh


usage()
{
    log "Usage: $0 <command> <param1> [param2]"
    log "possible commands:"
    log "       hostapd_psk <ssid> <passphrase>"
    log "       hostapd_channel <channel_num>" 
    log "       hostapd_hide_ssid <0/1>"
    log "       hostapd_use_mac_filter <0/1>"
    log "       wpa_conf <ssid> <passphrase>"
}


#function for generating a wpa_supplicant config -> from ssid and passphrase as input
#output: whole config file
write_config_file_for_wifi_access ()
{
    SSID=$1
    PW=$2

    # remove old config first
    rm -f ${GLOBAL_WIFI_ACCESS_CONF}

    # check arguments
    if [ $# -ne 2 ]
    then
        log "Error: wrong number of parameters"
        return 1
    fi

    # generate credentials
    # first do a test call for argument validation
    ${GLOBAL_WPA_PASSPHRASE_PATH} ${SSID} ${PW}
    TEST_RES=$?

    if [ ${TEST_RES} -ne 0 ]
    then
        log "Error: call to ${GLOBAL_WPA_PASSPHRASE_PATH} returned with error."
        return 1
    fi
    
    # input ok, now really generate wpa credentials ...
    WPA_CONFIG=`${GLOBAL_WPA_PASSPHRASE_PATH} ${SSID} ${PW} | sed "/#psk.*/d"`
      
    # ... write credentials to config file
    echo "${WPA_CONFIG}" > ${GLOBAL_WIFI_ACCESS_CONF}

    return 0
}



# function for generating a psk for AP mode -> from ssid and passphrase as input 
# output updated hostapd config file with new psk and ssid
write_credentials_to_hostapd_conf ()
{
    SSID=$1
    PW=$2

    # check arguments
    if [ $# -ne 2 ]
    then
        log "Error: wrong number of parameters"
        return 1
    fi

    # first clear old wpa_psk entry
    add_option_to_config_file ${GLOBAL_HOSTAPD_CONF} "wpa_psk" "=" ""

    PSK=`${GLOBAL_WPA_PASSPHRASE_PATH} ${SSID} ${PW} | grep psk=[^\"] | awk -F '=' ' { print $2}'`

    if [ -z ${PSK} ]
    then
        log "Error: call to generate_psk returned with empty key."
        return 3
    else
        # write to config file
        if [ -f ${GLOBAL_HOSTAPD_CONF} ]
        then
            add_option_to_config_file ${GLOBAL_HOSTAPD_CONF} "ssid" "=" ${SSID}
            if [ $? -ne 0 ]
            then
                log "Error: failed to add option ssid to config file ${GLOBAL_HOSTAPD_CONF}"
                return 4
            fi

            add_option_to_config_file ${GLOBAL_HOSTAPD_CONF} "wpa_psk" "=" ${PSK}
            if [ $? -ne 0 ]
            then
                log "Error: failed to add option ssid to config file ${GLOBAL_HOSTAPD_CONF}"
                return 5
            fi
        else
            log "Error: could not find configuration file ${GLOBAL_HOSTAPD_CONF}"
            return 2
        fi
    fi
    return 0
}

write_channel_to_hostapd_conf ()
{
    CHANNEL=$1
 
    # check arguments
    if [ $# -ne 1 ]
    then
        log "Error: wrong number of parameters"
        return 1
    fi

    # write to config file
    if [ -f ${GLOBAL_HOSTAPD_CONF} ]
    then
        add_option_to_config_file ${GLOBAL_HOSTAPD_CONF} "channel" "=" ${CHANNEL}
        if [ $? -ne 0 ]
        then
            log "Error: failed to add option channel to config file ${GLOBAL_HOSTAPD_CONF}"
            return 3
       fi
    else
        log "Error: could not find configuration file ${GLOBAL_HOSTAPD_CONF}"
        return 2
    fi
    return 0
}

write_hide_ssid_to_hostapd_conf ()
{
    HIDE_SSID=$1
 
    # check arguments
    if [ $# -ne 1 ]
    then
        log "Error: wrong number of parameters"
        return 1
    fi

    # write to config file
    if [ -f ${GLOBAL_HOSTAPD_CONF} ]
    then
        add_option_to_config_file ${GLOBAL_HOSTAPD_CONF} "ignore_broadcast_ssid" "=" ${HIDE_SSID}
        if [ $? -ne 0 ]
        then
            log "Error: failed to add option ignore_broadcast_ssid to config file ${GLOBAL_HOSTAPD_CONF}"
            return 3
        fi
    else
        log "Error: could not find configuration file ${GLOBAL_HOSTAPD_CONF}"
        return 2
    fi
    return 0
}

write_use_mac_filter_to_hostapd_conf ()
{
    USE_MAC_FILTER=$1
 
    # check arguments
    if [ $# -ne 1 ]
    then
        log "Error: wrong number of parameters"
        return 1
    fi

    # write to config file
    if [ -f ${GLOBAL_HOSTAPD_CONF} ]
    then
        add_option_to_config_file ${GLOBAL_HOSTAPD_CONF} "macaddr_acl" "=" ${USE_MAC_FILTER}
        if [ $? -ne 0 ]
        then
            log "Error: failed to add option macaddr_acl to config file ${GLOBAL_HOSTAPD_CONF}"
            return 3
        fi
    else
        log "Error: could not find configuration file ${GLOBAL_HOSTAPD_CONF}"
        return 2
    fi
    return 0
}


#################################################################################################
# script starts here
#################################################################################################

if [ $# -lt 2 ]
then
    log "Error, wrong number of parameters"
    usage
    exit 1
else 
    # arguments
    COMMAND=$1

    # license check, only continue if license exists on device
    lic_available=`is_wifi_license_available`
    if [ ${lic_available} = "no" ]
    then
        log "Error, license for wireless services missing on device"
        exit 2
    fi
   
    case ${COMMAND} in

        hostapd_psk)
            write_credentials_to_hostapd_conf $2 $3
            if [ $? -ne 0 ]
            then
                log "Error, call to function write_credentials_to_hostapd_conf failed!"
                exit 4
            fi
        ;;

        hostapd_channel)
            write_channel_to_hostapd_conf $2
            if [ $? -ne 0 ]
            then
                log "Error, call to function write_channel_to_hostapd_conf failed!"
                exit 5
            fi
        ;;

        hostapd_hide_ssid)
            write_hide_ssid_to_hostapd_conf $2
            if [ $? -ne 0 ]
            then
                log "Error, call to function write_hide_ssid_to_hostapd_conf failed!"
                exit 6
            fi
        ;;

        hostapd_use_mac_filter)
            write_use_mac_filter_to_hostapd_conf $2
            if [ $? -ne 0 ]
            then
                log "Error, call to function write_use_mac_filter_to_hostapd_conf failed!"
                exit 7
            fi
        ;;

        wpa_conf)
            write_config_file_for_wifi_access $2 $3
            if [ $? -ne 0 ]
            then
                log "Error, call to function write_config_file_for_wifi_access failed!"
                exit 8
            fi
        ;;

        *)
            log "Error, undefined command \"$1\" used"
            exit 3
        ;;
    esac

    exit 0
fi
