#!/bin/sh

#################################################################################################
#   (c) 2018 by Hella Aglaia Mobile Vision GmbH
#   Author: Andrej Wunrau
#   No usage without written permission.
#   All rights reserved.
#
#   Module: WirelessServices
#   File: wifiSetIPFilterRules.sh
#
#   Description:    
#
#       The script can be used to switch between set of iptable rules for the wireless services modes.
#
#################################################################################################


# import global variables and functions 
. wifiGlobalVariables.sh
. wifiGlobalFunctions.sh

usage()
{
    log "Usage: $0 <interface> <mode>"
    log "interface, e.g. wlan0"
    log "mode can be : MODE_OFF/MODE_WIRELESS_TRACKING/MODE_ACCESS_POINT/MODE_WIFI_ACCESS"
}



iptable_rules_clear()
{
    iptables --flush
}

iptable_rules_block_all()
{
    iptable_rules_clear
    iptables -A INPUT -i ${INTERFACE} -j DROP
}


iptable_set_rules_for_mode_access_point()
{
    iptable_rules_clear
    
    # dhcp
    iptables -A INPUT -i ${INTERFACE} -p udp --dport 67:68 --sport 67:68 -j ACCEPT
    # http
    iptables -A INPUT -i ${INTERFACE} -p tcp --dport 80 -j ACCEPT
    # https
    iptables -A INPUT -i ${INTERFACE} -p tcp --dport 443 -j ACCEPT

     # block all traffic that no rule exists for
    iptables -A INPUT -i ${INTERFACE} -j DROP

}

iptable_set_rules_for_mode_wifi_access()
{
    iptable_rules_clear

    # dhcp
    iptables -A INPUT -i ${INTERFACE} -p udp --dport 67:68 --sport 67:68 -j ACCEPT
    # http
    iptables -A INPUT -i ${INTERFACE} -p tcp --dport 80 -j ACCEPT
    iptables -A INPUT -i ${INTERFACE} -p tcp --sport 80 -j ACCEPT
    # https
    iptables -A INPUT -i ${INTERFACE} -p tcp --dport 443 -j ACCEPT
    iptables -A INPUT -i ${INTERFACE} -p tcp --sport 443 -j ACCEPT
    # ping
    iptables -A INPUT -i ${INTERFACE} -p icmp --icmp-type echo-reply -j ACCEPT
    # ftp
        # control
    iptables -A INPUT -i ${INTERFACE} -p tcp --sport 21 -m conntrack --ctstate ESTABLISHED,NEW -j ACCEPT
        # data
    iptables -A INPUT -i ${INTERFACE} -p tcp --sport 20 -m conntrack --ctstate ESTABLISHED,RELATED -j ACCEPT 

    # sftp
    iptables -A INPUT -i ${INTERFACE} -p tcp --sport 22 -j ACCEPT

    # ntpd (if daemon enabled)
    USE_NTP=`GetValueFromXmlFile -k UseNtp -v /var/Counter/config/network_services_params_values.xml -m /etc/Counter/ParamMetaFiles/network_services_params_meta.xml`
    if [ ${USE_NTP} -eq 1 ]
    then
        iptables -A INPUT -i ${INTERFACE} -p udp --sport 123 -j ACCEPT
    fi
 
    # snmpd (if daemon enabled)
    USE_SNMP=`GetValueFromXmlFile -k UseSnmp -v /var/Counter/config/network_services_params_values.xml -m /etc/Counter/ParamMetaFiles/network_services_params_meta.xml`
    if [ ${USE_SNMP} -eq 1 ]
    then
        iptables -A INPUT -i ${INTERFACE} -p udp --dport 161 -j ACCEPT
    fi

    # rest (if daemon enabled)
    USE_REST=`GetValueFromXmlFile -k UseRestApi -v /var/Counter/config/rest_api_params_values.xml -m /etc/Counter/ParamMetaFiles/rest_api_params_meta.xml`  
    if [ ${USE_REST} -eq 1 ]
    then
        REST_PORT=`GetValueFromXmlFile -k ServerPortNo -v /var/Counter/config/rest_api_params_values.xml -m /etc/Counter/ParamMetaFiles/rest_api_params_meta.xml`  
        iptables -A INPUT -i ${INTERFACE} -p tcp --dport ${REST_PORT} -j ACCEPT
    fi

    # IOT
    #iptables -A INPUT -i ${INTERFACE} -p tcp --dport 8883 -j ACCEPT
    #iptables -A INPUT -i ${INTERFACE} -p tcp --dport 1883 -j ACCEPT

    # DNS
    iptables -A INPUT -i ${INTERFACE} -p udp --sport 53 -j ACCEPT
    iptables -A INPUT -i ${INTERFACE} -p tcp --sport 53 -j ACCEPT

    # Ayla UDP
    iptables -A INPUT -i ${INTERFACE} -p udp --sport 55055 -j ACCEPT
    iptables -A INPUT -i ${INTERFACE} -p udp --sport 55056 -j ACCEPT
    
    # MsgBrokerProxy
    iptables -A INPUT -i ${INTERFACE} -p tcp --dport 10080 -j ACCEPT
    iptables -A INPUT -i ${INTERFACE} -p tcp --sport 10080 -j ACCEPT

    # TODO: RAS
    # !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

    # allow all incoming traffic on established connections (ports > 1024) (soap, ftp passive mode, ...)
    iptables -A INPUT -i ${INTERFACE} -p tcp --sport 1024: --dport 1024: -m conntrack --ctstate ESTABLISHED -j ACCEPT

    # block all incomming traffic that no rule exists for
    iptables -A INPUT -i ${INTERFACE} -j DROP
    
    # allow all outgoing traffic that no rule exists for
    iptables -A OUTPUT -o ${INTERFACE} -j ACCEPT

}

#################################################################################################
# script starts here
#################################################################################################

# arguments
INTERFACE=$1
MODE=$2

# check arguments
if [ $# -ne 2 ]
then
    log "Error, $0 wrong number of parameters!"
    usage
    exit 1
fi


# initialize iptable rules: block all traffic
    iptable_rules_block_all


# for specific modes enable a set of selected iptable rules
case ${MODE} in
    MODE_OFF)
        # leave traffic blocked
    ;;

    MODE_WIRELESS_TRACKING)
        # leave traffic blocked
    ;;

    MODE_ACCESS_POINT)
        iptable_set_rules_for_mode_access_point
    ;;

    MODE_WIFI_ACCESS)
        iptable_set_rules_for_mode_wifi_access
    ;;

    *)
        log "Error, undefined mode: \"${MODE}\""
        exit 2
    ;;
esac

# all ok
notice "Info, new iptable rules set for mode: ${MODE}."
exit 0
