#!/bin/sh

#################################################################################################
#   (c) 2016 by Hella Aglaia Mobile Vision GmbH
#   Author: Andrej Wunrau
#   No usage without written permission.
#   All rights reserved.
#
#   Module: WirelessServices
#   File: wifiSetMode.sh
#
#   Description:    
#
#       The script can be used to switch between the wireless services modes.
#       The following modes are defined: 
#           - MODE_OFF
#           - MODE_WIRELESS_TRACKING
#           - MODE_ACCESS_POINT
#           - MODE_WIFI_ACCESS
#
#################################################################################################


# import global variables and functions 
. wifiGlobalVariables.sh
. wifiGlobalFunctions.sh


usage()
{
    log "Usage: $0 <interface> <new_mode> [<old_mode>]"
    log "mode can be : MODE_OFF/MODE_WIRELESS_TRACKING/MODE_ACCESS_POINT/MODE_WIFI_ACCESS"
    log "old_mode is optional, if not provided script will take old_mode from param files"
}

stop_all_modes ()
{
    INTERFACE=$1
    if [ $# -ne 1 ]
    then
        log "Error, wrong number of parameters"
        return 1
    fi

    log "Warning, stopping all wireless modes."
    ${GLOBAL_MODE_TRACKING_SCRIPT_NAME} ${INTERFACE} stop
    ${GLOBAL_MODE_ACCESS_POINT_SCRIPT_NAME} ${INTERFACE} stop
    ${GLOBAL_MODE_WIFI_ACCESS_SCRIPT_NAME} ${INTERFACE} stop
}

return_mode_script_name_from_mode ()
{
    MODE=$1

    # check arguments
    if [ $# -ne 1 ]
    then
        log "Error, wrong number of parameters"
        return 1
    fi

    case ${MODE} in

        MODE_OFF)
            echo ""
        ;;

        MODE_WIRELESS_TRACKING)
            echo "${GLOBAL_MODE_TRACKING_SCRIPT_NAME}"
        ;;

        MODE_ACCESS_POINT)
            echo "${GLOBAL_MODE_ACCESS_POINT_SCRIPT_NAME}"
        ;;

        MODE_WIFI_ACCESS)
            echo "${GLOBAL_MODE_WIFI_ACCESS_SCRIPT_NAME}"
        ;;

        *)
            log "Error, undefined mode: \"${MODE}\""
            echo ""
            return 2
        ;;
    esac

    return 0
}

get_wireless_mode_from_param_file ()
{
    MODE=`GetValueFromXmlFile -k Mode -v ${GLOBAL_WIRELESS_SERVICES_PARAM_VALUE_FILE} -m ${GLOBAL_WIRELESS_SERVICES_PARAM_META_FILE}`
    if [ $? -ne 0 ]
    then
        log "Error, call to GetValueFromXmlFile failed."
        echo ""
        return 1
    fi

    echo ${MODE}
    return 0
}

get_regulatory_domain_from_param_file ()
{
    REG_DOMAIN=`GetValueFromXmlFile -k RegulatoryDomain -v ${GLOBAL_WIRELESS_SERVICES_PARAM_VALUE_FILE} -m ${GLOBAL_WIRELESS_SERVICES_PARAM_META_FILE}`
    if [ $? -ne 0 ]
    then
        log "Error, call to GetValueFromXmlFile failed."
        echo ""
        return 1
    fi

    echo ${REG_DOMAIN}
    return 0
}

validate_mode ()
{
    MODE=$1

    # check arguments
    if [ $# -ne 1 ]
    then
        log "Error, wrong number of parameters"
        return 1
    fi

    case ${MODE} in

        MODE_OFF|MODE_WIRELESS_TRACKING|MODE_ACCESS_POINT|MODE_WIFI_ACCESS)
            return 0
        ;;

        *)
            log "Error, undefined mode: \"${MODE}\""
            return 1
        ;;
    esac

    return 0
}


#################################################################################################
# script starts here
#################################################################################################

# arguments
INTERFACE=$1
NEW_MODE=$2
CURR_MODE=$3

# check arguments
if [ $# -lt 2 -o $# -gt 3 ]
then
    log "Error, $0 wrong number of parameters!"
    usage
    exit 1
fi

# check new mode argument
validate_mode ${NEW_MODE}
if [ $? -ne 0 ]
then
    log "Error, invalid argument ${NEW_MODE}"
    usage
    exit 1
fi

# check license
lic_available=`is_wifi_license_available`
if [ ${lic_available} = "no" ]
then
    log "Error, license for wireless services missing on device"
    exit 2
fi

# check wifi device existence
dev_state=`is_wifi_device_available ${INTERFACE}`
if [ $dev_state != "up" ]
then
    log "Error, no wifi device present in system!"
    exit 3
fi

 
# check regulatory domain -> set if current is different from param file
PARAM_REG_DOMAIN=`get_regulatory_domain_from_param_file`
CURR_REG_DOMAIN=`get_interface_regulatory_domain`
if [ -z ${CURR_REG_DOMAIN} ]
then
    log "Error, failed to read regulatory domain."
    exit 4
else
    if [ ${CURR_REG_DOMAIN} != ${PARAM_REG_DOMAIN} ]
    then
        iw reg set ${PARAM_REG_DOMAIN}
    fi
fi

# if current mode is not provided -> derive from param files
if [ -z ${CURR_MODE} ]
then
    CURR_MODE=`get_wireless_mode_from_param_file`
fi

# check current mode
validate_mode ${CURR_MODE}
if [ $? -ne 0 ]
then
    log "Warning, invalid mode \"${CURR_MODE}\" found in parameter file. "
    # the current mode is invalid, something is fishy
    # set current mode empty string, this will trigger error handling
    CURR_MODE=""
fi


# stop all modes if current mode is invalid
if [ -z ${CURR_MODE} ]
then
    stop_all_modes ${INTERFACE}
else 
    # script to use for turning off current mode
    CURR_MODE_SCRIPT_NAME=`return_mode_script_name_from_mode $CURR_MODE`
    if [ $? -ne 0 ]
    then
        log "Error, call to return_mode_script_name_from_mode failed!"
        exit 6
    fi
    # first disable old mode (if not off or unavailable)
    if [ ! -z ${CURR_MODE_SCRIPT_NAME} ]
    then
        ${CURR_MODE_SCRIPT_NAME} ${INTERFACE} stop
    fi
fi

# script to use for turning on new mode
NEW_MODE_SCRIPT_NAME=`return_mode_script_name_from_mode $NEW_MODE`
if [ $? -ne 0 ]
then
    log "Error, call to return_mode_script_name_from_mode failed!"
    exit 7
fi

# enable new mode
if [ ! -z ${NEW_MODE_SCRIPT_NAME} ]
then
    ${NEW_MODE_SCRIPT_NAME} ${INTERFACE} start
    ERR_CODE=$?
    if [ ${ERR_CODE} -ne 0 ]
    then
        log "Error, failed to enable new $NEW_MODE, ${NEW_MODE_SCRIPT_NAME} returned with error code ${ERR_CODE}"
        exit 8
    fi
fi

# set iptable rules corresponding to new mode
${GLOBAL_IP_FILTER_CONFIG_SCRIPT_NAME} ${INTERFACE} $NEW_MODE
if [ $? -ne 0 ]
then
    log "Error, call to ${GLOBAL_IP_FILTER_CONFIG_SCRIPT_NAME} failed!"
    exit 9
fi

# all ok
notice "Info, new wireless mode $NEW_MODE"
exit 0
