#!/usr/local/bin/python3
# -*- coding: utf8 -*-

##########################################################################
#
#    name:          zeroconfDiscover.py
#    author:        Andrej Wunrau (awu) 2018
#
#    description:   Script starts service discovery and creates a list
#                   of found services.
#
#                   Output depends on chosen parameters:
#                       - output of full list of discovered services
#                         (ip and hostname)
#                       - a hostname matching a specified ip
#                       - an ip matching a specified hostname
#
#                   Script also returns an error code:
#                       - 0 if no errors occurred
#                       - <0 on error (wrong params)
#                       - >0 if hostname/ip could not be found
#
##########################################################################

import argparse
import sys
import time
import socket
import threading
import logging
from zeroconf import ServiceBrowser, Zeroconf, ServiceStateChange


#############################################################################
# global vars
#############################################################################
MAX_DISCOVERY_DURATION          = 60
MAX_TIME_SINCE_LAST_DISCOVERY   = 30

g_debugMode                 = False
g_logger                    = logging.getLogger("iot_d")
g_Args                      = None
g_zeroconf                  = Zeroconf()
g_timeSinceLastDiscovery    = 0
g_timeoutLock               = threading.Lock()
g_discoveredServices        = {"aps_devices":[]}


#############################################################################
# discovery
#############################################################################

def handleDiscoveredService(zeroconf, service_type, name, state_change):
    global g_timeSinceLastDiscovery, g_timeoutLock, g_discoveredServices

    if state_change is ServiceStateChange.Added:
        serviceInfo = zeroconf.get_service_info(service_type, name)
        if serviceInfo:
            # new dev found: reset timeout
            with g_timeoutLock:
                g_timeSinceLastDiscovery = 0

            service             = {}
            service["ip"]       = socket.inet_ntoa(serviceInfo.address)
            service["hostname"] = serviceInfo.server.rstrip(".local.")

            g_discoveredServices["aps_devices"].append(service)


#############################################################################
# printing and searching
#############################################################################
def printFoundServices():
    for service in g_discoveredServices["aps_devices"]:
        print(" devIP: {0} \t devHostname: {1}".format(service["ip"], service["hostname"]))


def getHostnameFromIp(i_ip):
    for service in g_discoveredServices["aps_devices"]:
        if (service["ip"] == i_ip):
            return service["hostname"]
    # nothing found
    return None


def getIpFormHostname(i_hostname):
    for service in g_discoveredServices["aps_devices"]:
        if (service["hostname"] == i_hostname):
            return service["ip"]
    # nothing found
    return None


#############################################################################
# args
#############################################################################
def handleArguments():
    global g_Args
    # setup parser
    parser = argparse.ArgumentParser(description='zeroconfDiscover: discover services, return IPs for searched hostnames and the other way round')
    parser.add_argument('-v','--verbose', action='store_true', help='verbose output')
    parser.add_argument('--printAll', dest='all', action='store_true', help='pure discover mode, print list of all found services')
    parser.add_argument('--ip', dest='ip', help='return hostname matching this ip')
    parser.add_argument('--hostname', dest='hostname', help='return ip matching the hostname')

    # parse
    g_Args = parser.parse_args()

    if (g_Args.all == False and g_Args.ip == None and g_Args.hostname == None):
        g_logger.error("Error: provide at least one of the three arguments (all/ip/hostname)")
        parser.print_help()
        return False
    return True


#############################################################################
# main
#############################################################################
def main():
    global g_timeSinceLastDiscovery

    g_timeSinceLastDiscovery = 0
    discoveryDuration = 0

    if (False == handleArguments()):
        sys.exit(-1)

    #########################################################################
    # discovery
    browser = ServiceBrowser(g_zeroconf,"_aps._tcp.local.", handlers=[handleDiscoveredService])

    while ( (g_timeSinceLastDiscovery < MAX_TIME_SINCE_LAST_DISCOVERY)
       and  (discoveryDuration < MAX_DISCOVERY_DURATION) ):
        # increase timeout value which is decreased by function handleDiscoveredService,
        # so timeout will occure if handleDiscoveredService is not called for 5 secs
        with g_timeoutLock:
            g_timeSinceLastDiscovery = g_timeSinceLastDiscovery + 1
        time.sleep(1)
        discoveryDuration = discoveryDuration + 1

    # stop discovery
    browser.cancel()


    #########################################################################
    # search/print according to params
    if (g_Args.ip != None):
        # search for IP and return matching hostname
        hostname = getHostnameFromIp(g_Args.ip)
        if (hostname != None ):
            print("{0}".format(hostname))
        else:
            # ip not found, return error code
            sys.exit(1)
    elif (g_Args.hostname != None):
        # search for hostname and return matching IP
        ip = getIpFormHostname(g_Args.hostname)
        if (ip != None ):
            print("{0}".format(ip))
        else:
            # hostname not found, return error code
            sys.exit(2)

    if (g_Args.all == True):
        # print a list of all discovered services
        printFoundServices()


    sys.exit(0)


if __name__ == "__main__":
    main()


