#! /usr/local/bin/python3
# -*- coding: utf8 -*-

import logging
import json
import os
import datetime
import jwt
import hashlib
from time import time, localtime
from copy import deepcopy
#from datetime import timedelta
from tornado.web import RequestHandler
from tornado.ioloop import IOLoop
from tornado.httpserver import HTTPServer
from tornado.web import Application

# globals for authentication process
g_AuthIsBlocked   = False
g_AuthAttemptsCnt = 0
SECRET_KEY = hashlib.md5(str(time()).encode()).hexdigest()
USE_HTTPS = True
PORT      = 444 if USE_HTTPS else 88
JWT_EXPIRATION_DELTA   = datetime.timedelta(seconds=3600)
LOG_FILE_MAX_BYTES = 1048576   # max 1MB per file, if logBackupCount > 0 is set
DEFAULT_USER = "user"
DEFAULT_PW = "password"

logger = logging.getLogger("RESTPUSH Server")

class User():
    """
        Representation of an authenticated user formally you can use a lot of them
    """
    def __init__(self, username, password):
        self.id             = 1
        self.username       = username
        self.password       = password
    def verify_password(self, password):
        """ Verifies the given password for this user """
        return (password == self.password)
    def verify_username(self, username):
        return (username == self.username)
    def check_id(self, id):
        return (id == self.id)
    def __repr__(self):
        return '<User {username}>'.format(username=self.username)

class WebToken(object):
    # options used for decode/verification
    options = { "require_exp" : True,       # expiration time is required
                "require_iat" : True,       # issued at  time is required
                "require_nbf" : True        # not before time is required
              }

    def __init__(self, token):
        try:
            if not isinstance(token, str):
                self.token = token.decode()
            else:
                self.token = token
            self.payload = self._decode()
            self.user_id = int(self.payload['identity'])
            self.exp     = int(self.payload['exp'])
            self.iat     = int(self.payload['iat'])
            self.nbf     = int(self.payload['nbf'])

        except Exception as e:
            logger.error("Exception: {}".format(e))
            raise Exception("WebToken is not valid.")

    @classmethod
    # return: WebToken instance or None
    def create(cls, uid, expirationDelta):
        now = int(datetime.datetime.now().timestamp())
        exp = now + int(expirationDelta.total_seconds())
        payload = { "identity": uid,
                    "iat"     : now,
                    "exp"     : exp,
                    "nbf"     : now }
        try:
            token = jwt.encode(payload, SECRET_KEY)
            # create webtoken instance
            webtoken = WebToken(token)
            logger.debug("WebToken created: user_id: {0}, expTime: {1}, token: {2}"
                         .format(uid, exp , token))
            return webtoken
        except Exception as e:
            logger.error("WebToken creation failed! {}".format(e))
            return None

    @classmethod
    # return webtoken as string or None
    # token creation with custom user_id, expiration time, issued at time, not before time, secret key
    def create_custom(cls, uid, exp=None, iat=None, nbf=None, secret=None):
        try:
            now = int(datetime.datetime.now().timestamp())
            exp = int(exp) if exp is not None else now
            iat = int(iat) if iat is not None else now
            nbf = int(nbf) if nbf is not None else now
        except Exception as e:
            logger.error("Failed to create webtoken due to invalid parameter. {}".format(e))
            return None
        if secret is None:
            secret = SECRET_KEY

        payload = { "identity": uid,
                    "iat"     : iat,
                    "exp"     : exp,
                    "nbf"     : nbf }
        try:
            token = jwt.encode(payload, secret)
            return token.decode()
        except Exception as e:
            logger.error("WebToken creation failed! {}".format(e))
            return None

    def _decode(self, verify=True):
        try:
            return jwt.decode(self.token, SECRET_KEY, verify=verify, options=WebToken.options)
        except Exception as e:
            logger.error("Exception: {}".format(e))
            raise Exception("Decode error.")

    def verify(self):
        try:
            payload = self._decode()
            return True
        except Exception as e:
            logger.warning("WebToken is not valid. {}".format(e))
            return False

    def getLocalExpireTime(self):
        return localtime(self.exp)

    def getExpireTime(self):
        return self.exp

    def getRemainingTimeSec(self):
        #return int(self.exp - time() + 0.5)    # TODO ?
        return int(self.exp - time())

class AuthorizationError(Exception):
    def __init__(self, status_code=None, error_message=None):
        self.status_code = status_code if status_code else 403         # default error code: 403 - Forbidden
        self.error_message = error_message
        
    def __str__(self):
        return "AuthorizationError ({}){}".format(self.status_code, ": {}".format(self.error_message) if self.error_message else "")

"""
    Decorator for route handler classes, which require permissions
    - verifies webtoken
"""
def require_permission():
    def _require_permission(handler_class):
        def wrap_execute(handler_execute):
            def require_auth(handler, kwargs):
                try:
                    # Get 'Authorization'-header
                    auth = handler.request.headers.get('Authorization')
                    if not auth:
                        # status_code: 401 - Unauthorized
                        raise AuthorizationError(status_code=401, error_message="Missing authorization in request header.")
                    # Get parts of 'Authorization'-header
                    parts = auth.split()
                    if len(parts) != 2:
                        raise AuthorizationError(status_code=401, error_message="Invalid header authorization. Number of parts {} != 2.".format(len(parts)))
                    # Check 'Authorization'-header prefix ('Bearer')
                    authPref = "Bearer"
                    if parts[0] != authPref:
                        raise AuthorizationError(status_code=401, error_message="Missing auth prefix '{}' in request header.".format(authPref))
                    # Check validity of webtoken
                    try:
                        webtoken = WebToken(parts[1])
                    except Exception as e:
                        raise AuthorizationError(status_code=401, error_message="Invalid authorization. Webtoken is not valid.")
                    return True

                except AuthorizationError as autherr:
                    logger.error("{}".format(autherr))
                    handler._transforms = []
                    handler.set_status(autherr.status_code)
                    if autherr.error_message:
                        handler.write(autherr.error_message)
                    handler.finish()

                except Exception as e:
                    errMsg = "Authorization error: {}".format(e)
                    logger.error("{}".format(errMsg))
                    handler._transforms = []
                    handler.set_status(401)
                    handler.write(errMsg)
                    handler.finish()
                return True

            def _execute(self, transforms, *args, **kwargs):
                try:
                    require_auth(self, kwargs)
                except Exception:
                    return False

                return handler_execute(self, transforms, *args, **kwargs)

            return _execute
        handler_class._execute = wrap_execute(handler_class._execute)
        return handler_class
    return _require_permission

class MainHandler(RequestHandler):
    def get(self):
        self.render("rest_index.html")

"""
    Login handler used for route '/auth'
    - verifies credentials and creates webtoken
"""
class AuthHandler(RequestHandler):
    def post(self):
        try:
            data = json.loads(self.request.body.decode())
            username = data.get('username', '')
            password = data.get('password', '')
            user = User(DEFAULT_USER,DEFAULT_PW)
            pw_verified = False
            if user is not None:
                pw_verified = user.verify_password(password)
                    # user exist and is authenticated:
            if user is not None and pw_verified:
                logger.info("'{}' successfully authenticated".format(username))
                # create webtoken and send to client
                webtoken = WebToken.create(user.id, JWT_EXPIRATION_DELTA)
                self.write({'access_token': webtoken.token,})
                self.finish()
            else:
                raise AuthorizationError(status_code=401, error_message="Bad Request. Invalid credentials. User '{}' not authenticated."
                                            .format(username))
        except AuthorizationError as autherr:
            logger.error("{}".format(autherr.error_message))
            self.set_status(autherr.status_code)
            if autherr.error_message:
                self.write(autherr.error_message)
            self.finish()

        except Exception as e:
            errMsg = "Bad Request. Invalid credentials. {}".format(e)
            logger.error("{}".format(errMsg))
            self.set_status(401)
            self.write(errMsg)
            self.finish()

@require_permission()
class DeviceStatHandler(RequestHandler):
    def put(self,device):
        data = json.loads(self.request.body)
        logger.info("Got device status from device {0} with payload {1}.".format(device, data))

@require_permission()
class CountsHandler(RequestHandler):
    def put(self,device):
        data = json.loads(self.request.body)
        logger.info("Got counting data from device {0} with payload {1}.".format(device, data))

@require_permission()
class ZoneStatHandler(RequestHandler):
    def put(self,device):
        data = json.loads(self.request.body)
        logger.info("Got zone statistics from device {0} with payload {1}.".format(device, data))

@require_permission()
class ZoneObjListHandler(RequestHandler):
    def put(self,device):
        data = json.loads(self.request.body)
        logger.info("Got zone object list from device {0} with payload {1}.".format(device, data))

@require_permission()
class ZoneEventHandler(RequestHandler):
    def put(self,device):
        data = json.loads(self.request.body)
        logger.info("Got zone event from device {0} with payload {1}.".format(device, data))

@require_permission()
class CountEventHandler(RequestHandler):
    def put(self,device):
        data = json.loads(self.request.body)
        logger.info("Got count event from device {0} with payload {1}.".format(device, data))

@require_permission()
class ZoneAlertHandler(RequestHandler):
    def put(self,device):
        data = json.loads(self.request.body)
        logger.info("Got zone alert from device {0} with payload {1}.".format(device, data))

@require_permission()
class TrackingResultHandler(RequestHandler):
    def put(self,device):
        data = json.loads(self.request.body)
        logger.info("Got wireless logging state from device {0} with payload {1}.".format(device, data))

def configureLogging(logLevel=None, logFile=None, logBackupCount=0):
    if logLevel is None:                        # no logging
        logging.Logger.disabled = True
        return
    globLogger = logging.getLogger()
    if logFile is None:                         # log to console
        logHandler = logging.StreamHandler()
    else:                                       # log to file
        # rotate file if backupCount > 0 and size exceeds 1MB
        logHandler = RotatingFileHandler(logFile, maxBytes=LOG_FILE_MAX_BYTES, backupCount=logBackupCount)
    logHandler.setFormatter(logging.Formatter('%(asctime)s %(name)-18s %(levelname)-8s %(message)s'))
    globLogger.addHandler(logHandler)
    globLogger.setLevel(logLevel)

def start():
    logger.info("Start server on port {}".format(PORT))
    # haglpylibs.daemon.daemonize()
    IOLoop.instance().start()

def init():
    handlers = [
        (r'/',MainHandler),
        (r'/auth',AuthHandler),
        (r'/apiv1/([^/]+)/systemData/deviceStatus', DeviceStatHandler),
        (r'/apiv1/([^/]+)/sensorData/countingData', CountsHandler),
        (r'/apiv1/([^/]+)/sensorData/zoneStatistics', ZoneStatHandler),
        (r'/apiv1/([^/]+)/sensorData/zoneObjectList', ZoneObjListHandler),
        (r'/apiv1/([^/]+)/sensorData/zoneEvent', ZoneEventHandler),
        (r'/apiv1/([^/]+)/sensorData/countEvent', CountEventHandler),
        (r'/apiv1/([^/]+)/sensorData/zoneAlert', ZoneAlertHandler),
        (r'/apiv1/([^/]+)/sensorData/trackingResult', TrackingResultHandler)
    ]
    settings = {
        'debug': True,
    }
    ssl_options = { "certfile" : "/etc/ssl/certs/certificate.crt",
                    "keyfile"  : "/etc/ssl/certs/privateKey.key"   } if USE_HTTPS else None
    # initialize server and bind port
    application = Application(handlers, **settings)
    server = HTTPServer(application, ssl_options=ssl_options)
    server.listen(PORT)

if __name__ == '__main__':
    configureLogging(logging.INFO)
    init()
    start()

