#!/usr/local/bin/python3
# -*- coding: utf-8 -*-
##########################################################################
#
#    name:           VpnAutoConnection_d.py
#    author:         Tilo Gericke (tig)
#    description:    a daemon, that cyclically checks a flag on a server
#                    to see if a VPN connection should be established,
#                    if so, then the connection is established
#
##########################################################################

import sys
import os
import socket
import re
import syslog
import select
from datetime import datetime, timedelta
import struct
import subprocess
import signal
import pycurl
from io import BytesIO

import msgBrokerExt
from messages import ids


from ParamHandling.parameters import Parameters


restPath           = "https://{serverAddr}:{serverPort}{serverBasePath}/{hostname}/{command}"
restCmd_GetFlag    = "OpenVPNConnection"
restCmd_GetConfig  = "GetDeviceConfig"
restCommands       = { restCmd_GetFlag  : { 'method': 'GET'  },
                       restCmd_GetConfig: { 'method': 'POST' }
                     }
ovpnPath           = "/CustomData/openvpn"
ovpnConfigFile     = "/etc/openvpn/client.conf"
server_certificate = "/var/Counter/ssl/vpncontrol.crt"
ovpnLogFile        = "/tmp/openvpn.log"

ovpnTimeoutSec     = 3600     # close VPN connection (and exit openvpn daemon) after this time of inactivity

paramChanged       = False    # global flag to notice a parameter change

printDebugInfo     = False     # if True, print debugging info to console


"""
    ParameterError - exception if any error occurs when reading a required parameter
    -----------------------------------------------------------------------------------------------
"""
class ParameterError(Exception):
    def __init__(self, message):
        super(ParameterError, self).__init__(message)
        syslog.syslog(syslog.LOG_ERR, message)

"""
    CurlError - exception if any cURL connection error occurs
    -----------------------------------------------------------------------------------------------
"""
class CurlError(Exception):
    def __init__(self, message):
        super(CurlError, self).__init__(message)
        syslog.syslog(syslog.LOG_ERR, message)

"""
    OvpnError - exception if an OpenVPN error occurs
    -----------------------------------------------------------------------------------------------
"""
class OvpnError(Exception):
    def __init__(self, message):
        super(OvpnError, self).__init__(message)
        syslog.syslog(syslog.LOG_ERR, message)


"""
    LogFilter - avoid frequently logged errors/warnings
    -----------------------------------------------------------------------------------------------
"""
class LogFilter:
    CURL_CONN_ERR     = 'CURL_CONN_ERR'
    MISSING_CERT_WARN = 'MISSING_CERT_WARN'
    MISSING_CONF_WARN = 'MISSING_CONF_WARN'
    NO_FLAG_WARN      = 'NO_FLAG_WARN'
    GET_CONFIG_WARN   = 'GET_CONFIG_WARN'

    def __init__(self):
        self.alreadyLogged = { LogFilter.CURL_CONN_ERR     : { 'count' : 0, 'firstTs' : None, 'lastTs' : None },
                               LogFilter.MISSING_CERT_WARN : { 'count' : 0, 'firstTs' : None, 'lastTs' : None },
                               LogFilter.MISSING_CONF_WARN : { 'count' : 0, 'firstTs' : None, 'lastTs' : None },
                               LogFilter.NO_FLAG_WARN      : { 'count' : 0, 'firstTs' : None, 'lastTs' : None },
                               LogFilter.GET_CONFIG_WARN   : { 'count' : 0, 'firstTs' : None, 'lastTs' : None }
                             }

    def syslog(self, logLevel, type, message):
        # TODO: log every x times with time span
        if self.alreadyLogged[type]['count'] == 0:
            syslog.syslog(logLevel, message)
            self.alreadyLogged[type]['firstTs'] = datetime.now()
        self.alreadyLogged[type]['count'] += 1
        self.alreadyLogged[type]['lastTs'] = datetime.now()

    def reset(self, type):
        self.alreadyLogged[type] = { 'count' : 0, 'firstTs' : None, 'lastTs' : None }


"""
    Configuration class
    -----------------------------------------------------------------------------------------------
"""
class VpnAutoConnectionConfig:

    def __init__(self):
        self._parameters = Parameters()
        self.msgBrokerFD = -1
        self.params = {}
        self.hostname = socket.gethostname()
        self.cycleStart = datetime.now()
        self.selectTimeout = None       # timeout None: select will block without timeout
        self.logFilter = LogFilter()

    # raises ParameterError on failure
    def updateParams(self):
        params = self._parameters.get(Parameters.PARAMS_VPN_AUTO_CONNECTION, '/')
        if not params:
            raise ParameterError("Failed to get VPN Auto Connection Parameters.")
        else:
            # flatten param dict { paramName: value, ...}
            for k in params:
                self.params[k] = params[k]['value']
            PrintLog("Vpn auto connection parameters updated")

    def updateHostname(self):
        self.hostname = socket.gethostname()
        return self.hostname

    # raises ParameterError on failure
    def getParamValue(self, paramName):
        if not paramName in self.params:
            raise ParameterError("Required parameter '{}' not found in VPN Auto Connection Parameters.".format(paramName))
        return self.params[paramName]

    # returns True, if feature is activated, otherwise False
    # raises ParameterError on failure
    def isActive(self):
        try:
            active = self.getParamValue('EnableVpn')
        except ParameterError:
            raise
        if active == 1:
            return True
        else:
            return False

    """
        Determine, if the control server is used.
        -----------------------------------------------------------------------------------------------
    """
    # Is the control server used to acquire the configuration file and the server flag
    def useControlServer(self):
        try:
            useControlServer = self.getParamValue("UseControlServer")
        except ParameterError:
            raise
        return useControlServer != 0

    # raises ParameterError on failure
    def updateTimeout(self):
        try:
            isActive = self.isActive()
        except ParameterError:
            raise
        if not isActive:
            self.selectTimeout = None   # select will block without timeout
        elif self.selectTimeout is None:
            # initially trigger immediately
            self.cycleStart = datetime.now()
            self.selectTimeout = 0.0
        else:
            try:
                useControlServer = self.getParamValue('UseControlServer')
                if useControlServer == 0:
                    # If we do not use the control server we would like to try to connect every minute
                    cycleTimeMinutes = 1
                else:
                    cycleTimeMinutes = self.getParamValue('FlagCheckCycleTime_minutes')
            except ParameterError:
                raise
            cycleTime = timedelta(minutes = cycleTimeMinutes)
            elapsed = datetime.now() - self.cycleStart
            remainingSec = (cycleTime-elapsed).total_seconds()
            if remainingSec < -1.0:
                # time missed after parameter change (cycle time has decreased), so trigger now
                self.cycleStart = datetime.now()
                self.selectTimeout = 0.0
            elif remainingSec <= 0.0:
                # triggered recently, set new interval
                self.cycleStart = datetime.now()
                self.selectTimeout = cycleTime.total_seconds()
            else:
                self.selectTimeout = (cycleTime-elapsed).total_seconds()


"""
    Message Broker related functions - to notice changes to parameters
    -----------------------------------------------------------------------------------------------
"""
def MsgBrokerConnect():
    fd = msgBrokerExt.connect(__MessageCB, None)
    if fd == -1:
        syslog.syslog(syslog.LOG_ERR, "Could not connect to msgBroker. Terminate daemon.")
    else:
        # register to parameters changed message
        msgIdList = [[ids.APP_ID_PARAM_HANDLING, ids.PH_MSG_ID_PARAM_FILES_CHANGED]]
        msgBrokerExt.subscribe(len(msgIdList), msgIdList)
    return fd

def MsgBrokerClose():
    msgBrokerExt.disconnect()

# Callback method - called by C-Extension Callback
def __MessageCB(appID, msgID, i_Data):

    #PrintLog("MessageCB appID: {}, msgID: {}, i_Data: {}".format(appID, msgID, i_Data))

    # handle only parameter changed message
    if (appID != ids.APP_ID_PARAM_HANDLING) or (msgID != ids.PH_MSG_ID_PARAM_FILES_CHANGED):
        return

    # calculate idList-count
    idCount = int((len(i_Data) - 8) / 4)

    # unpack payload: int32:Pid, uint32:numElements, int32:IdList[max: PH_MAX_FILE_IDS_IN_MSG(128)]
    msgData = struct.unpack("iI{0}i".format(idCount), i_Data)

    # check number of IDs in list
    if (msgData[1] > idCount):
        syslog.syslog(syslog.LOG_WARNING, "Received number of parameter file IDs ({0}) is larger than ID-list size ({1}). Take only {2} IDs."
                      .format(msgData[1], idCount, idCount))
    else:
        # take received numElements, even if list is larger
        idCount = msgData[1]

    # create set of all IDs found in IdList
    idSet = set()
    for i in range(idCount):
        idSet.add(msgData[i+2])

    PrintLog("Changed parameters: {}".format(idSet))

    # check, if vpn-auto-connection parameters in list
    if (Parameters.getParamSetId(Parameters.PARAMS_VPN_AUTO_CONNECTION) in idSet):
        syslog.syslog(syslog.LOG_NOTICE, "vpn auto connection parameters changed")
        global paramChanged
        paramChanged = True


"""
    cURL helper functions
    -----------------------------------------------------------------------------------------------
"""
def GetRestUrl(config, restCommand):
    try:
        server = config.getParamValue('FlagServerAddr')
        path   = config.getParamValue('FlagServerPath')
        port   = config.getParamValue('FlagServerPortNo')
    except ParameterError:
        raise

    # always update hostname, since hostname change by dhcp is unnoticed
    config.updateHostname()

    # prepare server base path
    path = path.rstrip('/').lstrip('/')
    path = '/'+path if len(path) else ""

    # create device-URL
    url = restPath.format(serverAddr=server, serverPort=port, serverBasePath=path, hostname=config.hostname, command=restCommand)
    return url

def GetPassword(config):
    try:
        passw = config.getParamValue("PasswordForConfigRequest")
    except ParameterError:
        raise
    return passw


# response container with header, http_code and content
class Response():
    def __init__(self):
        self.header = BytesIO()
        self.header_list = []
        self.header_dict = {}
        self.content = None
        self.eff_url = ""
        self.http_code = 0

    def parseHeader(self):
        headerlines = self.header.getvalue().decode().splitlines()
        self.header_list = []
        self.header_dict = {}
        for line in headerlines:
            k, s, v = line.partition(': ')
            k = k.lstrip().rstrip()
            v = v.lstrip().rstrip()
            if s == ': ':
                self.header_dict.update({k: v})
            elif k:
                self.header_list.append(k)


"""
    cURL request functions - General REST request
    -----------------------------------------------------------------------------------------------
"""
# returns a Response-Object or None
def PerformRestRequest(config, restCommand):

    # create REST-URL
    url = GetRestUrl(config, restCommand)

    curl = pycurl.Curl()
    #curl.setopt(curl.VERBOSE, True)
    curl.setopt(curl.URL, url)

    # POST-command with password
    if restCommands[restCommand]['method'] == 'POST':
        curl.setopt(curl.POST, True)
        curl.setopt(curl.POSTFIELDS, '{{"password": "{passw}"}}'.format(passw=GetPassword(config)))
        #curl.setopt(curl.HTTPHEADER, ['Content-Type: application/json'])

    # open secure connection with server certificate / or do it insecure without verifying server
    if os.path.exists(server_certificate):
        curl.setopt(curl.CAINFO, server_certificate)
        config.logFilter.reset(LogFilter.MISSING_CERT_WARN)
    else:
        config.logFilter.syslog(syslog.LOG_WARNING,
                                LogFilter.MISSING_CERT_WARN,
                                "server certificate '{}' not found -> No peer verification.".format(server_certificate))
        curl.setopt(curl.SSL_VERIFYPEER, 0)  # set this option if server certificate is not available

    # prepare response object
    response = Response()
    curl.setopt(curl.HEADERFUNCTION, response.header.write)

    # perform the request
    try:
        response.content = curl.perform_rb()
        config.logFilter.reset(LogFilter.CURL_CONN_ERR)
    except Exception as e:
        config.logFilter.syslog(syslog.LOG_WARNING,
                                LogFilter.CURL_CONN_ERR,
                                "Failed to connect to vpn-control-server. {}".format(e))
        config.logFilter.reset(LogFilter.NO_FLAG_WARN)
        config.logFilter.reset(LogFilter.GET_CONFIG_WARN)
        return None

    # populate response object
    response.eff_url   = curl.getinfo(curl.EFFECTIVE_URL)
    response.http_code = curl.getinfo(curl.HTTP_CODE)
    response.parseHeader()

    curl.close()

    if response.http_code != 200:
        servErr = ", {}".format(response.content.decode()) if response.content else ""
        errMsg = "Request '{}' returned {}{}".format(response.eff_url, response.http_code, servErr)
        if restCommand == restCmd_GetFlag:
            config.logFilter.syslog(syslog.LOG_WARNING,
                                    LogFilter.NO_FLAG_WARN,
                                    errMsg)
        elif restCommand == restCmd_GetConfig:
            config.logFilter.syslog(syslog.LOG_WARNING,
                                    LogFilter.GET_CONFIG_WARN,
                                    errMsg)
    else:
        if restCommand == restCmd_GetFlag:
            config.logFilter.reset(LogFilter.NO_FLAG_WARN)
        elif restCommand == restCmd_GetConfig:
            config.logFilter.reset(LogFilter.GET_CONFIG_WARN)

    PrintLog("Eff-URL     : {}".format(response.eff_url))
    PrintLog("HTTP-Code   : {}".format(response.http_code))
    PrintLog("Content-Size: {}".format(len(response.content)))

    return response


"""
    cURL request function: Check if the VPN connection should be established
    -----------------------------------------------------------------------------------------------
"""
# raises ParameterError on parameter failure
def IsFlagSet(config):
    response = PerformRestRequest(config, restCmd_GetFlag)
    if response and response.http_code == 200:
        return True
    else:
        return False


"""
    cURL request function: Get VPN client config file
    -----------------------------------------------------------------------------------------------
"""
# raises ParameterError on parameter failure
def GetDeviceConfig(config):
    response = PerformRestRequest(config, restCmd_GetConfig)
    if response and response.http_code == 200:
        if not os.path.isdir(ovpnPath):
            try:
                os.makedirs(ovpnPath, mode=0o755, exist_ok=True)
                syslog.syslog(syslog.LOG_NOTICE, "Path '{}' created.".format(ovpnPath))
            except Exception as e:
                syslog.syslog(syslog.LOG_ERR, "Failed to create path '{}'. {}".format(ovpnPath, e))
        try:
            with open(ovpnConfigFile, 'wb') as file:
                file.write(response.content)
                syslog.syslog(syslog.LOG_NOTICE, "File '{}' saved.".format(ovpnConfigFile))
                PrintLog("File '{}' saved.".format(ovpnConfigFile))
                return True
        except Exception as e:
            syslog.syslog(syslog.LOG_ERR, "Failed to save '{}'. {}".format(ovpnConfigFile, e))
            PrintLog("Error: Failed to save '{}'. {}".format(ovpnConfigFile, e))

    return False


"""
    OpenVPN control functions
    -----------------------------------------------------------------------------------------------
"""
# raises OvpnError, if config file does not exist
def CheckOpenVpnConfig():
    succ = os.path.isfile(ovpnConfigFile)
    if not succ:
        raise Exception("Required file '{}' does not exist.".format(ovpnConfigFile))

def KillRunningOpenVpn():
    procName = "openvpn --config {}".format(ovpnConfigFile)
    p = subprocess.Popen(['pgrep', '-f', procName], stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    out, err = p.communicate()
    if err:
        PrintLog("pgrep failed: {}".format(err.decode()))
    if out:
        pidList = out.decode().split('\n')
        for pidStr in pidList:
            try:
                pid = int(pidStr)
            except:
                continue
            PrintLog("Try to kill process with PID {}".format(pid))
            os.kill(pid, signal.SIGTERM)  #or signal.SIGKILL

def ConnectOpenVpn():
    PrintLog("ConnectOpenVpn - Try to open VPN connection")

    KillRunningOpenVpn()

    ovpnProcess = subprocess.Popen(['openvpn', '--config', ovpnConfigFile, '--inactive', str(ovpnTimeoutSec), '--log-append', ovpnLogFile, '--daemon'],
                                   stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    out, err = ovpnProcess.communicate()
    if err:
        PrintLog("ConnectOpenVpn Error: {}".format(err.decode()))
        syslog.syslog(syslog.LOG_ERR, "Connect OpenVpn Error: {}".format(err.decode()))
    if out:
        PrintLog("ConnectOpenVpn Output: {}".format(out.decode()))


# returns True/False or None, if no statement possible
def IsOpenVpnConnected():
    proc = subprocess.Popen(['ifconfig', 'tun0'], stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    try:
        out, err = proc.communicate(timeout=2.0)
    except subprocess.TimeoutExpired:
        proc.kill()
        out, err = proc.communicate()
        PrintLog("ifconfig timed out (2sec)")
        # ifconfig timeout: no statement possible
        return None

    if err:
        # tun0 device not found - openvpn not connected
        return False
    if out:
        tun0Config = out.decode()
        m = re.search("inet addr:(.+?)\s", tun0Config, re.MULTILINE)
        if m:
            PrintLog("OpenVPN already connected, IP: {}".format(m.group(1)))
            return True
        else:
            return False


"""
    Helper functions
    -----------------------------------------------------------------------------------------------
"""
def GetTimestamp():
    return datetime.now().strftime("%Y-%m-%d %H:%M:%S")

# print debugging info to console, if 'printDebugInfo' is True
def PrintLog(message):
    if printDebugInfo:
        print("{} {}".format(GetTimestamp(), message))


"""
    OpenVPN Flag Request Chain
    -----------------------------------------------------------------------------------------------
"""
def isToBeConnected(config):
    # check openVPN connection
    connected = IsOpenVpnConnected()
    if connected is False:
        # is feature enabled
        if config.isActive():
            if not config.useControlServer():
                return True
            # does openVPN client config exist
            try:
                CheckOpenVpnConfig()
                clientConfigExists = True
                config.logFilter.reset(LogFilter.MISSING_CONF_WARN)
            except Exception as e:
                PrintLog("{}".format(e))
                config.logFilter.syslog(syslog.LOG_WARNING,
                                        LogFilter.MISSING_CONF_WARN,
                                        "{}".format(e))
                clientConfigExists = False
            if not clientConfigExists:
                # request client config from server
                clientConfigExists = GetDeviceConfig(config)
            if clientConfigExists:
                # get flag (OpenVPNConnection)
                flagSet = IsFlagSet(config)
                PrintLog("Flag set: {}".format(flagSet))
                return flagSet

    return False


"""
    Daemon live loop
    -----------------------------------------------------------------------------------------------
"""
def LiveLoop(config):

    try:
        # get vpn auto connection parameters
        config.updateParams()

        PrintLog("Hostname: {}".format(config.hostname))
        PrintLog("Parameters: {}".format(config.params))
        PrintLog("MsgBroker-FD: {}".format(config.msgBrokerFD))

        # try to open VPN-Connection, if desired and not already connected
        openConn = isToBeConnected(config)
        if openConn:
            ConnectOpenVpn()

        # init/update select timeout
        config.updateTimeout()

        PrintLog("Entering liveLoop ..")
        while True:

            PrintLog("Select timeout: {}".format(config.selectTimeout))

            ready = select.select([config.msgBrokerFD], [], [], config.selectTimeout)

            # parameter changed event from message broker
            if (len(ready[0]) > 0):

                # handle message broker message
                msgBrokerExt.handleIncomingData()

                global paramChanged
                if paramChanged:
                    config.updateParams()
                    PrintLog("Parameters: {}".format(config.params))
                    paramChanged = False
                    config.logFilter.reset(LogFilter.CURL_CONN_ERR)
                    config.logFilter.reset(LogFilter.GET_CONFIG_WARN)
                    config.logFilter.reset(LogFilter.NO_FLAG_WARN)

            # select timeout
            else:
                # try to open VPN-Connection, if desired and not already connected
                openConn = isToBeConnected(config)
                if openConn:
                    ConnectOpenVpn()

            # update select timeout
            config.updateTimeout()

    except ParameterError:
        syslog.syslog(syslog.LOG_ERR, "Terminate daemon due to a ParameterError.")
    except CurlError:
        syslog.syslog(syslog.LOG_ERR, "Terminate daemon due to a CurlError.")
    except OvpnError:
        syslog.syslog(syslog.LOG_ERR, "Terminate daemon due to an OvpnError.")
    except Exception as e:
        syslog.syslog(syslog.LOG_ERR, "Terminate daemon due to an Error: {}".format(e))


"""
    Main function
    -----------------------------------------------------------------------------------------------
"""
def main():

    # create configuration instance
    config = VpnAutoConnectionConfig()

    # connect to message broker
    config.msgBrokerFD = MsgBrokerConnect()
    if config.msgBrokerFD > 0:      # without message broker registration the service is not executable

        # enter the live loop
        LiveLoop(config)

    MsgBrokerClose()


if __name__ == "__main__":
    main()

