import http.server
import os
import datetime
import logging
import json
import ipaddress
from uuid import getnode
from time import sleep
from xml.dom import minidom

from xml.etree import ElementTree as et
from lxml import etree


#--------------------------------------------------------------------------------------------------
class XmlStruct:
    def __init__(self, Tag):
        if isinstance(Tag, tuple):
            Tag = ".".join(Tag)
        self.Top = et.Element(Tag)

    def addParent(self, Parent, Name):
        if Parent is None:
            Parent = self.Top
        if isinstance(Name, tuple):
            Name = ".".join(Name)
        return et.SubElement(Parent, Name)

    def add(self, Parent, Key, Value):
        if Parent is None:
            Parent = self.Top
        c = et.SubElement(Parent, Key)
        c.text = Value

    def addWithValue(self, Parent, Key, Value):
        if Parent is None:
            Parent = self.Top
        p = self.addParent(Parent, Key)
        self.add(p, "Value", Value)

    def addTimeStamp(self, Parent):
        self.addWithValue(Parent, "TimeStamp", XmlStruct.DateTime())

    @staticmethod
    def addComment(Parent, CommentStr):
        c = et.Comment(CommentStr)
        Parent.append(c)

    def toxml(self):
        t = et.tostring(self.Top, "utf-8")
        return minidom.parseString(t).toprettyxml()

    @staticmethod
    def DateTime():
        return datetime.datetime.now().strftime("%Y-%m-%dT%H:%M:%S")

#--------------------------------------------------------------------------------------------------
class XmlResponse:
    def __init__(self, Tag):
        self.XmlStruct = XmlStruct(Tag)

    def toxml(self):
        return self.XmlStruct.toxml()

#--------------------------------------------------------------------------------------------------
class XmlUnSubscribeResponse(XmlResponse):    # for subscribe and unsubscribe
    def __init__(self, ServiceName, Action, Content):
        super().__init__(Action+"Response")

    def setError(self, ErrorStr):
        self.XmlStruct.addWithValue(None, "OperationErrorMessage", ErrorStr)

    def setActive(self):
        self.XmlStruct.addWithValue(None, "Active", "true")

    def setInactive(self):
        self.XmlStruct.addWithValue(None, "Active", "false")

#--------------------------------------------------------------------------------------------------
class XmlDataAcceptedResponse(XmlResponse):
    def __init__(self):
        super().__init__("DataAcceptedResponse")

    def Accept(self):
        p = self.XmlStruct.addParent(None, "DataAcceptedResponseData")
        self.XmlStruct.addTimeStamp(p)
        self.XmlStruct.addWithValue(p, "DataAccepted", "true")

    def Decline(self, Reason):
        p = self.XmlStruct.addParent(None, "OperationErrorMessage")
        self.XmlStruct.add(p, "Value", Reason)

#--------------------------------------------------------------------------------------------------
class BaseIBISIPService:
    # For subscriber needed:
    Key_ClientIPAddress = "Client-IP-Address"
    Key_ReplyPort       = "ReplyPort"
    Key_ReplyPath       = "ReplyPath"

    State_notrunning    = 0     # s. chapter 4.3.1
    State_running       = 1
    State_defective     = 2
    State_starting      = 3
    State_standby       = 4

    def __init__(self, ServiceName, PortNo, UrlHeader):
        self.ServiceName = ServiceName
        self.PortNo      = PortNo
        self.UrlHeader   = UrlHeader
        if not self.UrlHeader.endswith("/"):
            self.UrlHeader += "/"
        if not self.UrlHeader.startswith("/"):
            self.UrlHeader = "/" + self.UrlHeader
        self.RequestFT = {}          # FunctionTable
        self.SubscriberList = dict()    # dict of list of subscribers
        self.State = BaseIBISIPService.State_notrunning
        self.DNSSDServiceNamePostfix = None

    @staticmethod
    def getSubscriber(RequestStruct, isSubscriber):
        if isSubscriber:
            d = {BaseIBISIPService.Key_ReplyPort: 80,
                 BaseIBISIPService.Key_ReplyPath: "/"}    # Filled with default values (they
        else:                                             # are optional in the request)
            d = {}

        if isinstance(RequestStruct, str):
            logging.debug(RequestStruct)
        else:
            logging.debug(RequestStruct.decode("utf-8"))
        root = et.fromstring(RequestStruct)
        # May check root.tag. Should be something like e.g.
        # "DeviceManagementService.SubscribeDeviceInformation"
        if (   (    isSubscriber and (root.tag ==   "SubscribeRequest"))
            or (not isSubscriber and (root.tag == "UnsubscribeRequest"))
            ):
            for c1 in root:
                if c1.tag == BaseIBISIPService.Key_ClientIPAddress:
                    for c2 in c1:
                        if c2.tag == "Value":
                            d[BaseIBISIPService.Key_ClientIPAddress] = c2.text
                elif c1.tag == BaseIBISIPService.Key_ReplyPort:
                    for c2 in c1:
                        if c2.tag == "Value":
                            d[BaseIBISIPService.Key_ReplyPort] = c2.text
                elif c1.tag == BaseIBISIPService.Key_ReplyPath:
                    for c2 in c1:
                        if c2.tag == "Value":
                            if c2.text.startswith("/"):
                                d[BaseIBISIPService.Key_ReplyPath] = c2.text
                            else:
                                d[BaseIBISIPService.Key_ReplyPath] = "/" + c2.text

        if not BaseIBISIPService.Key_ClientIPAddress in d:
            return None
        if not BaseIBISIPService.isSubscriberValid(d):
            return None
        return d

    @staticmethod
    def isSubscriberValid(ValueDict):
        try:
            PortNo = int(ValueDict[BaseIBISIPService.Key_ReplyPort])
        except ValueError:
            return False
        if str(PortNo) != ValueDict[BaseIBISIPService.Key_ReplyPort]:
            return False
        try:
            ipaddress.ip_address(ValueDict[BaseIBISIPService.Key_ClientIPAddress])
        except ValueError:
            return False
        return True

    def _GetDNSPayload(self, ServiceName, PortNo, UrlPath, Version="1.0", TXTDict = None):
        if self.DNSSDServiceNamePostfix is not None:
            ServiceName = ServiceName + self.DNSSDServiceNamePostfix
        if UrlPath.endswith("/"):
            UrlPath = UrlPath[:-1]
        if UrlPath.startswith("/"):
            UrlPath = UrlPath[1:]
        d = dict()
        d["ServiceName"] = ServiceName
        d["PortNo"]      = str(PortNo)
        d["AppProt"]     = "_ibisip_http"
        d["TransProt"]   = "_tcp"
        # DNS-SD TXT record:
        PropDict = dict()
        PropDict["ver"]  = Version
        PropDict["path"] = UrlPath

        if TXTDict is not None and isinstance(TXTDict, dict):
            PropDict.update(TXTDict)
        d["Properties"] = PropDict
        ret = json.dumps(d)
        ret = bytes(ret.encode("utf-8"))
        return ret

    def _useUniqueServiceName(self, doIt):
        if doIt:
            mac = getnode()
            self.DNSSDServiceNamePostfix = "-%012X" % mac
        else:
            self.DNSSDServiceNamePostfix = None

    def getURL(self):
        return self.UrlHeader + self.ServiceName

    def getServiceName(self):
        return self.ServiceName

    def _isAlreadySubscribed(self, Subscriber, Content):
        if not Content in self.SubscriberList:
            self.SubscriberList[Content] = list()
            return False
        for peer in self.SubscriberList[Content]:
            if peer == Subscriber:
                return True
        return False

    @staticmethod
    def _matchForUnsubscribe(candidate, ListElement):
        """Returns True when candidate equals ListElement. candidate need not to contain the
           keys ReplyPath and ReplyPort. In this case they match any in ListElement."""
        if (candidate  [BaseIBISIPService.Key_ClientIPAddress] !=
            ListElement[BaseIBISIPService.Key_ClientIPAddress]):
            return False
        if (BaseIBISIPService.Key_ReplyPath in candidate and
            (candidate  [BaseIBISIPService.Key_ReplyPath] !=
             ListElement[BaseIBISIPService.Key_ReplyPath])):
            return False
        if (BaseIBISIPService.Key_ReplyPort in candidate and
            (candidate  [BaseIBISIPService.Key_ReplyPort] !=
             ListElement[BaseIBISIPService.Key_ReplyPort])):
            return False
        return True

    def _Subscribe(self, Content, RequestStruct):
        SubResp = XmlUnSubscribeResponse(self.ServiceName, "Subscribe", Content)
        Subscriber = BaseIBISIPService.getSubscriber(RequestStruct, True)
        if Subscriber:
            if self._isAlreadySubscribed(Subscriber, Content):
                SubResp.setError("Error: Already subscribed")
            else:
                self.SubscriberList[Content].append(Subscriber)
                SubResp.setActive()
                logging.debug("Subscription [%s] added for" % Content, Subscriber)
        else:
            SubResp.setError("Error: Malformed SubscribeRequest")
        return 200, SubResp.toxml()

    def _Unsubscribe(self, Content, RequestStruct):
        UnsubResp = XmlUnSubscribeResponse(self.ServiceName, "Unsubscribe", Content)
        Unsubscriber = BaseIBISIPService.getSubscriber(RequestStruct, False)
        if Unsubscriber:
            if Content in self.SubscriberList:
                SubscriberCount_old = len(self.SubscriberList[Content])
                self.SubscriberList[Content][:] = [x for x in
                    self.SubscriberList[Content] if not self._matchForUnsubscribe(Unsubscriber, x)]
                if SubscriberCount_old > len(self.SubscriberList[Content]):
                    UnsubResp.setInactive()
                else:
                    UnsubResp.setError("Error: Is not currently subscribed.")
            else:
                UnsubResp.setError("Error: Is not currently subscribed.")
        else:
            UnsubResp.setError("Error: Malformed UnsubscribeRequest")
        return 200, UnsubResp.toxml()

    def _CallSubscribers(self, Kind, GetXmlFunc):
        if Kind not in self.SubscriberList:
            return  # nobody as subscribed

        val, xml = GetXmlFunc(None)
        LastClientIPAddress = None
        for sub in self.SubscriberList[Kind]:
            logging.debug("Calling subscriber %s:%s" % (
                    sub[BaseIBISIPService.Key_ClientIPAddress],
                    sub[BaseIBISIPService.Key_ReplyPort]))
            c = None
            try:
                if LastClientIPAddress == sub[BaseIBISIPService.Key_ClientIPAddress]:
                    # Not very clean: When calling http.client.HTTPConnection twice
                    # with the same IP address  the second failes with "Connection
                    # refused". Seems to be because the link is not fully FINalized.
                    # We make a workaround with waiting a little bit:
                    sleep(0.1)
                c = http.client.HTTPConnection(
                        sub[BaseIBISIPService.Key_ClientIPAddress],
                        sub[BaseIBISIPService.Key_ReplyPort])
                c.request("POST", sub[BaseIBISIPService.Key_ReplyPath], xml)
                LastClientIPAddress = sub[BaseIBISIPService.Key_ClientIPAddress]
                # We have no reason to read the response.
            except:
                pass
            finally:
                if c is not None:
                    c.close()

    def Request(self, RequestStr, RequestStruct):
        RequestFunc = self.RequestFT.get(RequestStr, None)  # get function from function table
        if RequestFunc is None:
            return 404, None
        return RequestFunc(RequestStruct)

#--------------------------------------------------------------------------------------------------
class BaseHttpHandler(http.server.BaseHTTPRequestHandler):
    "Both IbisService* class variables need to be overwritten."""
    IbisServiceIDStr = "2boverwritten"
    IbisService = None

    def log_message(self, Format, *args):
        """This function overrides the base function which logs to stdout which is normally
           not wanted."""
        pass

    @staticmethod
    def _checkPath(Path):
        if Path.startswith(".."):
            # Oh no! We do not want paths like this:
            return None
        if not Path.startswith("/"):
            Path = "/" + Path
        if not Path.startswith(BaseHttpHandler.IbisServiceIDStr):
            return None
        return Path

    def do_GET(self):
        Path = self._checkPath(self.path)
        if Path is None:
            self.send_response(404)
            return

        req = Path.split("/")[-1]
        HTTPStatus, Answer = BaseHttpHandler.IbisService.Request(req, None)
        self.send_response(HTTPStatus)
        if Answer is not None:
            Answer = Answer.encode("utf-8")
            self.send_header("Content-Type", "application/xml")
            self.send_header("Content-Length", str(len(Answer)))
            self.end_headers()
            self.wfile.write(Answer)

    def do_POST(self):
        Path = self._checkPath(self.path)
        if Path is None:
            self.send_response(404)
            return

        Data = self.ReadData()
        if not Data:
            self.send_response(404)
            return

        req = Path.split("/")[-1]
        HTTPStatus, Answer = BaseHttpHandler.IbisService.Request(req, Data)
        self.send_response(HTTPStatus)
        if Answer is not None:
            Answer = Answer.encode("utf-8")
            self.send_header("Content-Type", "application/xml")
            self.send_header("Content-Length", str(len(Answer)))
            self.end_headers()
            self.wfile.write(Answer)
        else:
            self.end_headers()

    def ReadData(self):
        for header in self.headers.items():
            if header[0] == "Content-Length":
                ContentLength = int(header[1])
                Data = self.rfile.read(ContentLength)
                return Data
        return None

#--------------------------------------------------------------------------------------------------
class BaseConfigParameter:
    """Base class for reading/writing config parameter files."""
    # TODO: Access may be simultanously from two processes. Use lock file for
    # syncronisation
    def __init__(self, Filename):
        self.ParameterFilename = os.path.join("/var/Counter/config", Filename)
        self.ConfigFileAvail = os.path.isfile(self.ParameterFilename)

        Filename = Filename.replace("_values.xml", "_meta.xml")
        self.MetaFilename = os.path.join("/etc/Counter/ParamMetaFiles", Filename)
        self.MetaFileAvail = os.path.isfile(self.MetaFilename)

    def _setValue(self, Name, Value):
        tree = etree.parse(self.ParameterFilename)
        root = tree.getroot()
        for c in root:
            if c.attrib["name"] == Name:
                c.attrib["value"] = Value
                tree.write(self.ParameterFilename, xml_declaration=True, encoding="utf-8")
                return

    def _getValue(self, Name):
        root = etree.parse(self.ParameterFilename).getroot()
        for c in root:
            if c.attrib["name"] == Name:
                return c.attrib["value"]
        return None

    def _getRange(self, Name):
        if not self.MetaFileAvail:
            return None
        try:
            root = etree.parse(self.MetaFilename).getroot()
            for c in root:
                if c.attrib["name"] == Name:
                    return [int(c.attrib["minValue"]), int(c.attrib["maxValue"])]
        except:
            pass
        return None


#--------------------------------------------------------------------------------------------------
class ConfigParameter_VDV301(BaseConfigParameter):
    """Allows read/write access to /var/Counter/config/vdv301_params_values.xml"""
    def __init__(self):
        super().__init__("vdv301_params_values.xml")
        self.DeviceIDTagname = "DeviceID"

    def setDeviceID(self, DeviceID):
        if not (self.ConfigFileAvail and self.MetaFileAvail):
            return False
        Range = self._getRange(self.DeviceIDTagname)
        if Range and Range[0] <= DeviceID <= Range[1]:
            self._setValue(self.DeviceIDTagname, str(DeviceID))
            return True
        return False

    def getDeviceID(self):
        DeviceID = -1
        if not self.ConfigFileAvail:
            return DeviceID
        try:
            DeviceIDStr = self._getValue(self.DeviceIDTagname)
            if DeviceIDStr:
                DeviceID = int(DeviceIDStr)
            else:
                DeviceID = -1
        except:
            DeviceID = -1
        return DeviceID
