#!/usr/bin/env python3

import time
import math
import struct
import hashlib
from os.path import isfile, splitext

class PositionCalculator:
    def __init__(self, modelStr,
                 ltFilter=100, stFilter=10,
                 threshLtSt=15.0, minNeed4UpdTime=5.0,
                 backupFile=None):
        self.model        = modelStr

        # Short time average for current position
        self.stX          = None
        self.stY          = None
        self.stZ          = None

        self.stPitch      = None
        self.stYaw        = None
        self.stNorm       = None

        # Long time average for rest position
        self.ltX          = None
        self.ltY          = None
        self.ltZ          = None

        self.ltPitch      = None
        self.ltYaw        = None
        self.ltNorm       = None

        self.ltFilter = float(ltFilter)
        self.stFilter = float(stFilter)

        self.threshLtSt = threshLtSt

        self.backUpTimeStamp = None
        self.timeStamp       = time.clock_gettime(time.CLOCK_MONOTONIC)
        self.minNeed4UpdTime = minNeed4UpdTime # In secounds

        self.backupFile = backupFile

        if self.backupFile is not None:
            self.restoreValues()

    def _getHash(self, data):
        m = hashlib.md5()
        m.update(data)
        return m.hexdigest()

    def restoreValues(self):
        md5File = splitext(self.backupFile)[0] + '.md5'
        if isfile(self.backupFile) and isfile(md5File):
            # Read data and MD5 value
            with open(self.backupFile, 'rb') as bf:
                data = bf.read()
            with open(md5File, 'r') as mdf:
                md5f = mdf.readlines()

                if len(md5f)>0:
                    md5f = md5f[0]
        else:
            return

        # Check and unpack data
        if len(data)>0:
            md5d = self._getHash(data)

        if md5d == md5f:
            # Unpack data
            uData = struct.unpack('ffffff', data)

            self.ltX     = uData[0]
            self.ltY     = uData[1]
            self.ltZ     = uData[2]

            self.ltPitch = uData[3]
            self.ltYaw   = uData[4]
            self.ltNorm  = uData[5]

    def dumpValues(self):
        if self.backupFile is None:
            return

        data = struct.pack('ffffff',
                           self.ltX, self.ltY, self.ltZ,
                           self.ltPitch, self.ltYaw, self.ltNorm)
        md5 = self._getHash(data)

        md5File = splitext(self.backupFile)[0] + '.md5'

        # Write accel values
        with open(self.backupFile, 'wb') as bf:
            bf.write(data)
        # Write MD5
        with open(md5File, 'w') as bf:
            bf.write(md5)

        self.backUpTimeStamp = time.clock_gettime(time.CLOCK_MONOTONIC)

    def update(self, xi, yi, zi, isInMotion):
        if (self.model.startswith('HH') or
            self.model.startswith('IH') or
            self.model.startswith('JH')):
            x =      yi/1000.0
            y = -1.0*xi/1000.0
            z =      zi/1000.0
        else:
            x = xi/1000.0
            y = yi/1000.0
            z = zi/1000.0

        # x,y,z -> pitch, yaw, norm
        norm  = math.sqrt(x*x + y*y + z*z)
        pitch = math.degrees(math.atan2(y, z))
        yaw   = math.degrees(math.atan2(x, z))

        #### Short time average

        # X
        if self.stX is None:
            self.stX = x
        else:
            self.stX = ( (self.stFilter-1.0)*self.stX + x ) / self.stFilter

        # Y
        if self.stY is None:
            self.stY = y
        else:
            self.stY = ( (self.stFilter-1.0)*self.stY + y ) / self.stFilter

        # Z
        if self.stZ is None:
            self.stZ = z
        else:
            self.stZ = ( (self.stFilter-1.0)*self.stZ + z ) / self.stFilter

        # Pitch
        if self.stPitch is None:
            self.stPitch = pitch
        else:
            self.stPitch = ( (self.stFilter-1.0)*self.stPitch + pitch ) / self.stFilter

        # Yaw
        if self.stYaw is None:
            self.stYaw = yaw
        else:
            self.stYaw = ( (self.stFilter-1.0)*self.stYaw + yaw ) / self.stFilter

        # Norm
        if self.stNorm is None:
            self.stNorm = norm
        else:
            self.stNorm = ( (self.stFilter-1.0)*self.stNorm + norm ) / self.stFilter

        #### Long time average

        # X
        if self.ltX is None:
            self.ltX = x
        elif isInMotion==0:
            self.ltX = ( (self.ltFilter-1.0)*self.ltX + x ) / self.ltFilter

        # Y
        if self.ltY is None:
            self.ltY = y
        elif isInMotion==0:
            self.ltY = ( (self.ltFilter-1.0)*self.ltY + y ) / self.ltFilter

        # Z
        if self.ltZ is None:
            self.ltZ = z
        elif isInMotion==0:
            self.ltZ = ( (self.ltFilter-1.0)*self.ltZ + z ) / self.ltFilter

        # Pitch
        if self.ltPitch is None:
            self.ltPitch = pitch
        elif isInMotion==0:
            self.ltPitch = ( (self.ltFilter-1.0)*self.ltPitch + pitch ) / self.ltFilter

        # Yaw
        if self.ltYaw is None:
            self.ltYaw = yaw
        elif isInMotion==0:
            self.ltYaw = ( (self.ltFilter-1.0)*self.ltYaw + yaw ) / self.ltFilter

        # Norm
        if self.ltNorm is None:
            self.ltNorm = norm
        elif isInMotion==0:
            self.ltNorm = ( (self.ltFilter-1.0)*self.ltNorm + norm ) / self.ltFilter

        #### Check for large changes
        # Update long time values if difference between LT and ST is above threshLtSt for a certain time
        ts       = time.clock_gettime(time.CLOCK_MONOTONIC)
        updPitch = self.threshLtSt<abs(self.stPitch - self.ltPitch)
        updYaw   = self.threshLtSt<abs(self.stYaw   - self.ltYaw  )
        if (isInMotion==0) and (updPitch or updYaw):
            tsDiff = abs(self.timeStamp - ts) > self.minNeed4UpdTime
            if tsDiff is True:
                self.ltPitch   = self.stPitch
                self.ltYaw     = self.stYaw
                self.ltNorm    = self.stNorm
                self.timeStamp = ts
        else:
            self.timeStamp = ts

        # BackUp Values every 30 secounds
        if self.backUpTimeStamp is None:
            self.dumpValues()
        elif abs(self.backUpTimeStamp-ts)>30.0:
            self.dumpValues()

    def getAccelVector(self):
        return [self.ltX-self.stX, self.ltY-self.stY, self.ltZ-self.stZ]

