#!/usr/bin/env python

import math
import time

from collections import deque
from armnn import armnn

class Hysteresis:
    def __init__(self, startState = True, trueDelay=0., falseDelay=0.):
        self.trueDelayParam  = trueDelay # in seconds
        self.falseDelayParam = falseDelay # in seconds
        self.trueDelay       = 0. # delay to switch from state 'true' to 'false' in seconds
        self.falseDelay      = 0. # delay to switch from state 'false' to 'true' in seconds
        self.state           = startState # current filtered state
        self.lastUpdateTime  = time.time() # last time an update had a different state

        self.update(startState)

    def update(self, state):
        t = time.time()
        tDiff = t - self.lastUpdateTime
        self.lastUpdateTime = t

        if state == self.state:
            self.trueDelay  = self.trueDelayParam
            self.falseDelay = self.falseDelayParam
        else:
            if state == True:
                self.trueDelay = max([0., self.trueDelay - tDiff])
                if self.trueDelay == 0.:
                    self.state = True
            if state == False:
                self.falseDelay = max([0., self.falseDelay - tDiff])
                if self.falseDelay == 0.:
                    self.state = False

        return self.state

    def getState(self):
        return self.state

    def setParams(self, params):
        self.trueDelayParam  = params["TrueDelay"]  / 1000.
        self.falseDelayParam = params["FalseDelay"] / 1000.

        self.trueDelay = self.trueDelayParam
        self.falseDelay = self.falseDelayParam

# --------------------------------------------------------------------------------------------------
class StartStopDetector_CNN:
    """ Uses a neural network to decide if the vehicle is moving
    """
    def __init__(self, threshold = 0.75, movAveSizeAccel = 20, movAveSizeBreak = 3):
        self.model         = None
        self.IsInMotion    = 2 # 0:False | 1:True | 2:Undefined
        self.movingAveDec  = None
        self.movAveSizeA   = float(movAveSizeAccel)
        self.movAveSizeB   = float(movAveSizeBreak)
        self.threshold     = threshold
        self.lastVvals     = None
        self.lastLvals     = None
        self.lastTvals     = None
        self.lastDecision  = None
        self.verbose       = False
        self.inMotionHysteresis = Hysteresis()
        self.nMeasurements = 0
        self.windowSize    = 0

        self.coloredState = ['\033[1;31;40m False \033[1;37;40m',
                             '\033[1;32;40m True \033[1;37;40m',
                             '\033[1;34;40m Undefined \033[1;37;40m']

    def setParams(self, params):
        hystParams = {"TrueDelay": params["StartDelay"], "FalseDelay": params["StopDelay"]}
        self.inMotionHysteresis.setParams(hystParams)

    def setVerbose(self, verbose):
        self.verbose = verbose

    def loadNetwork(self,
                    modelPath   = '',
                    inputLayer  = 'conv2d_input',
                    inputShape  = None,
                    outputLayer = 'conv2d_5/BiasAdd'):
        if inputShape is None:
            inputShape = [1, 3, 1, 64]

        self.lastVvals  = deque([0.0]*inputShape[-1], inputShape[-1])
        self.lastLvals  = deque([0.0]*inputShape[-1], inputShape[-1])
        self.lastTvals  = deque([0.0]*inputShape[-1], inputShape[-1])
        self.windowSize = inputShape[-1]

        # Unload old model
        if self.model is not None:
            self.model.unload()
            self.model = None

        # simple path check
        if len(modelPath)==0:
            return 'Empty model path'

        # Load model
        self.model = armnn()
        errMsg = self.model.init(modelPath,
                                 inputLayer, inputShape,
                                 outputLayer)

        return errMsg


    def update(self, v, l, t):
        self.lastVvals.append(float(v)/1024.0)
        self.lastLvals.append(float(l)/1024.0)
        self.lastTvals.append(float(t)/1024.0)
        if self.verbose:
            print('StartStopDetector_CNN::Update(): Timestamp: %.3f' % time.clock_gettime(time.CLOCK_MONOTONIC))
            print('StartStopDetector_CNN::Update(): V:%f, L:%f, T:%f' % (v,l,t))

        self.nMeasurements = min(self.nMeasurements + 1, self.windowSize)

    def isInMotion(self):
        if self.nMeasurements < self.windowSize:
            self.IsInMotion = 2

            if self.verbose:
                resStr = self.coloredState[int(self.IsInMotion)]
                print( 'StartStopDetector_CNN::isInMotion(): %s' % (resStr) )

            return self.IsInMotion

        if self.model is None:
            return False

        if self.verbose:
            print('StartStopDetector_CNN::isInMotion(): Evaluating network')

        # Exceute network
        result = self.model.run( list(self.lastVvals)
                                +list(self.lastLvals)
                                +list(self.lastTvals))

        if self.verbose:
            print('StartStopDetector_CNN::isInMotion(): ',result)

        # Apply SoftMax
        resultExp = [ math.exp(x) for x in result ]
        prob = resultExp[0] / sum(resultExp)

        # Prob > 0.5 -> vehicle is moving
        # Prob < 0.5 -> vehicle is not moving

        # Determine factor
        if self.lastDecision is None:
            self.lastDecision = prob
        else:
            diff = prob - self.lastDecision
            if diff > 0.0:
                factor = self.movAveSizeA
            else:
                factor = self.movAveSizeB

            self.lastDecision = prob

        # Apply moving average
        if self.movingAveDec is None:
            self.movingAveDec = prob
        else:
            self.movingAveDec = (self.movingAveDec*(factor-1.0) + prob) / factor

        # return result
        #self.IsInMotion = self.movingAveDec > self.threshold
        isInMotion = self.movingAveDec > self.threshold
        self.IsInMotion = int(self.inMotionHysteresis.update(isInMotion))

        if self.verbose:
            resStr = self.coloredState[int(self.IsInMotion)]
            print( 'StartStopDetector_CNN::isInMotion(): Prob %f AveProb %f -> %s' % (prob, self.movingAveDec, resStr) )

        return self.IsInMotion