import socket
import time
import struct
import sys

from messages import ids

class MsgBrokerLib:
    Header_Version    = 0
    Header_AppID      = 1
    Header_MsgID      = 2
    Header_TransID    = 3
    Header_PayloadLen = 4
    Header_Flags      = 5
    Header_Spare      = 6
    HeaderLen         = 7*4     # Bytes

    # From Interfaces/include/baSS_Interfaces/BaSS_Messages.h
    Version = 0xFA5005B0

    # TODO: Remove all these constants. They will be generated from XML files.
    AppID_PolygonCounting       = ids.APP_ID_POLYGON_COUNTING_D
    MsgID_PC_CountResults       = ids.APC_POLYGON_COUNTING_D_MSG_ID_COUNT_RESULTS
    MsgID_PC_CountFinished      = ids.APC_POLYGON_COUNTING_D_MSG_ID_COUNTING_FINISHED

    AppID_ParamHandling         = ids.APP_ID_PARAM_HANDLING
    MsgID_PH_ParamFilesChanged  = ids.PH_MSG_ID_PARAM_FILES_CHANGED

    AppID_AcceloReader             = 50
    MsgID_ACC_AccelVectorVltRaw    =  1

    AppID_StartStopDetector        = 34
    MsgID_SSD_StartDetected        =  1
    MsgID_SSD_StopDetected         =  2
    MsgID_SSD_REQ_MotionState      =  3
    MsgID_SSD_RES_MotionState      =  4
    MsgID_SSD_REQ_Orientation      =  5
    MsgID_SSD_RES_Orientation      =  6
    MsgID_SSD_ltOrientationChanged =  7
    MsgID_SSD_REQ_ltOrientation    =  8
    MsgID_SSD_RES_ltOrientation    =  9
    MsgID_SSD_accelVector          = 10


    # Message IDs of the MessageBroker (s. Linux/Daemons/HAGL/MsgBroker/include/MsgIDs.h)
    AppID_MsgBroker   = 1   # (s.  Interfaces/include/BaSS_Interfaces/AppIDs.h)
    MsgID_MsgBroker_ReqSubscribe       = 1
    MsgID_MsgBroker_ReqUnsubscribe     = 2
    MsgID_MsgBroker_ResSubscribe       = 3
    MsgID_MsgBroker_ResUnsubscribe     = 4
    MsgID_MsgBroker_ReqDisconnectFlush = 5
    MsgID_MsgBroker_ResDisconnectFlush = 6
    MsgID_MsgBroker_NotifyAppName      = 8
    MsgID_MsgBroker_DumpPeerList       = 9

    AppID_OSD = 24
    MsgID_OSD_StateChange = 1
    MsgID_OSD_StateReq    = 2
    MsgID_OSD_StateRes    = 3

    AppID_SystemCallSubst = 26
    MsgID_SCS_RunCommand = 1

    AppID_IBIS_DeviceManagementService = 27
    MsgID_DMS_ServiceStarted    = 1

    AppID_IBISIP_PassengerCoutingService = 28
    MsgID_PCS_StopService              = 1

    # Messages for DNS:
    AppID_DNSSD = 29
    MsgID_DNSSD_PingReq         = 3
    MsgID_DNSSD_PingRes         = 4
    MsgID_DNSSD_ServiceStarted  = 5
    MsgID_DNSSD_Advertise       = 6
    MsgID_DNSSD_Deadvertise     = 7

    AppID_DiskMounter = 36
    MsgID_DM_MountStatusReq         = 1
    MsgID_DM_MountStatusRes         = 2
    MsgID_DM_MountImmediateReq      = 3
    MsgID_DM_MountContinousReq      = 4
    MsgID_DM_MountContinousAbortReq = 5
    MsgID_DM_UnmountImmediateReq    = 6



    def __init__(self):
        self.Socket = None
        self.Host = "127.0.0.1"
        self.Port = 10079
        self.Connected = False
        self.TransactionID = 0
        self.InData = bytes()
        self.DNSSDServiceNamePostfix = None

    def _makeHeader(self, AppID, MsgID, PayloadLen):
        self.TransactionID += 1
        Flags = 0
        Spare = 0
        return struct.pack("IIIIIII", MsgBrokerLib.Version, AppID, MsgID, self.TransactionID,
                PayloadLen, Flags, Spare)

    def _transferAppName(self):
        Payload = sys.argv[0]       # script name ...
        Payload = Payload.rsplit("/", 1)[-1]    # ... without leading paths
        Payload = Payload.encode("utf-8") + bytes(1)    # ... "C"-formatted (with trailing NULL-byte)
        self.SendMsg(MsgBrokerLib.AppID_MsgBroker,
                     MsgBrokerLib.MsgID_MsgBroker_NotifyAppName,
                     Payload)

    def MsgCB(self, AppID, MsgID, Payload):
        # To be overridden by subclass
        pass

    def StatusCB(self, FailedCount):
        # To be overridden by subclass
        pass

    def ConnectToMessageBroker(self, Timeout_ms=3000):
        if self.Socket is None:
            self.Socket = socket.socket()
        EndTime_s = time.time() + float(Timeout_ms)/1000.0  # EndTime is float
        while not self.Connected:
            try:
                self.Socket.connect((self.Host, self.Port))
                self.Connected = True
            except:
                if time.time() >= EndTime_s:
                    break
                time.sleep(0.5)
        self._transferAppName()
        return self.GetUnderlyingFD()

    def Disconnect(self):
        if self.Socket is not None:
            self.Socket.shutdown(socket.SHUT_WR)
            self.Socket.close()
            self.Socket = None
            self.Connected = False

    def _SendIDList(self, MsgID, IDList):
        bIDList = bytes()
        for Id in IDList:
            bIDList += struct.pack("II", Id[0], Id[1])
        return self.SendMsg(MsgBrokerLib.AppID_MsgBroker, MsgID, bIDList)

    def SubscribeToMsgIDs(self, IDList):
        return self._SendIDList(MsgBrokerLib.MsgID_MsgBroker_ReqSubscribe, IDList)

    def Unsubscribe(self, IDList):
        return self._SendIDList(MsgBrokerLib.MsgID_MsgBroker_ReqUnsubscribe, IDList)

    def SendMsg(self, AppID, MsgID, Payload):
        if not self.Connected:
            return False
        if Payload is None:
            data = self._makeHeader(AppID, MsgID, 0)
        else:
            data = self._makeHeader(AppID, MsgID, len(Payload)) + Payload
        self.Socket.send(data) # TODO: check that len(data) bytes were sent
        return True

    def _ProcessPacket(self, AppID, MsgID, Payload):
        if AppID == MsgBrokerLib.AppID_MsgBroker:
            if MsgID == MsgBrokerLib.MsgID_MsgBroker_ResSubscribe:
                FailedCount = struct.unpack("I", Payload)
                self.StatusCB(FailedCount[0])
            elif MsgID == MsgBrokerLib.MsgID_MsgBroker_ResUnsubscribe:
                pass
            elif MsgID == MsgBrokerLib.MsgID_MsgBroker_ResDisconnectFlush:
                pass
        else:
            self.MsgCB(AppID, MsgID, Payload)


    def _ParseData(self):
        while self.InData:
            if len(self.InData) < MsgBrokerLib.HeaderLen:
                return  # Header not complete, try again after recv()
            h = struct.unpack("IIIIIII", self.InData[:MsgBrokerLib.HeaderLen])
            PacketLen = MsgBrokerLib.HeaderLen + h[MsgBrokerLib.Header_PayloadLen]
            # XXX May add plausibility test for PacketLen
            if len(self.InData) < PacketLen:
                # Data not fully arrived. Try again after next recv():
                return
            self._ProcessPacket(
                    h[MsgBrokerLib.Header_AppID],
                    h[MsgBrokerLib.Header_MsgID],
                    self.InData[MsgBrokerLib.HeaderLen:PacketLen])
            # Remove packet from InData:
            self.InData = self.InData[PacketLen:]


    def HandleIncomingData(self):
        OldLen = len(self.InData)
        self.InData += self.Socket.recv(8192)
        if len(self.InData) == OldLen:
            # Socket ist closed by peer:
            self.Disconnect()
            return
        self._ParseData()

    def GetUnderlyingFD(self):
        if self.Connected:
            return self.Socket.fileno()
        return -1
