import logging
import struct

logger = logging.getLogger("readmessages")

objClasses = { 0: "Unknown",
               1: "Adult",
               2: "Child",
               3: "Bike",
               4: "Group",
               5: "Cart",
               6: "Wheelchair",
               7: "Pram" }

objClassesMax = 8

class CountData_t:
    def __init__(self, i_objClass, i_cntIn, i_cntOut):
        self.objClass   = i_objClass
        self.cntIn      = i_cntIn
        self.cntOut     = i_cntOut

class PolygonCountingResult_t:
    def __init__(self, i_lineNumber, i_countDataCnt, i_countData):
        self.lineNumber     = i_lineNumber
        self.countDataCnt   = i_countDataCnt
        self.countData      = i_countData

class PolygonCountingResults_t:
    def __init__(self, i_frameCounter, i_countDataMaxCnt, i_numberOfResults, i_result):
        self.frameCounter       = i_frameCounter
        self.countDataMaxCnt    = i_countDataMaxCnt
        self.numberOfResults    = i_numberOfResults
        self.result             = i_result

class CountResult_t:
    def __init__(self, i_lineId, i_objClass, i_cntIn, i_cntOut):
        self.lineId     = i_lineId
        self.objClass   = i_objClass
        self.cntIn      = i_cntIn
        self.cntOut     = i_cntOut

class CountResultsMsg_t:
    def __init__(self, i_FrameCounter, i_resultsCnt, i_results):
        self.frameCounter   = i_FrameCounter
        self.resultsCnt     = i_resultsCnt
        self.results        = i_results

class SingleZoneResult_t:
    def __init__(self, i_zoneNumber, i_fillLevelList, i_avgDwellList, i_maxDwellList):
        self.zoneNumber     = i_zoneNumber
        self.fillLevel      = i_fillLevelList
        self.avgDwellTime   = i_avgDwellList
        self.maxDwellTime   = i_maxDwellList

class ZoneMonitoringResult_t:
    def __init__(self, i_FrameCounter, i_numOfRes, i_SingleResList):
        self.FrameCounter       = i_FrameCounter
        self.numberOfResults    = i_numOfRes
        self.result             = i_SingleResList

class SingleZoneStatisticsResult_t:
    def __init__(self, i_zoneNumber, i_countList, i_fillLevelList, i_avgDwellList, i_maxDwellList):
        self.zoneNumber     = i_zoneNumber
        self.countList      = i_countList
        self.fillLevel      = i_fillLevelList
        self.avgDwellTime   = i_avgDwellList
        self.maxDwellTime   = i_maxDwellList

class ZoneStatisticsResult_t:
    def __init__(self, i_IntervalStartFrameCounter, i_IntervalStartTimeStamp, i_numOfRes, i_SingleZoneStatisticsResultList):
        self.IntervalStartFrameCounter = i_IntervalStartFrameCounter
        self.IntervalStartTimeStamp    = i_IntervalStartTimeStamp
        self.numberOfResults           = i_numOfRes
        self.result                    = i_SingleZoneStatisticsResultList

class SingleZoneObject_t:
    def __init__(self, i_zoneNumber, i_uniqueId, i_objClass, i_entryPos, i_exitPos, i_exitTime, i_dwellTime, i_height):
        self.zoneNumber     = i_zoneNumber
        self.uniqueId       = i_uniqueId
        self.objClass       = i_objClass
        self.entryPos       = i_entryPos
        self.exitPos        = i_exitPos
        self.exitTime       = i_exitTime
        self.dwellTime      = i_dwellTime
        self.height         = i_height

class ZoneObjectList_t:
    def __init__(self, i_IntervalStartFrameCounter, i_IntervalStartTimeStamp, i_numOfZoneObjects, i_SingleZoneObjectsList):
        self.IntervalStartFrameCounter  = i_IntervalStartFrameCounter
        self.IntervalStartTimeStamp     = i_IntervalStartTimeStamp
        self.numberOfObjects            = i_numOfZoneObjects
        self.objects                    = i_SingleZoneObjectsList

class ZoneEvent_t:
    def __init__(self, i_zoneNumber, i_eventType, i_uniqueId, i_objClass, i_timeStamp):
        self.zoneNumber  = i_zoneNumber
        self.eventType   = i_eventType
        self.uniqueId    = i_uniqueId
        self.objClass    = i_objClass
        self.timeStamp   = i_timeStamp
        
class CountEvent_t:
    def __init__(self, i_lineNumber, i_eventType, i_uniqueId, i_objClass, i_timeStamp):
        self.lineNumber  = i_lineNumber
        self.eventType   = i_eventType
        self.uniqueId    = i_uniqueId
        self.objClass    = i_objClass
        self.timeStamp   = i_timeStamp

def _bytesToUIntsGenerator(s):
    """
    iterates through a bytearray and generates integer values
    """
    numberOfInts = int(len(s) / 4)
    unpackFormat = "{0}I".format(numberOfInts)
    intTuple = struct.unpack(unpackFormat, s)

    idx = 0
    while(idx < numberOfInts):
        yield(intTuple[idx])
        idx += 1

def _bytesToUShortsGenerator(s):
    """
    iterates through a bytearray and generates integer values
    """
    numberOfShorts = int(len(s) / 2)
    unpackFormat = "{0}H".format(numberOfShorts)
    shortTuple = struct.unpack(unpackFormat, s)

    idx = 0
    while (idx < numberOfShorts):
        yield(shortTuple[idx])
        idx += 1

def _uintToBytes(val):
    return struct.pack("I", val)

def _ushortToBytes(val):
    return struct.pack("H", val)

def readPolyCntResMsg(payload):
    """
    Input is the payload of the message.
    Output is a class object with fields as defined above.
    See PolygonCounting_d_Message.h for struct definition.
    """

    it = _bytesToUIntsGenerator(payload)

    countResults = None
    numberOfResults = 0
    try:
        frameCounter    = next(it)
        countDataMaxCnt = len(objClasses)  # next(it)
        numberOfResults = next(it)

        resultList = []
        for _ in range(numberOfResults):
            lineNumber    = next(it)
            countDataCnt  = next(it)
            countDataList = []
            for _ in range(countDataMaxCnt):
                objClass = next(it)
                cntIn    = next(it)
                cntOut   = next(it)
                countData = CountData_t(objClass, cntIn, cntOut)
                countDataList.append(countData)
            countResult = PolygonCountingResult_t(lineNumber, countDataCnt, countDataList)
            resultList.append(countResult)
        countResults = PolygonCountingResults_t(frameCounter, countDataMaxCnt, numberOfResults, resultList)
    except StopIteration:
        logger.error("Number of count results in message (%d) doesn't fit to received message size (%d)", numberOfResults, len(payload))

    return countResults

def readCountResultsMsg(payload):
    (frameCounter, resultsCnt) = struct.unpack("II", payload[:8])
    resultsList = []
    countResults = None
    try:
        idx = 8
        structFormat = "HHII"  # ushort, ushort, uint, uint
        sizeOfStruct = struct.calcsize(structFormat)
        for _ in range(resultsCnt):
            (lineId, objClass, cntIn, cntOut) = struct.unpack(structFormat, payload[idx:idx+sizeOfStruct])
            countResult = CountResult_t(lineId, objClass, cntIn, cntOut)
            resultsList.append(countResult)
            idx += sizeOfStruct
        countResults = CountResultsMsg_t(frameCounter, resultsCnt, resultsList)
    except:
        logger.error("Number of count results in message (%d) doesn't fit to received message size (%d)", resultsCnt, len(payload))
        countResults = None

    return countResults

def createCountResultsMsgPayload(countResultsMsg):
    """
    countResultsMsg must be of type CountResultsMgs_t (see above)
    """
    framecounter = countResultsMsg.frameCounter
    resultsCnt = countResultsMsg.resultsCnt

    payload = b''
    payload += _uintToBytes(framecounter)
    payload += _uintToBytes(resultsCnt)

    for i in range(resultsCnt):
        result = countResultsMsg.results[i]
        lineId   = result.lineId
        objClass = result.objClass
        cntIn    = result.cntIn
        cntOut   = result.cntOut

        payload += _ushortToBytes(lineId)
        payload += _ushortToBytes(objClass)
        payload += _uintToBytes(cntIn)
        payload += _uintToBytes(cntOut)

    return payload

def readZoneMonitoringMsg(payload):
    numClasses = len(objClasses)  # change this variable if new classes are added to interface APSObjectClasses.h

    it = _bytesToUIntsGenerator(payload)

    zoneResults = None
    numberOfResults = 0
    try:
        frameCounter = next(it)
        numberOfResults = next(it)
        resultList = []
        for _ in range(numberOfResults):
            zoneNumber = next(it)

            fillLevelList = []
            for _ in range(numClasses):
                fillLevelList.append(next(it))
            for _ in range(numClasses, objClassesMax):
                next(it)

            avgDwellTimeList = []
            for _ in range(numClasses):
                avgDwellTimeList.append(next(it))
            for _ in range(numClasses, objClassesMax):
                next(it)

            maxDwellTimeList = []
            for _ in range(numClasses):
                maxDwellTimeList.append(next(it))
            for _ in range(numClasses, objClassesMax):
                next(it)

            szr = SingleZoneResult_t(zoneNumber, fillLevelList, avgDwellTimeList, maxDwellTimeList)
            resultList.append(szr)
        zoneResults = ZoneMonitoringResult_t(frameCounter, numberOfResults, resultList)
    except StopIteration:
        logger.error("Number of results (%d) in message doesn't fit to received message size (%d)", numberOfResults, len(payload))

    return zoneResults

def readZoneStatisticsMsg(payload):
    it = _bytesToUIntsGenerator(payload)

    numClasses = len(objClasses)
    zoneResults = None
    numberOfResults = 0
    try:
        intervalStartFrameCounter = next(it)
        intervalStartTimeStamp = next(it)
        numberOfResults = next(it)
        resultList = []
        for _ in range(numberOfResults):
            zoneNumber = next(it)

            countList = []
            for _ in range(numClasses):
                countList.append(next(it))

            fillLevelList = []
            for _ in range(numClasses):
                fillLevelList.append(next(it))

            avgDwellTimeList = []
            for _ in range(numClasses):
                avgDwellTimeList.append(next(it))

            maxDwellTimeList = []
            for _ in range(numClasses):
                maxDwellTimeList.append(next(it))

            szr = SingleZoneStatisticsResult_t(zoneNumber, countList, fillLevelList, avgDwellTimeList, maxDwellTimeList)
            resultList.append(szr)
        zoneResults = ZoneStatisticsResult_t(intervalStartFrameCounter, intervalStartTimeStamp, numberOfResults, resultList)
    except StopIteration:
        logger.error("Number of results (%d) in message doesn't fit to received message size (%d)", numberOfResults, len(payload))

    return zoneResults


def readZoneObjectListMsg(payload):
    """ Convert given binary data into a ZoneObjectList_t
    """

    zoneObjectList = None
    numberOfObjects = 0
    try:
        (intervalStartFrameCounter, intervalStartTimeStamp, numberOfObjects) = struct.unpack("III", payload[:12])

        objectList = []

        # Use the format below instead of "HIIiiiiIIH", because the data is not packed. The struct size must be 40 byte.
        structFormat = "IIIiiiiIII"
        sizeOfStruct = struct.calcsize(structFormat)

        # Debug
        # print("sizeOfStruct: {0}".format(sizeOfStruct))

        idx = 12
        for _ in range(numberOfObjects):
            (zoneNumber, uniqueId, objClass, entryPosX, entryPosY, exitPosX, exitPosY, exitTime, dwellTime, height) = struct.unpack(structFormat, payload[idx:idx+sizeOfStruct])
            entryPos       = [entryPosX, entryPosY]
            exitPos        = [exitPosX, exitPosY]

            # Debug
            # print("Obj: {0} {1} {2} {3} {4} {5} {6} {7} {8} {9}".format(zoneNumber, uniqueId, objClass, entryPosX, entryPosY, exitPosX, exitPosY, exitTime, dwellTime, height))

            szo = SingleZoneObject_t(zoneNumber, uniqueId, objClass, entryPos, exitPos, exitTime, dwellTime, height)
            objectList.append(szo)

            idx += sizeOfStruct

        zoneObjectList = ZoneObjectList_t(intervalStartFrameCounter, intervalStartTimeStamp, numberOfObjects, objectList)
    except StopIteration:
        logger.error("Number of object (%d) in message doesn't fit to received message size (%d)", numberOfObjects, len(payload))
        zoneObjectList = None

    return zoneObjectList

def readZoneEventMsg(payload):
    it = _bytesToUIntsGenerator(payload)

    zoneEventData = None
    try:
        zoneNumber = next(it)
        eventType = next(it)
        uniqueId = next(it)
        objClass = next(it)
        timeStamp = next(it)
        zoneEventData = ZoneEvent_t(zoneNumber, eventType, uniqueId, objClass, timeStamp)

    except StopIteration:
        logger.error("readZoneEventMsg: Date doesn't fit to received message size")
        zoneEventData = None
    return zoneEventData

def readCountEventMsg(payload):
    it = _bytesToUIntsGenerator(payload)

    countEventData = None
    try:
        lineNumber = next(it)
        eventType = next(it)
        uniqueId = next(it)
        objClass = next(it)
        timeStamp = next(it)
        countEventData = CountEvent_t(lineNumber, eventType, uniqueId, objClass, timeStamp)

    except StopIteration:
        logger.error("readCountEventMsg: Data does not fit to received message size")
        countEventData = None
    return countEventData

