import socket
import select

class CSocketData:
    """In C this would be a struct holding the information regarding a specific socket."""
    # pylint: disable=too-many-instance-attributes

    # Some constants:
    TypeListening       = 1
    TypeListeningClosed = 2
    TypeActive          = 3

    def __init__(self):
        """The "struct" members:"""
        self.Type = 0
        self.Socket = 0
        self.LocalPort = 0
        self.MaxConnections = 0
        self.CurrentConnections = 0
        # Only Valid for Type == TypeActive:
        self.ParentSocketData = None
        self.ClientAddress = 0
        self.ReceiveBuffer = b""
        self.SendBuffer = b""

class SocketServer:
    "This class is a base class for reading from and writing to multiple sockets."

    def __init__(self):
        self.SocketDict = {}
        self.__CloseCalled = False
        self.__CurrentSocketData = None
        self.__poll = None

    def __del__(self):
        for k, v in self.SocketDict.items():
            v.Socket.shutdown(socket.SHUT_RDWR)
            v.Socket.close()
        self.SocketDict = {}

    @staticmethod
    def __OpenListeningSocket(Port, MaxConnections):
        sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
        ServerAddress = ("", Port)  # empty string means ANY
        sock.bind(ServerAddress)
        sock.listen(MaxConnections)
        return sock

    def OpenTCPSocket(self, Port, MaxConnections):
        sd = CSocketData()

        sd.Type               = CSocketData.TypeListening
        sd.LocalPort          = Port
        sd.MaxConnections     = MaxConnections
        sd.CurrentConnections = 0
        sd.Socket             = self.__OpenListeningSocket(Port, MaxConnections)

        self.SocketDict[sd.Socket.fileno()] = sd

        return sd.Socket.fileno()

    def CloseListeningSocket(self, fd):
        pass    # TODO

    def __SendSendBuffer(self, SocketData):
        if not SocketData.SendBuffer:
            self.__poll.modify(SocketData.Socket.fileno(), select.POLLIN)
            return

        BytesSent = SocketData.Socket.send(SocketData.SendBuffer)
        if BytesSent == len(SocketData.SendBuffer):
            SocketData.SendBuffer = b""
            self.__poll.modify(SocketData.Socket.fileno(), select.POLLIN)
        else:
            SocketData.SendBuffer = SocketData.SendBuffer[:BytesSent]
            self.__poll.modify(SocketData.Socket.fileno(), select.POLLIN | select.POLLOUT)

    def Write(self, Text):
        if isinstance(Text, str):
            Text = Text.encode()
        elif isinstance(Text, bytes):
            pass
        else:
            print("Unconvertible parameter received")
        sd = self.__CurrentSocketData
        sd.SendBuffer += Text
        self.__SendSendBuffer(sd)

    def NewConnectionEstablished(self, SocketData):
        """Should be implemented by derived class"""

    def ProcessData(self, SocketData):
        """Should be implemented by derived class"""

    def Close(self):
        """Maybe called from ProcessData(). In this case the current
           connection (i.e. the one from which the data is processed) will be closed"""
        self.__CloseCalled = True

    def __CloseActiveConnection(self, fd):
        print("Connection %d closed" % (fd))
        self.__poll.unregister(fd)
        sd = self.SocketDict[fd]
        sd.Socket.shutdown(socket.SHUT_RDWR)
        sd.Socket.close()
        del self.SocketDict[fd]

        Listeningsd = sd.ParentSocketData
        Listeningsd.CurrentConnections -= 1
        if Listeningsd.Type == CSocketData.TypeListeningClosed:
            Listeningsd.Socket = self.__OpenListeningSocket(Listeningsd.LocalPort,
                    Listeningsd.MaxConnections)
            Listeningsd.Type = CSocketData.TypeListening
            self.__poll.register(Listeningsd.Socket.fileno(), select.POLLIN)

    def __HandleConnectionRequest(self, sd, fd):
        newsd = CSocketData()
        newsd.Type = sd.TypeActive
        newsd.LocalPort = sd.LocalPort
        newsd.ParentSocketData = sd
        newsd.Socket, newsd.ClientAddress = sd.Socket.accept()
        newsd.Socket.setblocking(0)
        self.SocketDict[newsd.Socket.fileno()] = newsd
        sd.CurrentConnections += 1
        if sd.CurrentConnections == sd.MaxConnections:
            sd.Socket.close()
            sd.Type = sd.TypeListeningClosed
            self.__poll.unregister(fd)

        self.__poll.register(newsd.Socket, select.POLLIN | select.POLLOUT)
        self.NewConnectionEstablished(newsd)

    def __HandleIncomingData(self, sd, fd):
        ReceivedData = sd.Socket.recv(4096)
        if not ReceivedData:
            self.__CloseActiveConnection(fd)
        else:
            sd.ReceiveBuffer += ReceivedData

            self.__CurrentSocketData = sd
            self.__CloseCalled = False
            self.ProcessData(sd)
            self.__CurrentSocketData = None
            if self.__CloseCalled:
                self.__CloseActiveConnection(fd)

    def mainloop(self):
        self.__poll = select.poll()
        for _, value in self.SocketDict.items():
            PollType = select.POLLIN
            if value.Type == CSocketData.TypeActive:
                PollType = PollType | select.POLLOUT
            self.__poll.register(value.Socket, PollType)

        print("entering mainloop ...")
        while True:
            print("SP while() loop")
            for fd, event in self.__poll.poll():
                sd = self.SocketDict[fd]
                if event & (select.POLLHUP | select.POLLERR | select.POLLNVAL):
                    # XXX is this error handling appropriate? Do we have to
                    # distinguish between listening and active sockets here?
                    sd.Socket.shutdown(socket.SHUT_RDWR)
                    sd.Socket.close()
                    self.__poll.unregister(fd)
                    del self.SocketDict[fd]
                    continue
                if event & select.POLLIN:
                    if sd.Type == CSocketData.TypeListening:
                        self.__HandleConnectionRequest(sd, fd)
                    elif sd.Type == CSocketData.TypeActive:
                        self.__HandleIncomingData(sd, fd)
                if event & select.POLLOUT:
                    print("in pollout")
                    self.__SendSendBuffer(sd)
