'''
Created on Mar 6, 2015

@author: tig
'''
import logging
import sys
if 'tornado.ioloop' in sys.modules:                 # tornado-webserver-context: without own loop
    TornadoContext = True
    from tornado.ioloop import IOLoop

    LOG_MESSAGE_COUNT = False                        # to debug: log average count of handled messages per second
    if LOG_MESSAGE_COUNT:
        from tornado.ioloop import PeriodicCallback
else:                                               # own loop context, threaded
    TornadoContext = False
    import threading
    import selectors
    LOG_MESSAGE_COUNT = False

import msgBrokerExt
from Tools.metaclass import Singleton

logger = logging.getLogger("PyMsgBroker")


class PyMsgBroker(threading.Thread if not TornadoContext else object, metaclass=Singleton):
    def __init__(self):
        logger.info("PyMsgBroker {}".format("running in Tornado context." if TornadoContext else "running with own threaded loop."))
        self.fd = -1
        self.callbacks = {}     # {(appID, msgID): callback, ...}

        if not TornadoContext:
            threading.Thread.__init__(self)
            self.lock = threading.RLock()
            self.sendLock = threading.RLock()
            self.setDaemon(True)

        if LOG_MESSAGE_COUNT:
            self.msgCount = {}
            PeriodicCallback(self.logMsgStatistics, 1000).start()

    # TODO: for debug if LOG_MESSAGE_COUNT
    def logMsgStatistics(self):
        delItems = []
        for i in self.msgCount:
            if self.msgCount[i] == 0:
                delItems.append(i)
                continue
            logger.info("{}: {}".format(i, self.msgCount[i]))
            self.msgCount[i] = 0
        for i in delItems:
            del self.msgCount[i]


    def connectToMsgBroker(self):
        logger.info("Connect to msgBroker ..")
        self.fd = msgBrokerExt.connect(PyMsgBroker.__MessageCB, None)
        if self.fd == -1:
            logger.error("Could not connect to msgBroker.")
            return False
        logger.debug("Connected. FD={0}" .format(self.fd))
        # Register MBBP callback.
        msgBrokerExt.mbbpRegisterCallback(PyMsgBroker.__MbbpCB)
        return True


    def disconnect(self):
        logger.debug("Disconnect from message broker.")
        if TornadoContext:
            if self.fd != -1:
                IOLoop.instance().remove_handler(self.fd)
                logger.info("Handler removed from IOLoop: {}, FD: {}".format(self.__class__.__name__, self.fd))
        msgBrokerExt.disconnect()
        self.fd = -1

    # msgIdList is expected as a list of lists containing two unsigned int values each (appID, msgID)
    # i.e. [[4, 2], [4, 6], [4, 1]]
    def subscribeToMsgIDs(self, msgIdList, msgCB):
        if self.fd == -1:
            logger.error("Could not subscribe to msgBroker. Not connected. (FD=-1)")
            return False

        msgIdListToSubscribe = []
        logger.debug("Trying to subscribe to msgIDs {0}" .format(msgIdList))

        if not TornadoContext:
            self.lock.acquire()
        for msgId in msgIdList:
            if not msgId in self.callbacks:
                logger.debug("Adding missing MsgId: {0}" .format(msgId))
                msgIdListToSubscribe.append(msgId)

        if len(msgIdListToSubscribe) > 0:
            tId = msgBrokerExt.subscribe(len(msgIdListToSubscribe), msgIdListToSubscribe)
            logger.debug("Subscribed following Ids to msgBrokerExt: {0}, callbackObjID: {1:X}"
                         .format(msgIdListToSubscribe, id(msgCB)))
            if (tId == 0):
                logger.warning("Subscribe to msgIDs failed.")
                if not TornadoContext:
                    self.lock.release()
                return False
        else:
            # key (appID, msgID) exists but maybe there is a new callback function to register
            pass

        # here it does not matter if the new list or the ones with possible duplicates because of the used set
        PyMsgBroker.__updateDispatcherDict(self, msgIdList, msgCB)

        if not TornadoContext:
            self.lock.release()

        return True

    def unsubscribeMsgIDs(self, msgIdList, msgCB):
        if self.fd == -1:
            logger.error("Failed to unsubscribe msgIDs. Not connected to MsgBroker. (FD=-1)")
            return False

        logger.debug("Trying to unsubscribe msgIDs {0}" .format(msgIdList))

        if not TornadoContext:
            self.lock.acquire()

        msgIdListToUnSubscribe = []
        for msgId in msgIdList:
            if not msgId in self.callbacks:
                logger.debug("Unsubscribe MsgId {0} is not required, since MsgId is not subscribed." .format(msgId))
                continue

            cbSet = self.callbacks[msgId]
            if msgCB not in cbSet:
                logger.debug("Unsubscribe MsgId {0} is not required, since registered callback doesn't match." .format(msgId))
                continue

            # for this message ID multiple callbacks are registered
            # just remove the callback from set
            if len(cbSet) > 1:
                logger.debug("Remove callback for MsgId {0}.".format(msgId))
                try:
                    cbSet.remove(msgCB)
                except:
                    pass
            # for this message ID msgCB is the only callback
            # remove msgID from CbDict and append to list to unsubscribe to msgBroker
            else:
                try:
                    del self.callbacks[msgId]
                except:
                    pass
                msgIdListToUnSubscribe.append(msgId)

        # unsubscribe to msgBrokerExt
        if len(msgIdListToUnSubscribe) > 0:
            tId = msgBrokerExt.unsubscribe(len(msgIdListToUnSubscribe), msgIdListToUnSubscribe)
            if (tId == 0):
                logger.warning("Failed to unsubscribe MsgIDs to msgBrokerExt: {0}".format(msgIdListToUnSubscribe))
                if not TornadoContext:
                    self.lock.release()
                return False
            logger.debug("Unsubscribed MsgIDs to msgBrokerExt: {0}, callbackObjID: {1:X}"
                         .format(msgIdListToUnSubscribe, id(msgCB)))

        if not TornadoContext:
            self.lock.release()

        return True

    # used in TornadoContext
    def handleMessage(self, fd, events):
        msgBrokerExt.handleIncomingData()


    def startMsgListener(self):
        if self.fd == -1:
            logger.warning("Cannot start the message listener since the message broker is not yet connected.")
            return False

        if TornadoContext:
            IOLoop.instance().add_handler(self.fd, self.handleMessage, IOLoop.READ)
            logger.info("Handler added to IOLoop: {}, FD: {}".format(self.__class__.__name__, self.fd))
            return True
        else:
            if self.is_alive():
                logger.warning("The message listener is already running.")
                return False
            else:
                logger.info("Start the message listener.")
                self.start()    # starts the thread with run() method
                return True


    # payload object is expected as a bytes object
    # for example it can be created like: countPayload = struct.pack("II", countIn, countOut)
    def sendMessage(self, appID, msgID, payload=None, transactionID=0):
        if not TornadoContext:
            self.lock.acquire()
        res = -1
        if self.fd == -1:
            logger.warning("Could not send message since the message broker is not yet connected.")
        else:
            res = msgBrokerExt.sendMessage(appID, msgID, transactionID, payload)
        if not TornadoContext:
            self.lock.release()
        return res


    # Callback method - called by C-Extension Callback
    @staticmethod
    def __MessageCB(appID, msgID, i_Data):
        # call MessageCB from dict
        PyMsgBroker().__callCorrespondingMsgCBs(appID, msgID, i_Data)

        # TODO: for debug if LOG_MESSAGE_COUNT
        if LOG_MESSAGE_COUNT:
            ID = "{:2}, {:2}".format(appID, msgID)
            msgCount = PyMsgBroker().msgCount
            msgCount[ID] = msgCount[ID] + 1 if ID in msgCount else 0



    # Callback method for MBBP (shared memory) - called by C-Extension Callback
    @staticmethod
    def __MbbpCB(appID, msgID, i_Data):
        # call callback from dict
        PyMsgBroker().__callCorrespondingMsgCBs(appID, msgID, i_Data)

        # TODO: for debug if LOG_MESSAGE_COUNT
        if LOG_MESSAGE_COUNT:
            ID = "{:2}, {:2}".format(appID, msgID)
            msgCount = PyMsgBroker().msgCount
            msgCount[ID] = msgCount[ID] + 1 if ID in msgCount else 0



    def __callCorrespondingMsgCBs(self, appID, msgID, i_Data):
        # Find out which cbFuncs to call over appID, msgID and call with args(appID, msgID, i_Data)
        if not TornadoContext:
            self.lock.acquire()
        for elem in self.callbacks:
            if (appID == elem[0] and msgID == elem[1]):
                logger.debug("Calls for MsgId: {0} " .format(elem))
                logger.debug("MsgID: {0}  Functionset: {1} " .format(elem, self.callbacks[elem]))
                # Call all cb functions which are subscribed
                for cbFunc in self.callbacks[elem]:
                    logger.debug("cbFunc: {0} " .format(cbFunc))
                    cbFunc(appID, msgID, i_Data)
        if not TornadoContext:
            self.lock.release()


    # Dispatcher Dict for holding all the message broker contracts
    def __updateDispatcherDict(self, msgIdList, msgCb):
        temp_set = set()
        logger.debug("MsgIds for dict update:")
        for idx, elem in enumerate(msgIdList):
            logger.debug("{0}: {1}" .format(idx+1, elem))
            if self.callbacks.get(elem) is None:
                self.callbacks.update({elem: set({msgCb})})
            else:
                temp_set |= self.callbacks[elem]  # cb - set of cbs
                temp_set.update({msgCb})
                self.callbacks.update({elem: temp_set})
        logger.debug("Latest Callback Function Dict: \n {0} \n"  .format(self.callbacks))



    # ----------- Worker (Listener method) --------------------------------------------------
    # used in NOT TornadoContext: own loop in thread
    def run(self):

        # create selector and register Fd
        logger.debug("Create selector for polling.")
        selector = selectors.SelectSelector()
        selector.register(self.fd, selectors.EVENT_READ)

        # ###### running event loop #########################################

        while True:
            # TODO: analyze key?
            selector.select()
            msgBrokerExt.handleIncomingData()

        self.disconnect()
