#!/bin/sh

######################################################################
# Script executes a modul test for the multiline counting (MLC) daemon.
# It loads ground truth data (GTD) files:
# - tracked object list file as input for the MLC
# - XML parameter file for the MLC
# - a GTD file with counting results
#
# argument 1: Test case directory (with ending "/")
#
# Exit codes are:
#  0: test case passed
#  1: test case failed
# 10: number of arguments is invalid
# 11: given test case directory not found
# 12: one ore more file(s) in test case directory missing
# 13: one ore more test relevant application(s) missing 
# 14: ObjectListPlayer returns an error
#
#
# TODOs:
# - TODOs in script file
# - remove echo's in script
# - remove printf's in test applications
# - currently object list player prozess could started 31 times
#   (a "bug" in MBBP MsgBroker / library)
######################################################################



### Defines
# Defines: return values
EXIT_TC_PASSED=0
EXIT_TC_FAILED=1
EXIT_ERR_NUM_ARGS=10
EXIT_ERR_TC_DIR_NOT_EXIST=11
EXIT_ERR_TC_FILES_MISSING=12
EXIT_ERR_TEST_APP_MISSING=13
EXIT_ERR_OBJLISTPLAYER_ERR=14
EXIT_ERR_LICFILES_MISSING=15
# Defines: test relevant application paths
BIN_NAME_TRACKEDOBJLIST_D="TrackedObjList_d"
BIN_NAME_MULTILINECOUNTING_D="MultilineCounting_d"
TT_BIN_NAME_OBJ_LIST_PLAYER="ObjectListPlayer"
TT_BIN_NAME_MLC_PARAMCONVERTER="MLC_ParamConverter"
TT_BIN_NAME_CNT_RES_COMP="MLC_TargetActualComparision"
TT_APC_SEND_MSG="/usr/local/bin/APC_SendMsg"
# Defines: files from test case directory
GTD_FILE_OBJLIST="TrackedObjectList.tol"
GTD_FILE_CNTRES="GTCountResult.csv"
PARAM_FILE_MLC="countingpolygons_values.xml"
# Defines: temporary directory and files
TMP_DIR_ROOT="/tmp/tmp_MLC_modtest/"
TMP_FILE_CNTRES_FROM_MLC="mlc_cntres.csv"
# License Files
LICENSE_DIR="/var/Counter/licenses/"
OC_LICENSEFILE="ObjectClassification.lic"
INVALID_OC_LICENSEFILE="ObjectClassification.tmp"

#### Check arguments
# Number of arguments
if [ $# -ne 2 ] ; then
    echo "invalid arguments"
    exit ${EXIT_ERR_NUM_ARGS}
fi
# Check test env directory
PATH_TO_TT=$1
echo "PATH_TO_TT: ${PATH_TO_TT}"
if [ ! -d ${PATH_TO_TT} ] ; then
    echo "PATH_TO_TT is not a directory"
    exit ${EXIT_ERR_TC_DIR_NOT_EXIST}
fi
# Check test case directory
TC_DIR=$2
echo "TC_DIR: ${TC_DIR}"
if [ ! -d ${TC_DIR} ] ; then
    echo "TC_DIR is not a directory"
    exit ${EXIT_ERR_TC_DIR_NOT_EXIST}
fi
# Check if TC files exist and TC relevant files
if [ ! -f "${TC_DIR}${GTD_FILE_OBJLIST}" ] ; then
    echo "GTD file not found: ${TC_DIR}${GTD_FILE_OBJLIST}"
    exit ${EXIT_ERR_TC_FILES_MISSING}
fi
if [ ! -f "${TC_DIR}${GTD_FILE_CNTRES}" ] ; then
    echo "GTD file not found: ${TC_DIR}${GTD_FILE_CNTRES}"
    exit ${EXIT_ERR_TC_FILES_MISSING}
fi
if [ ! -f "${TC_DIR}${PARAM_FILE_MLC}" ] ; then
    echo "Parameter file not found: ${TC_DIR}${PARAM_FILE_MLC}"
    exit ${EXIT_ERR_TC_FILES_MISSING}
fi
# Check if all required binaries are available
if [ ! -x "${PATH_TO_TT}${TT_BIN_NAME_OBJ_LIST_PLAYER}" ] ; then
    echo "Test relevant app not found: ${PATH_TO_TT}${TT_BIN_NAME_OBJ_LIST_PLAYER}"
    exit ${EXIT_ERR_TEST_APP_MISSING}
fi
if [ ! -x "${PATH_TO_TT}${TT_BIN_NAME_CNT_RES_COMP}" ] ; then
    echo "Test relevant app not found: ${PATH_TO_TT}${TT_BIN_NAME_CNT_RES_COMP}"
    exit ${EXIT_ERR_TEST_APP_MISSING}
fi
if [ ! -x "${TT_APC_SEND_MSG}" ] ; then
    echo "Test relevant app not found: ${TT_APC_SEND_MSG}"
    exit ${EXIT_ERR_TEST_APP_MISSING}
fi

# Check if license files exist
if [ ! -f "${LICENSE_DIR}${OC_LICENSEFILE}" ] ; then
    echo "ObjClassification license file missing"
    exit ${EXIT_ERR_LICFILES_MISSING}
fi



### Prepare the system
# Create or clean a tmp directory
if [ ! -d "${TMP_DIR_ROOT}" ] ; then
    mkdir -p "${TMP_DIR_ROOT}"
    echo "directory created: ${TMP_DIR_ROOT}"
else
    rm -rf "${TMP_DIR_ROOT}"*
    echo "directory cleaned: ${TMP_DIR_ROOT}"
fi
# Save original MLC XML parameter file
cp -f /var/Counter/config/${PARAM_FILE_MLC} /tmp/
# Convert multiline counting parameter and overwrite original parameter file
${PATH_TO_TT}${TT_BIN_NAME_MLC_PARAMCONVERTER} "${TC_DIR}${PARAM_FILE_MLC}" /var/Counter/config/${PARAM_FILE_MLC}

# Rename licence files that are not needed for the test case
if [ ! -f "${TC_DIR}${OC_LICENSEFILE}" ] ; then
    mv "${LICENSE_DIR}${OC_LICENSEFILE}" "${LICENSE_DIR}${INVALID_OC_LICENSEFILE}"
fi

# kill some tools
/usr/bin/killall -6 ${TT_BIN_NAME_CNT_RES_COMP} 2>/dev/null
/usr/bin/killall -6 ${TT_BIN_NAME_OBJ_LIST_PLAYER} 2>/dev/null
# kill TrackedObjList_d + MultilineCounting_d process
/usr/bin/killall -6 ${BIN_NAME_TRACKEDOBJLIST_D} 2>/dev/null
/usr/bin/killall -6 ${BIN_NAME_MULTILINECOUNTING_D} 2>/dev/null
sleep 1 #sleep a little while ("manual" sync, a TODO)



# Restart MultilineCounting_d
/sbin/MultilineCounting_d
sleep 1 #sleep a little while ("manual" sync, a TODO)



### Start tool to compare GTD counting results versus calculated counting results in background
# TODO currently tool records the received counting results to a file in same format like the GTD data
${PATH_TO_TT}${TT_BIN_NAME_CNT_RES_COMP} ${TMP_DIR_ROOT}${TMP_FILE_CNTRES_FROM_MLC} &
sleep 1 #sleep a little while ("manual" sync, a TODO)



### Start ObjectListPlayer tool with GTD
${PATH_TO_TT}${TT_BIN_NAME_OBJ_LIST_PLAYER} -i "${TC_DIR}${GTD_FILE_OBJLIST}"
RES=$?
if [ ${RES} -eq 0 ] ; then
    echo "tracked object list ground truth data successfully send"
else
    echo "ObjectListPlayer tool returned an error (exit code: ${RES})"
    exit ${EXIT_ERR_OBJLISTPLAYER_ERR}
fi
sleep 1 #sleep a little while ("manual" sync, a TODO)



### Signalize test end to count result recorder tool (TODO signal ?)
${TT_APC_SEND_MSG} 20 1 0
sleep 1 #sleep a little while ("manual" sync, a TODO)



### Clean-up some things
# TODO further clean up, ...
# Restore original MLC XML parameter file
cp -f /tmp/${PARAM_FILE_MLC} /var/Counter/config/

# Restore license files
if [ ! -f "${LICENSE_DIR}${OC_LICENSEFILE}" ] ; then
    mv "${LICENSE_DIR}${INVALID_OC_LICENSEFILE}" "${LICENSE_DIR}${OC_LICENSEFILE}"
fi

### Test passed or failed (return value)
# TODO (currently a file diff is used)
# TODO store test results ?
# TODO generate a test report ?
/usr/bin/diff -s ${TMP_DIR_ROOT}${TMP_FILE_CNTRES_FROM_MLC} ${TC_DIR}${GTD_FILE_CNTRES} 1>/dev/null 2>/dev/null
RES=$?
if [ ${RES} -eq 0 ] ; then
    echo "test passed"
    exit ${EXIT_TC_PASSED}
else
    echo "test failed"
    exit ${EXIT_TC_FAILED}
fi


