#!/bin/sh

######################################################################
# Script executes a modul test for the zone monitoring (ZM) daemon.
# It loads ground truth data (GTD) files:
# - tracked object list file as input for the MLC
# - XML parameter file for the MLC
# - a GTD file with results
#
# argument 1: Test case directory (with ending "/")
#
# Exit codes are:
#  0: test case passed
#  1: test case failed
# 10: number of arguments is invalid
# 11: given test case directory not found
# 12: one ore more file(s) in test case directory missing
# 13: one ore more test relevant application(s) missing 
# 14: ObjectListPlayer returns an error
#
#
# TODOs:
# - TODOs in script file
# - remove echo's in script
# - remove printf's in test applications
# - currently object list player prozess could started 31 times
#   (a "bug" in MBBP MsgBroker / library)
######################################################################



### Defines
# Defines: return values
EXIT_TC_PASSED=0
EXIT_TC_FAILED=1
EXIT_ERR_NUM_ARGS=10
EXIT_ERR_TC_DIR_NOT_EXIST=11
EXIT_ERR_TC_FILES_MISSING=12
EXIT_ERR_TEST_APP_MISSING=13
EXIT_ERR_OBJLISTPLAYER_ERR=14
EXIT_ERR_LICFILES_MISSING=15
# Defines: test relevant application paths
BIN_NAME_TRACKEDOBJLIST_D="TrackedObjList_d"
BIN_NAME_ZONEMONITORING_D="ZoneMonitoring_d"
TT_BIN_NAME_OBJ_LIST_PLAYER="ObjectListPlayer"
TT_BIN_NAME_RES_COMP="ZM_TargetActualComparision"
TT_APC_SEND_MSG="/usr/local/bin/APC_SendMsg"
# Defines: files from test case directory
GTD_FILE_OBJLIST="TrackedObjectList.tol"
GTD_FILE_RES="ZMResult.csv"
GTD_FILE_SNAP="SnapShotData.csv"
GTD_FILE_STAT="ZoneStatisticsData.csv"
GTD_FILE_ZOL="ZoneObjList.csv"
GTD_FILE_ZML="ZoneMonitoring.lic"
GTD_FILE_OCL="ObjectClassification.lic"
PARAM_FILE_ZM="ip_zonemonitoring_values.xml"
# Defines: temporary directory and files
TMP_DIR_ROOT="/tmp/tmp_ZM_modtest/"
TMP_FILE_RES_FROM_ZM="zm_cntres.csv"
TMP_FILE_SNAP_LOG="zm_snap.csv"
TMP_FILE_STATISTICS="zm_statistics.csv"
TMP_FILE_ZOL="zm_objlist.csv"
LICENSE_DIR="/var/Counter/licenses/"
ZM_LICENSEFILE="ZoneMonitoring.lic"
OC_LICENSEFILE="ObjectClassification.lic"
INVALID_ZM_LICENSEFILE="ZoneMonitoring.tmp"
INVALID_OC_LICENSEFILE="ObjectClassification.tmp"


#### Check arguments
# Number of arguments
if [ $# -ne 2 ] ; then
    echo "invalid arguments"
    exit ${EXIT_ERR_NUM_ARGS}
fi
# Check test env directory
PATH_TO_TT=$1
echo "PATH_TO_TT: ${PATH_TO_TT}"
if [ ! -d ${PATH_TO_TT} ] ; then
    echo "PATH_TO_TT is not a directory"
    exit ${EXIT_ERR_TC_DIR_NOT_EXIST}
fi
# Check test case directory
TC_DIR=$2
echo "TC_DIR: ${TC_DIR}"
if [ ! -d ${TC_DIR} ] ; then
    echo "TC_DIR is not a directory"
    exit ${EXIT_ERR_TC_DIR_NOT_EXIST}
fi
# Check if TC files exist and TC relevant files
if [ ! -f "${TC_DIR}${GTD_FILE_OBJLIST}" ] ; then
    echo "GTD file not found: ${TC_DIR}${GTD_FILE_OBJLIST}"
    exit ${EXIT_ERR_TC_FILES_MISSING}
fi
if [ ! -f "${TC_DIR}${GTD_FILE_RES}" ] ; then
    echo "GTD file not found: ${TC_DIR}${GTD_FILE_RES}"
    exit ${EXIT_ERR_TC_FILES_MISSING}
fi

# Check if all required binaries are available
if [ ! -x "${PATH_TO_TT}${TT_BIN_NAME_OBJ_LIST_PLAYER}" ] ; then
    echo "Test relevant app not found: ${PATH_TO_TT}${TT_BIN_NAME_OBJ_LIST_PLAYER}"
    exit ${EXIT_ERR_TEST_APP_MISSING}
fi
if [ ! -x "${PATH_TO_TT}${TT_BIN_NAME_RES_COMP}" ] ; then
    echo "Test relevant app not found: ${PATH_TO_TT}${TT_BIN_NAME_RES_COMP}"
    exit ${EXIT_ERR_TEST_APP_MISSING}
fi
if [ ! -x "${TT_APC_SEND_MSG}" ] ; then
    echo "Test relevant app not found: ${TT_APC_SEND_MSG}"
    exit ${EXIT_ERR_TEST_APP_MISSING}
fi

# Check if license files exist
if [ ! -f "${LICENSE_DIR}${ZM_LICENSEFILE}" ] ; then
    echo "ZoneMonitoring license file missing"
    exit ${EXIT_ERR_LICFILES_MISSING}
fi
if [ ! -f "${LICENSE_DIR}${OC_LICENSEFILE}" ] ; then
    echo "ObjClassification license file missing"
    exit ${EXIT_ERR_LICFILES_MISSING}
fi


### Prepare the system
# Create or clean a tmp directory
if [ ! -d "${TMP_DIR_ROOT}" ] ; then
    mkdir -p "${TMP_DIR_ROOT}"
    echo "directory created: ${TMP_DIR_ROOT}"
else
    rm -rf "${TMP_DIR_ROOT}"*
    echo "directory cleaned: ${TMP_DIR_ROOT}"
fi
# Save original ZM XML parameter file
cp -f /var/Counter/config/${PARAM_FILE_ZM} /tmp/

# Copy TC dependend ZM XML parameter file
cp "${TC_DIR}${PARAM_FILE_ZM}" /var/Counter/config/

# Rename licence files that are not needed for the test case
if [ ! -f "${TC_DIR}${ZM_LICENSEFILE}" ] ; then
    mv "${LICENSE_DIR}${ZM_LICENSEFILE}" "${LICENSE_DIR}${INVALID_ZM_LICENSEFILE}"
fi
if [ ! -f "${TC_DIR}${OC_LICENSEFILE}" ] ; then
    mv "${LICENSE_DIR}${OC_LICENSEFILE}" "${LICENSE_DIR}${INVALID_OC_LICENSEFILE}"
fi

STAT_TMPFILE="no_stat"
WAIT_OPT=""  # Option: Wait for statistic msg before starting ObjectListPlayer
if [ -f "${TC_DIR}${GTD_FILE_STAT}" ] ; then
    STAT_TMPFILE=${TMP_DIR_ROOT}${TMP_FILE_STATISTICS}
    WAIT_OPT="-wait"
fi

ZOL_TMPFILE="no_zol"
if [ -f "${TC_DIR}${GTD_FILE_ZOL}" ] ; then
    ZOL_TMPFILE=${TMP_DIR_ROOT}${TMP_FILE_ZOL}
    WAIT_OPT="-wait"
fi

# kill some tools
/usr/bin/killall -6 ${TT_BIN_NAME_RES_COMP} 2>/dev/null
/usr/bin/killall -6 ${TT_BIN_NAME_OBJ_LIST_PLAYER} 2>/dev/null
# kill TrackedObjList_d process + ZoneMonitoring
/usr/bin/killall -6 ${BIN_NAME_TRACKEDOBJLIST_D} 2>/dev/null
/usr/bin/killall -6 ${BIN_NAME_ZONEMONITORING_D} 2>/dev/null
sleep 1 #sleep a little while ("manual" sync, a TODO)

# Restart ZoneMonitoring
/sbin/ZoneMonitoring_d


### Re-initialize the ZoneMonitoring_d
#XML_FILE_ID_ZM_PARAMS="161"
#${TT_APC_SEND_MSG} 16 1 0 1 ${XML_FILE_ID_ZM_PARAMS}
sleep 1 #sleep a little while ("manual" sync, a TODO)



### Start tool to compare GTD ZoneMonitoring results versus calculated ZoneMonitoring results in background
# TODO currently tool records the received ZoneMonitoring results to a file in same format like the GTD data
${PATH_TO_TT}${TT_BIN_NAME_RES_COMP} ${TMP_DIR_ROOT}${TMP_FILE_RES_FROM_ZM} ${TC_DIR}${GTD_FILE_SNAP} ${TMP_DIR_ROOT}${TMP_FILE_SNAP_LOG} ${STAT_TMPFILE} ${ZOL_TMPFILE} &
sleep 1 #sleep a little while ("manual" sync, a TODO)



### Start ObjectListPlayer tool with GTD
${PATH_TO_TT}${TT_BIN_NAME_OBJ_LIST_PLAYER} -i ${WAIT_OPT} "${TC_DIR}${GTD_FILE_OBJLIST}"
RES=$?
if [ ${RES} -eq 0 ] ; then
    echo "tracked object list ground truth data successfully send"
else
    echo "ObjectListPlayer tool returned an error (exit code: ${RES})"
    exit ${EXIT_ERR_OBJLISTPLAYER_ERR}
fi
sleep 1 #sleep a little while ("manual" sync, a TODO)



### Signalize test end to ZoneMonitoring result recorder tool (TODO signal ?)
${TT_APC_SEND_MSG} 33 1 0
sleep 1 #sleep a little while ("manual" sync, a TODO)



### Clean-up some things
# TODO further clean up, ...
# Restore original MLC XML parameter file
cp -f /tmp/${PARAM_FILE_ZM} /var/Counter/config/

# Restore license files
if [ ! -f "${LICENSE_DIR}${ZM_LICENSEFILE}" ] ; then
    mv "${LICENSE_DIR}${INVALID_ZM_LICENSEFILE}" "${LICENSE_DIR}${ZM_LICENSEFILE}"
fi
if [ ! -f "${LICENSE_DIR}${OC_LICENSEFILE}" ] ; then
    mv "${LICENSE_DIR}${INVALID_OC_LICENSEFILE}" "${LICENSE_DIR}${OC_LICENSEFILE}"
fi


# Copy actual data for debugging
rm -rf ${TC_DIR}/zm_*
cp ${TMP_DIR_ROOT}/zm_* ${TC_DIR}

### Test passed or failed (return value)

### Compare Zone Statistics Data

if [ -f "${TC_DIR}${GTD_FILE_STAT}" ] ; then

    if [ ! -f "${TMP_DIR_ROOT}${TMP_FILE_STATISTICS}" ] ; then
        echo "Error: Zone Statistics result file was not written"
        echo "test failed"
        exit ${EXIT_TC_FAILED}
    fi
    ### Compare files
    # /usr/bin/diff -s ${TC_DIR}${GTD_FILE_STAT} ${TMP_DIR_ROOT}${TMP_FILE_STATISTICS} 1>/dev/null 2>/dev/null
    /usr/local/verification/bin/diffregexp.sh ${TMP_DIR_ROOT}${TMP_FILE_STATISTICS} ${TC_DIR}${GTD_FILE_STAT} 1>/dev/null 2>/dev/null
    RES=$?
    if [ ${RES} -ne 0 ] ; then
        echo "Statistics files do not match"
        echo "test failed"
        exit ${EXIT_TC_FAILED}
    fi
    echo "Zone statistics test passed"
fi

### Compare Zone Object List Data

if [ -f "${TC_DIR}${GTD_FILE_ZOL}" ] ; then

    if [ ! -f "${TMP_DIR_ROOT}${TMP_FILE_ZOL}" ] ; then
        echo "Error: Zone object list result file was not written"
        echo "test failed"
        exit ${EXIT_TC_FAILED}
    fi
    
    ### Compare files
    /usr/bin/diff -s ${TC_DIR}${GTD_FILE_ZOL} ${TMP_DIR_ROOT}${TMP_FILE_ZOL} 1>/dev/null 2>/dev/null
    RES=$?
    if [ ${RES} -ne 0 ] ; then
        echo "Zone object list files do not match"
        echo "test failed"
        exit ${EXIT_TC_FAILED}
    fi
    echo "Zone object list test passed"
fi

### Compare snapshot test data

if [ -f "${TC_DIR}${GTD_FILE_SNAP}" ] ; then
    if [ ! -f "${TMP_DIR_ROOT}${TMP_FILE_SNAP_LOG}" ] ; then
        echo "Error: SnapShot result file was not written"
        echo "test failed"
        exit ${EXIT_TC_FAILED}
    fi
    ### Compare files
    /usr/bin/diff -s ${TC_DIR}${GTD_FILE_SNAP} ${TMP_DIR_ROOT}${TMP_FILE_SNAP_LOG} 1>/dev/null 2>/dev/null
    RES=$?
    if [ ${RES} -ne 0 ] ; then
        echo "SnapShot files differ"
        echo "test failed"
        exit ${EXIT_TC_FAILED}
    fi

    echo "Snapshot test passed"
fi

if [ -f "${TC_DIR}${GTD_FILE_ZOL}" ] || [ -f "${TC_DIR}${GTD_FILE_SNAP}" ]; then
    exit ${EXIT_TC_PASSED}
fi

### Compare fill level results
/usr/bin/diff -s ${TMP_DIR_ROOT}${TMP_FILE_RES_FROM_ZM} ${TC_DIR}${GTD_FILE_RES} 1>/dev/null 2>/dev/null
RES=$?
if [ ${RES} -eq 0 ] ; then
    echo "test passed"
    exit ${EXIT_TC_PASSED}
else
    echo "test failed"
    exit ${EXIT_TC_FAILED}
fi


