#!/usr/bin/env /usr/local/bin/python3
# -*- coding: utf-8 -*-
##########################################################################
#
#    name:           paramTool.py
#    author:         Andrej Wunrau@hella.com (awu) mod by jeg
#    description:    Gets or sets single parameter file values. 
#
##########################################################################

import os
import sys
import optparse
import json

# set correct import paths and import needed modules
# A second path to folder "server" is inserted. The project APS-M does not contain a "lib" folder.
# TODO: Adapt the webserver libraries for projekt APS-M . 
sys.path.append(os.path.join(os.path.dirname(__file__), '/opt/WebServices/server/'))
sys.path.append(os.path.join(os.path.dirname(__file__), '/opt/WebServices/server/lib/'))
from ParamHandling.parameters import Parameters
from msgBroker.PyMsgBroker import PyMsgBroker
msgBroker   = PyMsgBroker()
parameters  = Parameters()


class ParamData:
    # holds all information concerning the parameter read/write operation
    setName         = ""
    groupPath       = ""
    valueName       = ""
    valueType       = ""
    newValue        = -1
    setID           = -1


class ParamHandler:
    # executes the operation

    def __init__(self):
        msgBroker.connectToMsgBroker()
        self.beVerbose       = False
        self.doWrite         = False
        self.paramData       = ParamData()
        self.paramDict       = None
        self.useJson         = False

    def parse_args(self):
        """
            In json mode (-j) and write mode, a json-dict representation is expected as Parameter-Argument (can be recursive)
            In json mode (-j) and read mode, a json-dict representation is returned (containing recursive parameters)
        """
        parser = optparse.OptionParser("paramTool.py [Options] ParameterSet ParameterGroup Parameter Type/NewValue") 
        parser.add_option("-m","--mode",type="choice",choices=["r","w"], dest="mode",help="MODE w = write or r = read",default="r")
        parser.add_option("-v", "--verbose", action="store_true",dest="verbose", default=False,help="switch verbose mode")
        parser.add_option("-j", "--json", action="store_true", dest="json", default=False, help="write-mode: Parameter argument expected as json, read-mode: returns parameters as json")
        (opts,args) = parser.parse_args()
        
        self.doWrite = True if (opts.mode=="w") else False
        self.useJson = opts.json

        # use json as parameter (write mode) or return json (read mode)
        if self.useJson:
            # check number of arguments
            if (self.doWrite and len(args) != 3 ):
                print("in json-write-mode, exactly 3 arguments expected")
                sys.exit(1)
            elif (not self.doWrite and len(args) != 2):
                print("in json-read-mode, exactly 2 arguments expected")
                sys.exit(1)

            # load from json
            if self.doWrite:
                try:
                    self.paramDict = json.loads(args[2])
                except Exception as e:
                    print("failed to load json-parameters from {0}, error: {1}".format(args[2], e))
                    sys.exit(1)
        
        # single argument
        elif len(args) != 4:
            print("in single parameter mode we need exactly 4 args")
            sys.exit(1)

        self.beVerbose = opts.verbose
        self.paramData.setName   = args[0]
        self.paramData.groupPath = args[1]

        if (not self.useJson):
            self.paramData.valueName    = args[2]
            if self.doWrite:
                self.paramData.newValue = args[3]
            else:
                self.paramData.valueType = args[3]
        
       
    def execute_operation(self):
        # try to find parmeter
        for setID in Parameters.paramSets:
            #print ("id: ",setID, "name: ", Parameters.paramSets[setID]['name'])
            if (Parameters.paramSets[setID]['name'] == self.paramData.setName):
                self.paramData.setID = setID
                #print ("yeah, id:", setID)
        
        # parameter found
        if (self.paramData.setID >= 0):
            
            # write parameters
            if self.doWrite:
                # write a single parameter
                if (not self.useJson):
                    try:
                        # set parameter value
                        data = {}
                        data[self.paramData.valueName] = {}
                        data[self.paramData.valueName]["value"] = self.paramData.newValue
                        if (False == parameters.set( self.paramData.setID,
                                                     self.paramData.groupPath,
                                                     data,
                                                     paramNamesSet={self.paramData.valueName},
                                                     recursive=False)):
                            print("error, failed to write parameter {0} {1} {2}"
                                  .format(self.paramData.setName, self.paramData.groupPath, self.paramData.valueName))
                            sys.exit(1)
                    except Exception as e:
                        print("error, failed to write parameter {0} {1} {2}, error: {3}"
                              .format(self.paramData.setName, self.paramData.groupPath, self.paramData.valueName, e))
                        sys.exit(1)
                
                # write a json parameter dict
                else:
                    try:
                        if (False == parameters.set(self.paramData.setID, self.paramData.groupPath, self.paramDict, recursive=True)):
                            print("error, failed to write json-parameter-dict {0} {1}".format(self.paramData.setName, self.paramData.groupPath))
                            sys.exit(1)
                    except Exception as e:
                        print("error, failed to write json-parameter-dict {0} {1}, error: {2}".format(self.paramData.setName, self.paramData.groupPath, e))
                        sys.exit(1)
            
            # read parameters
            else:
                # read a single parameter
                if (not self.useJson):
                    try:
                        data = parameters.get( self.paramData.setID,
                                               self.paramData.groupPath,
                                               paramNamesSet={self.paramData.valueName},
                                               recursive=False)
                        print(data[self.paramData.valueName][self.paramData.valueType])   # write return value to stdout
                    except Exception as e:
                        print("failed to get parameter {0} {1} {2}, error: {3}"
                              .format(self.paramData.setName, self.paramData.groupPath, self.paramData.valueName, e))
                        sys.exit(1)
                    
                # read a json parameter dict
                else:
                    try:
                        dataDict = parameters.get( self.paramData.setID,
                                                   self.paramData.groupPath,
                                                   recursive=True,
                                                   metaFilter=Parameters.META_FILTER_VALUES_ONLY)
                        
                        try:
                            jsonStr = json.dumps(dataDict)
                        except Exception as e:
                            print("Failed to create json from parameter dict {0}, error: {1}".format(dataDict, e))
                            sys.exit(1)
                        print(jsonStr)    # write json dict to stdout    

                    except Exception as e:
                        print("failed to get parameter-dict {0} {1}, error: {2}".format(self.paramData.setName, self.paramData.groupPath, e))
                        sys.exit(1)
            
        else:
            print("error, no parameter set id for parameter file name \"",self.paramData.setName , "\" found")
            sys.exit(1)          


def main (argv):
    # create handler object
    handler = ParamHandler()
    # get script call arguments and execute the corresponding operations
    handler.parse_args()
    handler.execute_operation()
    
if __name__ == "__main__":
    main(sys.argv)
